/* about.cpp - About window
 * 
 *  Bronco File Manager
 * 
 *  Copyright (c) 2000, 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#include "about.h"

#include <qframe.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qtextbrowser.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>

#include "bronco.h"

/* 
 *  Constructs a AboutWin which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
AboutWin::AboutWin( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
   QPixmap image0( ( const char** ) bronco64_xpm );

   if ( !name )
      setName( "AboutWin" );

   setMinimumSize( 381, 289 ); 
   resize( 381, 289 ); 
   setCaption( tr( "About Bronco" ) );
   AboutWinLayout = new QVBoxLayout( this ); 
   AboutWinLayout->setSpacing( 6 );
   AboutWinLayout->setMargin( 11 );

   Frame8 = new QFrame( this, "Frame8" );
   QPalette pal;
   QColorGroup cg;
   cg.setColor( QColorGroup::Foreground, black );
   cg.setColor( QColorGroup::Button, QColor( 192, 192, 192) );
   cg.setColor( QColorGroup::Light, white );
   cg.setColor( QColorGroup::Midlight, QColor( 223, 223, 223) );
   cg.setColor( QColorGroup::Dark, QColor( 96, 96, 96) );
   cg.setColor( QColorGroup::Mid, QColor( 128, 128, 128) );
   cg.setColor( QColorGroup::Text, black );
   cg.setColor( QColorGroup::BrightText, white );
   cg.setColor( QColorGroup::ButtonText, black );
   cg.setColor( QColorGroup::Base, white );
   cg.setColor( QColorGroup::Background, white );
   cg.setColor( QColorGroup::Shadow, black );
   cg.setColor( QColorGroup::Highlight, black );
   cg.setColor( QColorGroup::HighlightedText, white );
   pal.setActive( cg );
   cg.setColor( QColorGroup::Foreground, black );
   cg.setColor( QColorGroup::Button, QColor( 192, 192, 192) );
   cg.setColor( QColorGroup::Light, white );
   cg.setColor( QColorGroup::Midlight, QColor( 220, 220, 220) );
   cg.setColor( QColorGroup::Dark, QColor( 96, 96, 96) );
   cg.setColor( QColorGroup::Mid, QColor( 128, 128, 128) );
   cg.setColor( QColorGroup::Text, black );
   cg.setColor( QColorGroup::BrightText, white );
   cg.setColor( QColorGroup::ButtonText, black );
   cg.setColor( QColorGroup::Base, white );
   cg.setColor( QColorGroup::Background, white );
   cg.setColor( QColorGroup::Shadow, black );
   cg.setColor( QColorGroup::Highlight, black );
   cg.setColor( QColorGroup::HighlightedText, white );
   pal.setInactive( cg );
   cg.setColor( QColorGroup::Foreground, QColor( 128, 128, 128) );
   cg.setColor( QColorGroup::Button, QColor( 192, 192, 192) );
   cg.setColor( QColorGroup::Light, white );
   cg.setColor( QColorGroup::Midlight, QColor( 220, 220, 220) );
   cg.setColor( QColorGroup::Dark, QColor( 96, 96, 96) );
   cg.setColor( QColorGroup::Mid, QColor( 128, 128, 128) );
   cg.setColor( QColorGroup::Text, black );
   cg.setColor( QColorGroup::BrightText, white );
   cg.setColor( QColorGroup::ButtonText, QColor( 128, 128, 128) );
   cg.setColor( QColorGroup::Base, white );
   cg.setColor( QColorGroup::Background, white );
   cg.setColor( QColorGroup::Shadow, black );
   cg.setColor( QColorGroup::Highlight, black );
   cg.setColor( QColorGroup::HighlightedText, white );
   pal.setDisabled( cg );
   Frame8->setPalette( pal );
   Frame8->setFrameShape( QFrame::Box );
   Frame8->setFrameShadow( QFrame::Sunken );
   Frame8Layout = new QHBoxLayout( Frame8 ); 
   Frame8Layout->setSpacing( 6 );
   Frame8Layout->setMargin( 11 );

   Layout2 = new QVBoxLayout; 
   Layout2->setSpacing( 6 );
   Layout2->setMargin( 0 );

   TextLabel1 = new QLabel( Frame8, "TextLabel1" );
   QFont TextLabel1_font(  TextLabel1->font() );
   TextLabel1_font.setPointSize( 26 );
   TextLabel1_font.setFamily("Times New Roman");
   TextLabel1_font.setBold( TRUE );
   TextLabel1->setFont( TextLabel1_font ); 
   TextLabel1->setText( tr( "Bronco" ) );
   Layout2->addWidget( TextLabel1 );

   Layout1 = new QHBoxLayout; 
   Layout1->setSpacing( 6 );
   Layout1->setMargin( 0 );

   TextLabel2 = new QLabel( Frame8, "TextLabel2" );
   QFont TextLabel2_font(  TextLabel2->font() );
   TextLabel2_font.setPointSize( 16 );
   TextLabel2_font.setFamily("Times New Roman");
   TextLabel2_font.setBold( TRUE ); 
   TextLabel2->setFont( TextLabel2_font ); 
   TextLabel2->setText( tr( "File Manager " ) + QString(VERSION) );
   Layout1->addWidget( TextLabel2 );
   Layout2->addLayout( Layout1 );
   Frame8Layout->addLayout( Layout2 );
   QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
   Frame8Layout->addItem( spacer );

   PixmapLabel1 = new QLabel( Frame8, "PixmapLabel1" );
   PixmapLabel1->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, PixmapLabel1->sizePolicy().hasHeightForWidth() ) );
   PixmapLabel1->setPixmap( image0 );
   PixmapLabel1->setScaledContents( TRUE );
   Frame8Layout->addWidget( PixmapLabel1 );
   AboutWinLayout->addWidget( Frame8 );

   TextBrowser1 = new QTextBrowser( this, "TextBrowser1" );
   TextBrowser1->setText( tr( "<html>&copy; 2000, 2001<br>\nKevin Worcester, kworces@earthlink.net<br>\nhttp://home.earthlink.net/~kworces/projects.html<br>" ) );
   TextBrowser1->setFrameShape( QFrame::Box );
   TextBrowser1->setLineWidth( 1 );
   TextBrowser1->setFrameShadow( QFrame::Sunken );
   AboutWinLayout->addWidget( TextBrowser1 );

   Layout6 = new QHBoxLayout; 
   Layout6->setSpacing( 6 );
   Layout6->setMargin( 0 );
   QSpacerItem* spacer_2 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
   Layout6->addItem( spacer_2 );

   PushButton1_3 = new QPushButton( this, "PushButton1_3" );
   PushButton1_3->setText( tr( "Ok" ) );
   PushButton1_3->setDefault( TRUE );
   Layout6->addWidget( PushButton1_3 );
   AboutWinLayout->addLayout( Layout6 );

   // signals and slots connections
   connect( PushButton1_3, SIGNAL( clicked() ), this, SLOT( accept() ) );
}

/*  
 *  Destroys the object and frees any allocated resources
 */
AboutWin::~AboutWin()
{
   // no need to delete child widgets, Qt does it all for us
}

/*  
 *  Main event handler. Reimplemented to handle application
 *  font changes
 */
bool AboutWin::event( QEvent* ev )
{
   bool ret = QDialog::event( ev ); 
   if ( ev->type() == QEvent::ApplicationFontChange ) {
      QFont TextLabel1_font(  TextLabel1->font() );
      TextLabel1_font.setPointSize( 26 );
      TextLabel1->setFont( TextLabel1_font ); 
      QFont TextLabel2_font(  TextLabel2->font() );
      TextLabel2_font.setPointSize( 16 );
      TextLabel2->setFont( TextLabel2_font ); 
   }
   return ret;
}

