/* bronco.cpp - Bronco initialization
 * 
 *  Bronco File Manager
 * 
 *  Copyright (c) 2000, 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#include <errno.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <fcntl.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "bronco.h"
#include "prefs.h"
#include "mainwin.h"
#include "llist.h"

/* Application icon, 32x32 */
const char * bronco32_xpm[] = {
"32 32 81 1",
" 	c None",
".	c #1B3A4D",
"+	c #364751",
"@	c #2A92D0",
"#	c #4E96BE",
"$	c #76B5DA",
"%	c #BABBBD",
"&	c #BAD2DE",
"*	c #156089",
"=	c #D0D8DC",
"-	c #CEE9F7",
";	c #2F769D",
">	c #4A6A7A",
",	c #116CA1",
"'	c #7D99A9",
")	c #EAEAEA",
"!	c #0879BA",
"~	c #8A969B",
"{	c #8AC1E2",
"]	c #EEEEEF",
"^	c #6B7175",
"/	c #097EC1",
"(	c #B5DDF2",
"_	c #238BC7",
":	c #EEF8FD",
"<	c #7B8D99",
"[	c #144562",
"}	c #959596",
"|	c #585D60",
"1	c #9BBBCB",
"2	c #1A87C6",
"3	c #999A9B",
"4	c #2A8AC2",
"5	c #F2F2F2",
"6	c #9ECFE9",
"7	c #F2F8FA",
"8	c #3E4E5A",
"9	c #0E567E",
"0	c #7F8184",
"a	c #4A92BA",
"b	c #0982C8",
"c	c #A0A8AD",
"d	c #528EAE",
"e	c #ABB2B5",
"f	c #F9FAFA",
"g	c #C2C5C8",
"h	c #1285C8",
"i	c #E1E6E9",
"j	c #D6DCDF",
"k	c #FEFEFE",
"l	c #E2F1F8",
"m	c #4A5961",
"n	c #AEC4D1",
"o	c #7DA5BC",
"p	c #5BA6D1",
"q	c #ACAEAE",
"r	c #2E8EC6",
"s	c #BCE0F6",
"t	c #1A78B0",
"u	c #BACEDA",
"v	c #89A6B6",
"w	c #245572",
"x	c #35586E",
"y	c #6C7A81",
"z	c #407492",
"A	c #0786CD",
"B	c #408AB6",
"C	c #EAF5FD",
"D	c #23638A",
"E	c #C8E1F0",
"F	c #2E92CB",
"G	c #4AA5D9",
"H	c #E0E1E2",
"I	c #6597B3",
"J	c #888D90",
"K	c #244454",
"L	c #AFBBC1",
"M	c #C9CCCF",
"N	c #3B6177",
"O	c #50778E",
"P	c #369AD2",
"kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk",
"f-sssssssssssE-lCkkkkkkkkkk7:Ckk",
"k{r2hhhhhhhhAh2_Pp6lkkkkkk-BdO7k",
"k]%<aAAAA,9DDDt/AAAF{]kkkkfa1lkk",
"kf]H&_bAA[|}3}JyzhbAhpjkkkfBgkkk",
"kkkklPAbbwc)75]HM'2bbhIHkkkBnkkk",
"kkkkCPAAbwqfkkkkk)$AbAtc7kHzvkkk",
"kkkk:@bbAwqfkkkkkksbAbbyHk:n1ikk",
"kkkk:PAbbwqfkkkkkkl_AAbNMko'Ezjk",
"kkkkCPbAAwqfkkkkkk-@bbbxgC;MkI'k",
"kkkkCPbbAwqfkkkkkkEhAb/+g:;uko<k",
"kkkkCPAAbwqfkkkkkf{AAb,m=ko'lzgk",
"kkkkCPbAAwqfkkkk:(@bb/.0)kkLcukk",
"kkkk:@Abb>L:::C-{@AA/.mgfkunoekk",
"kkkk:PAbb@GGGGP@AA/9.|e]kfoxLx=k",
"kkkk:@bAAb!/bAAbAAPd~M5kkk1OfNgk",
"kkkk:PbAb[+mmxNDtbbb2$Hkkk1OkNnk",
"kkkkCPAbbwJMMg%c<Ohb//#=fkvzlxLk",
"kkkkCPAAbwq7kfkf]Moh//!d=fgejqMk",
"kkkk:@bbAwqfkkkkkk)Gb//t~]:c<Lkk",
"kkkk:PAbbwqfkkkkkkf6//!!zM'~im)k",
"kkkkCPbAbwqfkkkkkkks///!DqNMk5kk",
"kkkkCPbAbwqfkkkkkkkE!!!!93>ck=7k",
"kkkkCPAbbwqfkkkkkkk(/!!!w3LKOy5k",
"kkkkC@AAbwqfkkkkkk:$!!!![ckMq]kk",
"kkkkCPAbbwqfkkkkkkE_!!!*+gHJ<cfk",
"kkkkl@/b/wqfkkkk:sG!!!,.0i|c5mqk",
"kkkk(hb/b;LlCl-($r!!!,.mg].Mf0^k",
"k-({Pbb///2@Fr2!!!!,9.|%5f8ekyyk",
"k$B2t,,,,,,,,,,**9[KmJM5kk%|%mHk",
"k]%}0^^^^^^^^^y^y0}q=)fkkkkHM)kk",
"kf]HiHjHHHHHHHHHH]5fkkkkkkkkkkkk"};

/* Application icon, 48x48 */
const char * bronco48_xpm[] = {
"48 48 81 1",
" 	c None",
".	c #19252F",
"+	c #27333A",
"@	c #33779F",
"#	c #48A0D2",
"$	c #9EA2A6",
"%	c #A4CCE3",
"&	c #C1C4C5",
"*	c #134058",
"=	c #B7DDF3",
"-	c #3E82AA",
";	c #125981",
">	c #DDDDDD",
",	c #E2F0F7",
"'	c #0E6A9E",
")	c #70797D",
"!	c #2B5269",
"~	c #629AB6",
"{	c #EDEDEE",
"]	c #3B5869",
"^	c #087EC1",
"/	c #ECF5F8",
"(	c #7EA2B6",
"_	c #7E7F81",
":	c #8AC5E8",
"<	c #F4F5F5",
"[	c #1B84C1",
"}	c #63676A",
"|	c #858686",
"1	c #F6FCFE",
"2	c #9CAEB9",
"3	c #295D7A",
"4	c #1685C8",
"5	c #2C83B6",
"6	c #7AB4D8",
"7	c #728FA2",
"8	c #2E3A3F",
"9	c #0782C8",
"0	c #496777",
"a	c #BAD0DA",
"b	c #4C91BA",
"c	c #A7B2B9",
"d	c #1A71A4",
"e	c #C2E0EE",
"f	c #C8CACA",
"g	c #FAFCFC",
"h	c #44515A",
"i	c #1283C6",
"j	c #127AB5",
"k	c #2D90C9",
"l	c #8E9CA3",
"m	c #9ED2EE",
"n	c #859098",
"o	c #0E82C9",
"p	c #434649",
"q	c #52788E",
"r	c #B5B5B5",
"s	c #2E4756",
"t	c #144E6F",
"u	c #62A5CB",
"v	c #CEDADF",
"w	c #50616B",
"x	c #AAD9F2",
"y	c #E4E5E6",
"z	c #91A8B7",
"A	c #133345",
"B	c #0686CE",
"C	c #5D869D",
"D	c #6D8490",
"E	c #D1E6F2",
"F	c #51575B",
"G	c #68B2DC",
"H	c #2A6B90",
"I	c #3E474D",
"J	c #0879BA",
"K	c #949495",
"L	c #FEFEFE",
"M	c #156391",
"N	c #B8BCBE",
"O	c #D1D3D5",
"P	c #8EB6CD",
"LLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLL",
"LL1L1L1L1L1L1L1L1L1LLLLLLLLLLLLLLLLLLLLLLLLLLLLL",
"LExmmmmmmmmmmmmmmmmmx=eE,/1LLLLLLLLLLLLL1,/E,LLL",
"L,b[[44444444444444444kk#G:E/gLLLLLLLLL/6-u-C/LL",
"L1eP75[BBBBBBBBio4o4iBBBBBi4#m,1LLLLLLLL%jqPl<LL",
"Lg{&$K7kBBBBB9;*t!!!3MdjB9BBi4ua{gLLLLLL=@(gLLLL",
"LLg{>OaG99999jAp)_|__))wHjBBB9BkPygLLLLL=dzLLLLL",
"LLLLg1,%9BBBBjA)rOOOOf&clC599B99kz>gLLL1vdzLLLLL",
"LLLLLL1x99999jsn><LgL<<{>fzk9B99i-cyLLLL=dzLLLLL",
"LLLLLg1x9BBBBjsKyLLLLLLgg{v6B9BBBjC&<LL/(H~yLLLL",
"LLLLLLgx^9999jsK{gLLLLLLLL1Ek99BB9@lyLL1a2cELLLL",
"LLLLLL1xoBBBBj!KyLLLLLLLLLg,GBB999H)OgL1ez~2,LLL",
"LLLLLLgx99B99jsKyLLLLLLLLLL/6BBBB9dw&<L%Hlv~qELL",
"LLLLLLLxoB9BBjsK{LLLLLLLLLL1:9999Bdhr<<b0yLE37gL",
"LLLLLL1x9B999jsKyLLLLLLLLLL/:BBB9Bdhc<,@C/L1@0/L",
"LLLLLLgxo9BBBj!KyLLLLLLLLLL,GB99B9Mpc<,@q{L1@3{L",
"LLLLLLgx9999BjsK{LLLLLLLLL1E#BB999tFr<1~HvL/3)<L",
"LLLLLL1xoBBB9j!KygLLLLLLLL/x499B9jA}f<LvHz,P3&LL",
"LLLLLLgx999B9jsKyLLLLLLLL/=#BB99^t8K>gLLv7D|&1LL",
"LLLLLL1xoBB9BjsK{LLLLLL1E=GBB9B9M.}r<LL11<//LLLL",
"LLLLLLgx^9B99iq2E////,Ex:kB9B9o;.h$O<LLaCPC32gLL",
"LLLLLLgxoB9BBoG6GGGGGG#kBB99oM*+FKO<LLgPt]lC!fLL",
"LLLLLLgxoB99B999BBBB9BBBBB4#Hsp)$>{LLLLE!Cga*2LL",
"LLLLLLLx99BB99Jjjoo9999B99B4#~zN><LLLLLE!7La*$LL",
"LLLLLLLxo99BB^AAs!!!!3Md^9B99^4ua{gLLLLEt7La*zLL",
"LLLLLL1x9BB99jAF|K|||)}w0Ho999^JkP><LLLE!7ga*$LL",
"LLLLLLgx99BB9js|fy>>vOfN2nq[999^Ji(v<Lg%*D/z*nLL",
"LLLLLLLxoB999jsKy<gggL<<yOr(49999J[l><gr7nO2DK/L",
"LLLLLLgx9BBB9JsKygLLLLLLLg{O6i^^9J^@2>LLg/y/LLLL",
"LLLLLL1xo999BjsK{LLLLLLLLLL<vk9^^^JJ)&<LN)qwNgLL",
"LLLLLLgx9BB9ojsKyLLLLLLLLLLL/6^J^^JJHKyvsl/D8>LL",
"LLLLLLgxoB9BBj!KyLLLLLLLLLLL1:^^^^^JM)v7!yL>l{LL",
"LLLLLLgx^9B99jslyLLLLLLLLLLLLxiJJJ^^'w&0w{LLLLLL",
"LLLLLL1xoB99BjsKyLLLLLLLLLLL1x^^JJJJ'hrq!yLLy1LL",
"LLLLLLgx9BB9oisKyLLLLLLLLLLL1mj^JJJJ'hr2*n,v)>LL",
"LLLLLL1xoB9B9jsK{LLLLLLLLLLL/:^J^JJJ;Ir/wAs8n<LL",
"LLLLLLgx^B999jsKyLLLLLLLLLLLEGJJJJJJ*FNL,$Dc<LLL",
"LLLLLL1xo99B9jsK{LLLLLLLLLL1=kJJJJJ'+}OL1yO{LLLL",
"LLLLLL1xoB999jsKyLLLLLLLLL1eGJJJJJJt8K>/)w_hl<LL",
"LLLLLL<m^9^99jslyLLLLLLLL1E:jJJJJJM.}c/n+&1c.$LL",
"LLLLLL,:^9^9^j!KyLLLLL1/ExG[JJJJJ'A8KO{!h{L<8p<L",
"LLLLL1e#^99^9^-Pe,,,Ee=mukJJJJJJMA8_&{,8h/LgI+{L",
"L/,,e=G4^99^99okb##k#kj^JJJJJ''tA8)r{g<hIyL1I8{L",
"LEG##k9^9^^^JJJJJJ^JJJJJJJJ''*A+F|&{<LL$.NL>._LL",
"L/ub55ddMM;MM;MMMM'MMMM;;t*+8I}|rO{LLLL<)h$}}yLL",
"LLyNK_)}wFFhFFhFFFFFwwFFF}})|$NO{<LLLLLL<fc&{LLL",
"LL/O&NNrrrrrrNrNrNrrrrNNN&fOyy<<gLLLLLLLLLLLLLLL",
"LLgg<{{{{{/{{{{{{{<{{<{{{<<gggLLLLLLLLLLLLLLLLLL"};

/* Application icon, 64x64 */
const char * bronco64_xpm[] = {
"64 64 81 1",
" 	c None",
".	c #132733",
"+	c #293237",
"@	c #2E7FAE",
"#	c #7C8084",
"$	c #72AED0",
"%	c #A9AAAB",
"&	c #A8D4EE",
"*	c #133C53",
"=	c #B3DDF5",
"-	c #15587F",
";	c #DEDFDF",
">	c #136899",
",	c #2B5871",
"'	c #E0ECF3",
")	c #63AFDD",
"!	c #EEEFF0",
"~	c #3C5F75",
"{	c #427E9F",
"]	c #097EC1",
"^	c #7EB6D7",
"/	c #EFF8FC",
"(	c #4DA0D0",
"_	c #58585A",
":	c #1E8AC8",
"<	c #293A43",
"[	c #7694A6",
"}	c #F6F6F6",
"|	c #CBDCE5",
"1	c #ABBDC7",
"2	c #537E97",
"3	c #464E53",
"4	c #B2D0DF",
"5	c #7F8D94",
"6	c #82C3E8",
"7	c #1884C3",
"8	c #F6FBFC",
"9	c #0982C9",
"0	c #5E6368",
"a	c #C3E3F4",
"b	c #1B76AC",
"c	c #89A9BB",
"d	c #969EA3",
"e	c #486F86",
"f	c #BFD3DE",
"g	c #0879BA",
"h	c #5C92B0",
"i	c #91CAEC",
"j	c #B6B7B8",
"k	c #254A5F",
"l	c #2A739D",
"m	c #2D8EC5",
"n	c #123448",
"o	c #BEC2C4",
"p	c #627079",
"q	c #1283C6",
"r	c #9BB6C8",
"s	c #155072",
"t	c #667F8E",
"u	c #0686CE",
"v	c #D4EBF8",
"w	c #66A1C4",
"x	c #8F8F8F",
"y	c #C9CACC",
"z	c #354A56",
"A	c #1674A8",
"B	c #FEFEFE",
"C	c #95989A",
"D	c #9BA6AC",
"E	c #E4F4FC",
"F	c #1275AF",
"G	c #E3E6E8",
"H	c #4D5C64",
"I	c #428DB8",
"J	c #16618C",
"K	c #6F7173",
"L	c #124765",
"M	c #286487",
"N	c #9FCFED",
"O	c #D1D2D3",
"P	c #64899E",
"BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB",
"BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB",
"B8/EEEEEEEEEEEEEEEEEEEEEEE///88BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB",
"B!=iiiiiiiiiiiiiiiiiiiiiiiiN&&aavE/8BBBBBBBBBBBBBBBBBB/vEvaEBBBB",
"B84m::::::::::::::::::::::::mmm((^i=vE8BBBBBBBBBBBBB8=$I^hlw!BBB",
"BBv^wI@q99u99uuuuuu9u99uuuuuuuu9uuq:(6='}BBBBBBBBBBB81IA{2e2GBBB",
"BB};1dP2Iquu999999u]b]bbb7bqquuu9uu99qm)='}BBBBBBBBBB/$>e|O|BBBB",
"BB}!OjDx[w99uuuuu9gLn*kkkkkk,JAb99uuuuu7m^|G8BBBBBBBB8)>[}BBBBBB",
"BB8}!;Oyoim99999u9J.+_KKK#KKK0H~Mb9999u9q7w4G}BBBBBBB8$>[/BBBBBB",
"BBBB}}}!G=(uuuuu99-+_x%jojojj%Dxte@9uu9uuuqh1;}BBBBBB8$>[}BBBBBB",
"BBBBBBBB}v)]uuu9uuJkKo;G!G!GG;OOoD[I]u9uuuu7hj;}BBBBB/$>[8BBBBBB",
"BBBBBBBB8v)u9uuu99-zxy!}BB}BBB8!GO1cI9u99u99@[o!8BBB8/$>[}BBBBBB",
"BBBBBBBBBv)9u9uu9uJzxy}BBBBBBBBBB}GO6:uu9u99q{CO}BBB8|I>P|BBBBBB",
"BBBBBBBB8v)9uuu9u9-zxO}BBBBBBBBBBB8'|(9uu9uu9btjG8BB/1P2tr'BBBBB",
"BBBBBBBBBv)9u9uu9uJzxy}BBBBBBBBBBBB}'6q9u9999beC;}BBB}'v|'8BBBBB",
"BBBBBBBBBv)9u9u9u9JzxO}BBBBBBBBBBBBB/=:u9uuuu],#y!BB8EcP2PrEBBBB",
"BBBBBBBBBv)9u9uu99Jzxy}BBBBBBBBBBBBB8=mu9u9uuqMpj!BBah,dfcMPGBBB",
"BBBBBBBBBv)9uuu999JzxO}BBBBBBBBBBBBB8a(uu9u99]-0%GB8$M2!B/[J[/BB",
"BBBBBBBBBv)9u9uuu]Jzxy}BBBBBBBBBBBBB8a(9uu9u9q-_DGB'h-D8BBN-eOBB",
"BBBBBBBBBv)99uu999JzxO}BBBBBBBBBBBBB8=(uu9u99]s_dG8v{JrBBB|M,oBB",
"BBBBBBBBBv)uu9uuu9Jz#O}BBBBBBBBBBBBB/&muuu999gL3%;Bv{JcBBBaM-oBB",
"BBBBBBBBBv)9u9u999JzxO}BBBBBBBBBBBBBvN79u9uu9bn_%GB!hJ[}BBfM,yBB",
"BBBBBBBBBv)9u9uuu9Jzxy}BBBBBBBBBBBB8v6]uuu9u9>+0j!B81>2vB8rstGBB",
"BBBBBBBBBv)9uuu999JzxO}BBBBBBBBBBB8v&m9u9999FL<#o!B8E[Mcv=e,oBBB",
"BBBBBBBBBv)9u9uuu9Jzxy}BBBBBBBBBB8E=6uuuu999Jn3CO}BBB'cetpto}BBB",
"BBBBBBBBBv)999u999JzxO}BBBBBBBBB8v=679uu9u9An+KjG}BBBB}GO;!BBBBB",
"BBBBBBBBBv)uuuuuu]JzxO!BBBBBBB/Ea=):uuu999F*._xy}BBBB/v/'f|BBBBB",
"BBBBBBBBBv)99u9u99@PDf'EEEEvvv=N6(qu9u999Jn+3#oG}BB}12ere,~c}BBB",
"BBBBBBBBBv)9u9uuu9m666666666^)(muuu9u9]>L.+_#j;}BBB81Ms~t5,,oBBB",
"BBBBBBBBBv)99u9u99uuqqqqqq:uuuuu999u7ALn+z0xj;}BBBBBE2Lt'E2Ld8BB",
"BBBBBBBBBv)uu9uuuuuuuuuuu9uuuu9uuu9:()Pe0#%yG!8BBBBBE2LdB8P*58BB",
"BBBBBBBBBv)9u9uu99]FFFb]qqqq99u9uuuuu:mw^1yG}BBBBBBBE2LDB8P*5}BB",
"BBBBBBBBBv)9u9uuu9>nnkkkkks,J>Ag]uu99uuq7(r|!}BBBBBBE2LD88P*58BB",
"BBBBBBBBBv)9u99u99-.<_pKpp0HHH3,MAqu999]]9:w4;}BBBBBE2LDB8P*5}BB",
"BBBBBBBBBv)9u9uuu9J<0x%j%%%%dCx#KeM7]999]]9qI1|}BBBBE2Lc8/2*58BB",
"BBBBBBBBBv)9uu9uu9-z#o;GGGG;;Oyojd52@99999]]]IrO!8B84~L[!fe*tGBB",
"BBBBBBBBBv)9u99uu9Jzxy!B}B}}B}}!G;o%[I]9999]]FI%;!8}rp0ty1p0tyB8",
"BBBBBBBBBv)99u9u9u-zxy}BBBBBBBBB}}!;o^:]]]]]]]F2j;}B8!!!8}E!!}BB",
"BBBBBBBBBv)uu99uuuJzxy}BBBBBBBBBBB8}Gf$99]]]]]]b5o!BBB/f1o'8BBBB",
"BBBBBBBBBv)9uu9u99JzxO}BBBBBBBBBBBBB}G=7]]]]]]gFeDO!BvtHp~zj8BBB",
"BBBBBBBBBv)9u9uuu]Jzxy}BBBBBBBBBBBBBB}v(]]g]]]]gM#o!'t<%!1<zOBBB",
"BBBBBBBBBv)99u9u99JzxO}BBBBBBBBBBBBBBBv6]]]gggggle%;1Lp!BEt0OBBB",
"BBBBBBBBBv)uu9uu99Jzxy}BBBBBBBBBBBBBBBE6]]]]]ggg>,CO[*C}BB';8BBB",
"BBBBBBBBBv)9u9uuu9JzxO}BBBBBBBBBBBBBBB/Nq]]ggggggz#ytndBBBBBBBBB",
"BBBBBBBBBv)9u9u9u]Jzxy}BBBBBBBBBBBBBBB/Nqg]]ggggFkKoPn5}BBB!8BBB",
"BBBBBBBBBv)9u9uu99JzxO}BBBBBBBBBBBBBBBENqggg]]ggF*Kocn~|BBvdOBBB",
"BBBBBBBBBv)9u9uuu9Jzxy}BBBBBBBBBBBBBBB'6]]ggg]gg><Ko;,np11pp;BBB",
"BBBBBBBBBv)9u9uu9]JzxO}BBBBBBBBBBBBBBBv^gggggggg><Ky8j*n.n<%}BBB",
"BBBBBBBBBv)9u9u]u9Jzxy}BBBBBBBBBBBBBB/a(g]ggggggL<#yB8ytHKy8BBBB",
"BBBBBBBBBv)999u99]JzxO}BBBBBBBBBBBBB8'&:gggggggF*<xOBBB}!/BBBBBB",
"BBBBBBBBBv)]u9u99]Jzxy}BBBBBBBBBBBBB/a$]ggggggFJ._d;B}fd5Cf8BBBB",
"BBBBBBBB8v)u9]999]JzxO}BBBBBBBBBBBB/vNmgggggFgF*+KjG/dzpCK<x!BBB",
"BBBBBBBB8a)gu]]]9]Jzxy}BBBBBBBBBBBEa&(gggggggFs.3xO!r<3y};0.d8BB",
"BBBBBBBB8a(u]9999]-zxO}BBBBBBBBB/vai(gggggFggJ.+K%;!p.58BBd.3;BB",
"BBBBBBBBE&m9999]99>~5y!BBBB8//EvaN):FgggggFF-*n_dO!'3.C8BBO..jBB",
"BBBBBBBEa6799]]9]]qIrfavvvva=&N6(mFgggggggF-n+3xo!8;z.CBBBO+.%BB",
"BB//Evv=im9]99]99]99m((((((((:7Fg]gggggF>>L.n3xjG}B'H.#8BBO+.jBB",
"BE=NN6^(:9]9]]]]]]]]]]]]]]]ggggggggggg>>s.+<0xj;}BBB[._GBBj.3OBB",
"B8Nm77qgg]g]]]ggggg]gggggggggggFFF>>-sn.nz_#%yG}BBBB;3njB'p.x}BB",
"BB|$hI{@@l>J--------J-JJMJJJM--ssLL*n+z_K#doO!}BBBBB8yH3503xGBBB",
"BB};od5#K0H333z3z3z333333_33_333___0KKxC%yO!!}BBBBBBBB;%C%o!BBBB",
"BB8'Oj%dCCxCxxxxxxxxxxxCxxCxxCxCCCD%joyOG!}}BBBBBBBBBBBBB}BBBBBB",
"BBB}G;OOOOOOOOOOOOOOOOOOOOOOOOOOO;;;GG}}}BBBBBBBBBBBBBBBBBBBBBBB",
"BBBBB}}B}}}}}}}}}}}}}}}}}!}!}}}}}}}}BBBBBBBBBBBBBBBBBBBBBBBBBBBB"};

Preferences prefs;
LinkedList clipDir;
char clip_dir[4096] = "";
MainWin *mainwin;
int move_operation = -1;
int fifo_fd = 0;

bool helper_debug;

void sighup(int s){
   if(mainwin){
      prefs.load();
      mainwin->refresh();
   }
}

void sigusr(int s){
   if(mainwin){
      mainwin->setActiveWindow();
      mainwin->raise();
   }
}

void sigint(int s){
   printf("Caught SIGINT\n");
}

void sigpipe( int s )
{
   return;
}

void text_help(char *pname){
   printf("Bronco %s.  A Qt-based file manager for the X Windowing System.\n", VERSION);
   printf("\n");
   printf("Usage: %s [OPTION] [DIRECTORY]\n\n", pname);
   printf("Available options:\n");
   printf("  -h, --help       Show help message and exit\n");
   printf("  -v, --version    Show version and exit\n");
   printf("  -f, --nofork     Do not fork on start\n");
   printf("  -t, --text       Use text interface (assumes -f)\n");
   printf("  -d, --default    Use default settings\n");
   printf("      --kill       Kill stale bronco process and restart\n");
   printf("      --home=DIR   Use DIR as home directory\n");
   printf("      --helper     Turn on helper application debugging\n");
   printf("\n");

   exit(0);
}

void print_version(char *pname){
   printf("\n");
   printf("Bronco Version %s, Copyright (c) 2000, 2001 Kevin Worcester\n", VERSION);
   printf("\n");
   printf("This program is Free software.  It is covered under version 2 of\n");
   printf("the GNU General Public License as published by the Free Software\n");
   printf("Foundation.  See Help->License or COPYING for details.\n");
   printf("\n");
   exit(0);
}

class MyApp 
#ifdef USE_KDE
   : public KApplication 
#else /* ! USE_KDE */
   : public QApplication 
#endif /* USE_KDE */
{
public:
   MyApp::MyApp(int, char **);
};

MyApp::MyApp(int argc, char **argv) 
#ifdef USE_KDE
   : KApplication(argc, argv, "Bronco")
#else /* ! USE_KDE */
   : QApplication(argc, argv)
#endif /* USE_KDE */
{
}

int get_bronco_pid ()
{
   char file[1024];
   char pidstr[20];

   FILE *fp;

   snprintf(file, sizeof(file), "%s/.bronco/pid", prefs.homeDir());
   if ( (fp = fopen(file, "r")) != NULL )
   {
      fgets(pidstr, sizeof(pidstr), fp);
      int pid = atoi(pidstr);
      fclose(fp);
      return ( pid );
   }

   return ( 0 );
}

int is_running()
{
   int pid = get_bronco_pid();

   if ( pid )
   {
      /* If this is true, we can't send a signal to the last Bronco
       * process... Bronco is not running. */
      if ( kill(pid, 0) != 0)
      {
         return 0;
      }
   }

   int fd = open(prefs.getFifoFileName(), (O_WRONLY | O_NONBLOCK) );

   if ( fd < 0 )
   {
      fd = 0;
   }

   return( fd );
}

int init_setup()
{
   struct stat buf;

   QString filename;

   filename.sprintf("%s", prefs.homeDir());
   if ( lstat(filename.latin1(), &buf) == 0)
   {
      if ( ! S_ISDIR(buf.st_mode) )
      {
         fprintf(stderr, "%s is not a directory\n", filename.latin1() );
         return( 0 );
      }
   }
   else
   {
      if ( mkdir(filename.latin1(), 0700) != 0 )
      {
         fprintf(stderr, "Could not create %s as directory: %s\n", filename.latin1(), strerror(errno));
         return( 0 );
      }
   }

   filename.sprintf("%s/.bronco", prefs.homeDir());
   if ( lstat(filename.latin1(), &buf) == 0)
   {
      if ( ! S_ISDIR(buf.st_mode) )
      {
         fprintf(stderr, "%s is not a directory\n", filename.latin1() );
         return( 0 );
      }
   }
   else
   {
      if ( mkdir(filename.latin1(), 0700) != 0 )
      {
         fprintf(stderr, "Could not create %s as directory: %s\n", filename.latin1(), strerror(errno));
         return( 0 );
      }
   }

   filename.sprintf("%s", prefs.getFifoFileName());
   if ( lstat(filename.latin1(), &buf) == 0)
   {
      if ( ! S_ISFIFO(buf.st_mode) )
      {
         fprintf(stderr, "%s is not a FIFO\n", filename.latin1() );
         return( 0 );
      }
   }
   else
   {
      if ( mkfifo(filename.latin1(), 0600) != 0 )
      {
         fprintf(stderr, "Could not create %s as FIFO: %s\n", filename.latin1(), strerror(errno));
         return( 0 );
      }
   }

   return( 1 );
}

void clear_bronco_rc(void)
{
   DIR *dp;
   struct dirent *d;

   QString file;
   QString dir = QString(prefs.homeDir()) + "/.bronco";

   if((dp = opendir(dir.latin1())) != NULL)
   {
      while((d = readdir(dp)) != NULL)
      {
         if ( strncmp(d->d_name, "child", 5) == 0 || strncmp(d->d_name, "errorlog", 8) == 0 )
         {
            file = dir + QString("/") + QString(d->d_name);
            unlink(file.latin1());
            printf("Cleaned up %s\n", file.latin1());
         }
      }
      closedir(dp);
   }
   else
   {
      fprintf(stderr, "Could not open %s: %s\n", dir.latin1(), strerror(errno));
      return;
   }
   return;
}

int write_dirs(int fd, int argc, char *argv[])
{
   int c;
   int t;
   int mainset = 0;
   const char *offset;
   const char *home = NULL;
   char pwd[1024];
   char buffer[4096];
   bool optsDone = false;

   QString filename;

   home = prefs.homeDir();
   getcwd(pwd, sizeof(pwd) - 1);

   for(c=1; c<argc; c++){
      if(optsDone || argv[c][0] != '-'){
         if(argv[c][0] == '~'){
            strcpy(buffer, home);
            strcat(buffer, argv[c] + 1);
         }
         else if(argv[c][0] != '/'){
            snprintf(buffer, sizeof(buffer), "%s/", pwd);
            offset = argv[c];
            while(1){
               if((*offset == '.') && (*(offset + 1) == '/')){
                  offset += 2;
               }
               if((*offset == '.') && (*(offset + 1) == '\0')){
                  offset += 1;
               }
               else if((*offset == '.') /* Go up one directory */
                     && (*offset + 1) 
                     && (*(offset + 2) == '/' || *(offset + 2) == '\0')){

                  offset += 2;
                  if(*offset != '\0') offset++;

                  t = strlen(buffer) - 1;
                  if(t > 0) buffer[t] = '\0';
                  while(t > 0){
                     if(buffer[t] == '/'){
                        break;
                     }
                     else{
                        buffer[t] = '\0';
                     }
                     t--;
                  }
               }
               else{
                  break;
               }
            }
            strcat(buffer, offset);
         }
         else{
            strcpy(buffer, argv[c]);
         }

         filename.sprintf("50 %d %s\n", getpid(), buffer);
         write( fd, filename.latin1(), filename.length() );

         mainset = 1;
      }

      if ( strlen(argv[c]) == 2 && strcmp(argv[c], "--") == 0)
      {
         optsDone = true;
      }
   }

   if(!mainset){
      char *dir;
      if((dir = getcwd(NULL, 0)) != NULL){
         filename.sprintf("50 %d %s\n", getpid(), dir);
         write( fd, filename.latin1(), filename.length() );
         free(dir);
      }
   }

   return(0);
}

int main(int argc, char *argv[]){
   char *carg;

   int opt_fork    = 1;
   int opt_text    = 0;
   int opt_pimp    = 0;
   int opt_default = 0;
   int opt_kill    = 0;
   char *opt_home  = NULL;

   helper_debug    = false;

   clipDir.clear();

   for(int c=1; c<argc; c++){
      carg = argv[c];
      if(strncmp(carg, "-", 1)                 == 0){
         if(  (strcmp(carg, "-h")              == 0) 
           || (strcmp(carg, "--help")          == 0)) text_help(argv[0]);
         else if((strcmp(carg, "-v")           == 0) 
           || (strcmp(carg, "--version")       == 0)) print_version(argv[0]);
         else if((strcmp(carg, "-f")           == 0) 
           || (strcmp(carg, "--nofork")        == 0)) opt_fork=0;
         else if((strcmp(carg, "-d")           == 0) 
           || (strcmp(carg, "--default")       == 0)) opt_default=1;
         else if((strcmp(carg, "-t")           == 0) 
           || (strcmp(carg, "--text")          == 0)) opt_text=1, opt_fork=0;
         else if( strncmp(carg, "--home=", 7)  == 0)  opt_home = (strlen(carg)>7) ? carg+7 : NULL;
         else if( strcmp(carg, "--kill")       == 0)  opt_kill=1;
         else if( strcmp(carg, "--pimp")       == 0)  opt_pimp=1, printf("Pimp mode is on by default.  Oh yeah!\n");
         else if( strcmp(carg, "--helper")     == 0)  helper_debug=true;
         else if( strcmp(carg, "--nopimp")     == 0)  printf("Pimp mode can not be disabled.  Oh yeah!\n");
         else if( strcmp(carg, "--train")      == 0)  printf("Santa Fe, all the way!\n");
         else if( strcmp(carg, "--")           == 0)  break;  /* end of arguments */
         else printf("Ignoring unrecognized argument: %s\n", carg);
      }
   }

   if(opt_default){
      if(opt_home) prefs.setHome(opt_home);
      prefs.useDefault();
   }
   else{
      if(opt_home) prefs.setHome(opt_home);
      prefs.load();
   }

   if ( ! init_setup() )
   {
      fprintf(stderr, "Could not create necessary directories and config files.\n");
   }

   clear_bronco_rc();

   int fd;
   QString filename;

   if ( opt_kill )
   {
      int pid = get_bronco_pid();

      if ( pid )
      {
         if ( kill(pid, SIGKILL) != 0 )
         {
            fprintf(stderr, "Could not send kill signal\n");
         }
      }
      else
      {
         fprintf(stderr, "Unknown pid\n");
      }
   }
   else
   {
      fd = is_running();

      if ( fd )
      {
         write_dirs(fd, argc, argv);
         close( fd );
         return(0);
      }
   }

   filename.sprintf("%s", prefs.getFifoFileName());
   fifo_fd = open(filename.latin1(), (O_RDWR | O_NONBLOCK) );

   if(opt_fork){
      switch(fork()){
         case -1:
            fprintf(stderr, "%s: fork failed: %s\n", argv[0], strerror(errno));
            fprintf(stderr, "Attempting to continue in foreground\n");
            break;
         case 0:
            break;
         default:
            return(0);
      }
   }

   if(opt_text){
      printf("Using text interface.  Starting /bin/tcsh...\n");
      if(system("/bin/tcsh") < 0)  
         fprintf(stderr, "%s\n", strerror(errno));
      else                         
         printf("Bronco exiting...\n");
      return(0);
   }

   filename.sprintf("%s/.bronco/pid", prefs.homeDir() );
   FILE *fp = fopen(filename.latin1(), "w");
   if ( fp )
   {
      fprintf(fp, "%d\n", getpid() );
      fclose(fp);
   }

   filename.sprintf("%s/.bronco/version", prefs.homeDir() );
   fp = fopen(filename.latin1(), "w");
   if ( fp )
   {
      fprintf(fp, "%s\n", VERSION );
      fclose(fp);
   }

   signal(SIGHUP,  &sighup);
   signal(SIGUSR1, &sigusr);
   signal(SIGINT,  &sigint);
   signal(SIGPIPE, &sigpipe);

   MyApp app(argc, argv);
   //app.setStyle(new QWindowsStyle);

   mainwin = new MainWin(NULL, "");
   if(opt_pimp && getenv("USER")){
      char text[80];
      snprintf(text, sizeof(text), "%s is pimp!\n", getenv("USER"));
      mainwin->setCaption(text);
   }
   else{         
      mainwin->setCaption("Bronco File Manager");
   }
   mainwin->setIconText("Bronco");
   mainwin->show();
   app.setMainWidget(mainwin);

   mainwin->passArgs(argc, argv);
   
   return app.exec();
}
