/* chmod.cpp - chmod dialog window
 * 
 *  Bronco File Manager
 * 
 *  Copyright (c) 2000, 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#include <qstring.h>
#include <qlabel.h>
#include <qmessagebox.h>
#include <dirent.h>
#include "chmod.h"
#include "direct.h"

#define BLK_SIZE 512

Chmod::Chmod() : QWidget(NULL, "", WDestructiveClose){
   setCaption("Permissions");

   dir  = new LinkedList();

   grid = new QGridLayout(this, 0, 0, 5, 3, "");

   read_l      = new QLabel("Read",      this, "");
   write_l     = new QLabel("Write",     this, "");
   exec_l      = new QLabel("Exec",      this, "");
   sticky_l    = new QLabel("Sticky",    this, "");

   user_l      = new QLabel("User:",     this, "");
   group_l     = new QLabel("Group:",    this, "");
   other_l     = new QLabel("Other:",    this, "");
   recursive_l = new QLabel("Recursive", this, "");

   user_r      = new QCheckBox(this, "");
   user_w      = new QCheckBox(this, "");
   user_x      = new QCheckBox(this, "");
   group_r     = new QCheckBox(this, "");
   group_w     = new QCheckBox(this, "");
   group_x     = new QCheckBox(this, "");
   other_r     = new QCheckBox(this, "");
   other_w     = new QCheckBox(this, "");
   other_x     = new QCheckBox(this, "");
   sticky_u    = new QCheckBox(this, "");
   sticky_g    = new QCheckBox(this, "");
   sticky_o    = new QCheckBox(this, "");
   recursive_v = new QCheckBox(this, "");

   recursive_v->setChecked( false );

   grid->addWidget(read_l,      0, 1, 0);
   grid->addWidget(write_l,     0, 2, 0);
   grid->addWidget(exec_l,      0, 3, 0);
   grid->addWidget(sticky_l,    0, 4, 0);

   grid->addWidget(user_l,      1, 0, 0);
   grid->addWidget(group_l,     2, 0, 0);
   grid->addWidget(other_l,     3, 0, 0);
   grid->addWidget(recursive_l, 4, 0, 0);

   grid->addWidget(user_r,      1, 1, 0);
   grid->addWidget(user_w,      1, 2, 0);
   grid->addWidget(user_x,      1, 3, 0);
   grid->addWidget(group_r,     2, 1, 0);
   grid->addWidget(group_w,     2, 2, 0);
   grid->addWidget(group_x,     2, 3, 0);
   grid->addWidget(other_r,     3, 1, 0);
   grid->addWidget(other_w,     3, 2, 0);
   grid->addWidget(other_x,     3, 3, 0);
   grid->addWidget(sticky_u,    1, 4, 0);
   grid->addWidget(sticky_g,    2, 4, 0);
   grid->addWidget(sticky_o,    3, 4, 0);

   grid->addMultiCellWidget(recursive_l, 4, 4, 0, 3, AlignLeft);
   grid->addWidget(recursive_v, 4, 4, 0);

   connect(user_r, SIGNAL(stateChanged(int)),   this, SLOT(change_ur(int)));
   connect(user_w, SIGNAL(stateChanged(int)),   this, SLOT(change_uw(int)));
   connect(user_x, SIGNAL(stateChanged(int)),   this, SLOT(change_ux(int)));

   connect(group_r, SIGNAL(stateChanged(int)),  this, SLOT(change_gr(int)));
   connect(group_w, SIGNAL(stateChanged(int)),  this, SLOT(change_gw(int)));
   connect(group_x, SIGNAL(stateChanged(int)),  this, SLOT(change_gx(int)));

   connect(other_r, SIGNAL(stateChanged(int)),  this, SLOT(change_or(int)));
   connect(other_w, SIGNAL(stateChanged(int)),  this, SLOT(change_ow(int)));
   connect(other_x, SIGNAL(stateChanged(int)),  this, SLOT(change_ox(int)));

   connect(sticky_u, SIGNAL(stateChanged(int)), this, SLOT(change_su(int)));
   connect(sticky_g, SIGNAL(stateChanged(int)), this, SLOT(change_sg(int)));
   connect(sticky_o, SIGNAL(stateChanged(int)), this, SLOT(change_so(int)));

   change = new QPushButton("Change", this, "Change");
   change->setDefault(true);
   change->setFocus();
   connect(change, SIGNAL(clicked()), this, SLOT(changeMode()));

   cancel = new QPushButton("Cancel", this, "Cancel");
   connect(cancel, SIGNAL(clicked()), this, SLOT(close()));

   grid->addMultiCellWidget(change,  5, 5, 0, 1, AlignCenter);
   grid->addMultiCellWidget(cancel,  5, 5, 3, 4, AlignCenter);

   clear();
}

Chmod::~Chmod()
{
}

/* 
 * We actually set the values of the widgets here because the
 * values can change so much between creation and display
 */
void Chmod::show(void){
   
   user_r->setTristate(TRUE);
   user_w->setTristate(TRUE);
   user_x->setTristate(TRUE);
   group_r->setTristate(TRUE);
   group_w->setTristate(TRUE);
   group_x->setTristate(TRUE);
   other_r->setTristate(TRUE);
   other_w->setTristate(TRUE);
   other_x->setTristate(TRUE);
   sticky_u->setTristate(TRUE);
   sticky_g->setTristate(TRUE);
   sticky_o->setTristate(TRUE);

   if((S_IRUSR) & (min_mode & max_mode))
      user_r->setChecked(1);
   else if((S_IRUSR) & (min_mode ^ max_mode))
      user_r->setNoChange();
   else
      user_r->setChecked(0), changed_ur = 1;

   if((S_IWUSR) & (min_mode & max_mode))
      user_w->setChecked(1);
   else if((S_IWUSR) & (min_mode ^ max_mode))
      user_w->setNoChange();
   else
      user_w->setChecked(0), changed_uw = 1;

   if((S_IXUSR) & (min_mode & max_mode))
      user_x->setChecked(1);
   else if((S_IXUSR) & (min_mode ^ max_mode))
      user_x->setNoChange();
   else
      user_x->setChecked(0), changed_ux = 1;

   if((S_IRGRP) & (min_mode & max_mode))
      group_r->setChecked(1);
   else if((S_IRGRP) & (min_mode ^ max_mode))
      group_r->setNoChange();
   else
      group_r->setChecked(0), changed_gr = 1;

   if((S_IWGRP) & (min_mode & max_mode))
      group_w->setChecked(1);
   else if((S_IWGRP) & (min_mode ^ max_mode))
      group_w->setNoChange();
   else
      group_w->setChecked(0), changed_gw = 1;

   if((S_IXGRP) & (min_mode & max_mode))
      group_x->setChecked(1);
   else if((S_IXGRP) & (min_mode ^ max_mode))
      group_x->setNoChange();
   else
      group_x->setChecked(0), changed_gx = 1;

   if((S_IROTH) & (min_mode & max_mode))
      other_r->setChecked(1);
   else if((S_IROTH) & (min_mode ^ max_mode))
      other_r->setNoChange();
   else
      other_r->setChecked(0), changed_or = 1;

   if((S_IWOTH) & (min_mode & max_mode))
      other_w->setChecked(1);
   else if((S_IWOTH) & (min_mode ^ max_mode))
      other_w->setNoChange();
   else
      other_w->setChecked(0), changed_ow = 1;

   if((S_IXOTH) & (min_mode & max_mode))
      other_x->setChecked(1);
   else if((S_IXOTH) & (min_mode ^ max_mode))
      other_x->setNoChange();
   else
      other_x->setChecked(0), changed_ox = 1;

   if((S_ISUID) & (min_mode & max_mode))
      sticky_u->setChecked(1);
   else if((S_ISUID) & (min_mode ^ max_mode))
      sticky_u->setNoChange();
   else
      sticky_u->setChecked(0), changed_or = 1;

   if((S_ISGID) & (min_mode & max_mode))
      sticky_g->setChecked(1);
   else if((S_ISGID) & (min_mode ^ max_mode))
      sticky_g->setNoChange();
   else
      sticky_g->setChecked(0), changed_ow = 1;

   if((S_ISVTX) & (min_mode & max_mode))
      sticky_o->setChecked(1);
   else if((S_ISVTX) & (min_mode ^ max_mode))
      sticky_o->setNoChange();
   else
      sticky_o->setChecked(0), changed_ox = 1;

   QWidget::show();
}

void Chmod::hide(){
   QWidget::hide();
}

int Chmod::addFile(const char *filename){
   struct stat statbuf;
   if(lstat(filename, &statbuf) < 0){
      QMessageBox::warning( NULL, filename,
            QString(strerror(errno)) , QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape, 0);
   }
   return(addFile(filename, &statbuf));
}

int Chmod::addFile(const char *filename, const struct stat *statbuf){

   dir->add(filename);

   files++;

   if(files > 1){
      max_mode |= statbuf->st_mode;
      min_mode &= statbuf->st_mode;
   }
   else{
      max_mode  = statbuf->st_mode;
      min_mode  = statbuf->st_mode;
   }

   return(0);
}

void Chmod::changeMode(void){
   char fname[2048];

   mode_t mode_on  = 0;
   mode_t mode_off = (S_IRWXU | S_IRWXG | S_IRWXO | S_ISUID | S_ISGID | S_ISVTX);

   if(changed_ur)
      if(user_r->isChecked())
         mode_on  |= S_IRUSR;
      else
         mode_off ^= S_IRUSR;
   if(changed_uw)
      if(user_w->isChecked())
         mode_on  |= S_IWUSR;
      else
         mode_off ^= S_IWUSR;
   if(changed_ux)
      if(user_x->isChecked())
         mode_on  |= S_IXUSR;
      else
         mode_off ^= S_IXUSR;
   if(changed_gr)
      if(group_r->isChecked())
         mode_on  |= S_IRGRP;
      else
         mode_off ^= S_IRGRP;
   if(changed_gw)
      if(group_w->isChecked())
         mode_on  |= S_IWGRP;
      else
         mode_off ^= S_IWGRP;
   if(changed_gx)
      if(group_x->isChecked())
         mode_on  |= S_IXGRP;
      else
         mode_off ^= S_IXGRP;
   if(changed_or)
      if(other_r->isChecked())
         mode_on  |= S_IROTH;
      else
         mode_off ^= S_IROTH;
   if(changed_ow)
      if(other_w->isChecked())
         mode_on  |= S_IWOTH;
      else
         mode_off ^= S_IWOTH;
   if(changed_ox)
      if(other_x->isChecked())
         mode_on  |= S_IXOTH;
      else
         mode_off ^= S_IXOTH;

   if(changed_su)
      if(sticky_u->isChecked())
         mode_on  |= S_ISUID;
      else
         mode_off ^= S_ISUID;
   if(changed_sg)
      if(sticky_g->isChecked())
         mode_on  |= S_ISGID;
      else
         mode_off ^= S_ISGID;
   if(changed_so)
      if(sticky_o->isChecked())
         mode_on  |= S_ISVTX;
      else
         mode_off ^= S_ISVTX;

   if ( recursive_v->isChecked() && !( mode_off & S_IXUSR) )
   {
      QMessageBox::warning( NULL, "Recursion error", "You cannot remove execute permissions\nrecursively for directory owner.", QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape, 0);
   }
   else
   {
      if(dir->first(fname)){
         do{
            myChmod(fname, mode_on, mode_off, recursive_v->isChecked());
         }while(dir->next(fname));
      }

      this->close();
   }

   return;
}

void Chmod::myChmod(const char *file, mode_t mode_on, mode_t mode_off, bool recursive)
{
   char subdir[4096];
   Directory d;
   mode_t new_mode;
   struct stat sb;

   if(lstat(file, &sb) == 0){

      if ( recursive && S_ISDIR(sb.st_mode) )
      {
         d.loadDir(file);
         if(d.first(subdir, &sb))
         {
            do
            {
               QString path;
               path.sprintf("%s/%s", file, subdir);
               myChmod(path.latin1(), mode_on, mode_off, recursive);
            } while( d.next(subdir, &sb) );
         }
      }

      new_mode = ((sb.st_mode | mode_on) & mode_off);
      if(chmod(file, new_mode) != 0){
         QMessageBox::warning( NULL, file, QString(strerror(errno)) , QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape, 0);
      }
   }
   else{
      QMessageBox::warning( NULL, file, QString(strerror(errno)) , QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape, 0);
   }

   return;
}

void Chmod::clear(){
   files      = 0;
   min_mode   = 0;
   max_mode   = 0;

   changed_ur = 0;
   changed_uw = 0;
   changed_ux = 0;
   changed_gr = 0;
   changed_gw = 0;
   changed_gx = 0;
   changed_or = 0;
   changed_ow = 0;
   changed_ox = 0;
   changed_su = 0;
   changed_sg = 0;
   changed_so = 0;
}

void Chmod::change_su(int s){
   if(s == QButton::NoChange)
      changed_su = 0;
   else
      changed_su = 1;
}

void Chmod::change_sg(int s){
   if(s == QButton::NoChange)
      changed_sg = 0;
   else
      changed_sg = 1;
}

void Chmod::change_so(int s){
   if(s == QButton::NoChange)
      changed_so = 0;
   else
      changed_so = 1;
}

void Chmod::change_ur(int s){
   if(s == QButton::NoChange)
      changed_ur = 0;
   else
      changed_ur = 1;
}

void Chmod::change_uw(int s){
   if(s == QButton::NoChange)
      changed_uw = 0;
   else
      changed_uw = 1;
}

void Chmod::change_ux(int s){
   if(s == QButton::NoChange)
      changed_ux = 0;
   else
      changed_ux = 1;
}

void Chmod::change_gr(int s){
   if(s == QButton::NoChange)
      changed_gr = 0;
   else
      changed_gr = 1;
}

void Chmod::change_gw(int s){
   if(s == QButton::NoChange)
      changed_gw = 0;
   else
      changed_gw = 1;
}

void Chmod::change_gx(int s){
   if(s == QButton::NoChange)
      changed_gx = 0;
   else
      changed_gx = 1;
}

void Chmod::change_or(int s){
   if(s == QButton::NoChange)
      changed_or = 0;
   else
      changed_or = 1;
}

void Chmod::change_ow(int s){
   if(s == QButton::NoChange)
      changed_ow = 0;
   else
      changed_ow = 1;
}

void Chmod::change_ox(int s){
   if(s == QButton::NoChange)
      changed_ox = 0;
   else
      changed_ox = 1;
}

