/* chmod.h - chmod dialog window
 * 
 *  Bronco File Manager
 * 
 *  Copyright (c) 2000, 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#ifndef CHOMD_H
#define CHOMD_H

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>
#include <time.h>
#include <qwidget.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qmessagebox.h>
#include "llist.h"

class Chmod : public QWidget {
      Q_OBJECT

   public:
      Chmod();
      virtual ~Chmod();

      void hide();
      int addFile(const char *);
      int addFile(const char *, const struct stat *statbuf);
      void clear(void);

   signals:
      void closeMe(Chmod *);

   public slots:
      void show(void);
      void changeMode(void);
      void change_ur(int);
      void change_uw(int);
      void change_ux(int);
      void change_gr(int);
      void change_gw(int);
      void change_gx(int);
      void change_or(int);
      void change_ow(int);
      void change_ox(int);
      void change_su(int);
      void change_sg(int);
      void change_so(int);

   protected:

      void myChmod(const char *, mode_t, mode_t, bool);

      int files;

      int changed_ur;
      int changed_uw;
      int changed_ux;
      int changed_gr;
      int changed_gw;
      int changed_gx;
      int changed_or;
      int changed_ow;
      int changed_ox;
      int changed_su;
      int changed_sg;
      int changed_so;

      mode_t max_mode;
      mode_t min_mode;

      QGridLayout *grid;

      QLabel *user_l;
      QLabel *other_l;
      QLabel *group_l;
      QLabel *recursive_l;

      QLabel *read_l;
      QLabel *write_l;
      QLabel *exec_l;
      QLabel *sticky_l;

      QCheckBox *user_r;
      QCheckBox *user_w;
      QCheckBox *user_x;
      QCheckBox *group_r;
      QCheckBox *group_w;
      QCheckBox *group_x;
      QCheckBox *other_r;
      QCheckBox *other_w;
      QCheckBox *other_x;
      QCheckBox *sticky_u;
      QCheckBox *sticky_o;
      QCheckBox *sticky_g;

      QCheckBox *recursive_v;

      QPushButton *change;
      QPushButton *cancel;

      LinkedList *dir;
};

#endif // CHOMD_H
