/* direct.h - Class for handling a directory listing
 * 
 *  Bronco File Manager
 * 
 *  Copyright (c) 2000, 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#ifndef DIRECT_H
#define DIRECT_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <errno.h>

#define SORT_ASCII   1
#define SORT_ALPHA   2
#define SORT_MTIME   3
#define SORT_ATIME   4
#define SORT_SIZE    5
#define SORT_PERM    6
#define SORT_MASK    255
#define DIRS_FIRST   256
#define SHOW_DOTS    512
#define SHOW_IMPLIED 1024
#define DIRS_ONLY    2048
#define DIR_REVERSE  4096
#define SORT_REVERSE 65536

#define S_ISSDIR(mode) (((mode) & S_IFMT) == (S_IFDIR | S_IFLNK))

struct direct {
   char *name;
   struct stat dirdata;
   struct direct *next;
   struct direct *prev;
};

class Directory{

   public:
      Directory();
      virtual ~Directory();

      int add(const char *, struct stat *);
      int add(const char *);
      int find(const char *, struct stat *);
      int first(char *, struct stat *);
      int next(char *, struct stat *);
      int prev(char *, struct stat *);
      int print();
      int size();
      int clear();
      int setSorting(int);
      int loadDir(const char *);
      int sort();

      int errorNumber();
      const char * errorString();

   protected:
      int compare(struct direct *, struct direct *);

      struct direct * top;
      struct direct * cur;

      const char * m_error;
      int m_errno;
      int sort_order;
};

int isDir(char *);

#endif // DIRECT_H
