/* diritem.cpp - QListViewItem widget for DirTree
 * 
 *  Bronco File Manager
 * 
 *  Copyright (c) 2000, 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#include "bronco.h"
#include "prefs.h"
#include "direct.h"
#include "diritem.h"

void DirItem::setOpen(bool o){
   if(o){
      if(!this->firstChild()){
         if(!addDir(this, this->fullPath())) setExpandable(FALSE);
      }

   }
   
   QListViewItem::setOpen(o);
   return;
}

DirItem::DirItem(QListView *parent = NULL, QListViewItem *after = NULL, const char *name = NULL, const char *path = "") : QListViewItem(parent, after, name){
   setFullPath(path);
   setExpandable(TRUE);
   return;
}

DirItem::DirItem(QListViewItem *parent = NULL, QListViewItem *after = NULL, const char *name = NULL, const char *path = "") : QListViewItem(parent, after, name){
   setFullPath(path);
   setExpandable(TRUE);
   return;
}

int DirItem::addDir(DirItem *parent, char const *dir){
   Directory newdir;

   struct stat statbuf;
   int count = 0;

   char dirbuf[4096];

   //newdir.setSorting(prefs.asciiSort() | prefs.getDirTreeOpts());
   newdir.setSorting(prefs.asciiSort() | prefs.getDirTreeOpts() | DIRS_ONLY);
   newdir.loadDir(dir);

   DirItem *temp = NULL;
   char name[80];

   if(newdir.first(name, &statbuf)){
      do{
         if(strcmp(name, ".") && strcmp(name, "..")){
            if(strcmp(dir, "/")){
               snprintf(dirbuf, sizeof(dirbuf), "%s/%s", dir, name);
            }
            else{
               snprintf(dirbuf, sizeof(dirbuf), "/%s", name);
            }
            temp = new DirItem(parent, temp, name, dirbuf);
            count++;
         }
      }while(newdir.next(name, &statbuf));
   }

   return(count);
}

int DirItem::setFullPath(const char *path){
   strncpy(full_path, path, sizeof(full_path));
   full_path[sizeof(full_path) - 1] = 0;
   return(0);
}

const char * DirItem::fullPath(void){
   return((const char *) full_path);
}

void DirItem::activate(void){
   /* emit activated(this); */
}

