/* dirlist.cpp - QListView widget for a single directory listing
 * 
 *  Bronco File Manager
 * 
 *  Copyright (c) 2000, 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <qapplication.h>
#include <qwidget.h>
#include <qlistview.h>
#include <qclipboard.h>
#include <qmessagebox.h>
#include <qinputdialog.h>
#include <qmenudata.h>
#include <time.h>
#include <unistd.h>
#include <ctype.h>
#include <qregexp.h>
#include <qtooltip.h>
#include <qfiledialog.h>
#include "bronco.h"
#include "dirlist.h"
#include "prefs.h"
#include "prefwin.h"
#include "llist.h"
#include "openwith.h"
#include "find.h"
#include "findwin.h"
#include "grepwin.h"
#include "filewin.h"
#include "chmod.h"
#include "tarwin.h"
#include "tardlg.h"
#include "execbr.h"
#include "mainwin.h"

static const char* link_xpm[]={
    "17 12 4 1",
    "c c #98a6ff", "o c #0019ff", "@ c #000000", ". c None",
    "..................",
    "..................",
    "..................",
    "............c.....",
    "............oc....",
    ".....oooooooooc...",
    ".....ooooooooooc..",
    ".....oooooooooo...",
    "............oo....",
    "............o.....",
    "..................",
    ".................."};

static const char* folder_link_xpm[]={
    "17 12 14 1",
    "i c #eeeee6", "j c #eeeade", "u c #dddd00", "e c #b4ba9c",
    "a c #b4b69c", "h c #a5a48b", "g c #7b796a", "q c #6a6d62",
    "c c #98a6ff", "o c #0019ff", "* c #52554a", "# c #313029",
    "@ c #000000", ". c None",
    "....@@@@..........",
    "...@ahah@.........",
    "..@ahahah@@@@@@@..",
    "..@ijijijiijcieu@.",
    "..@jeueueueuocex@.",
    "..@iuooooooooocg@.",
    "..@jeooooooooooc@.",
    "..@iuooooooooooq@.",
    "..@jeueueueuooeg@.",
    "..@iueueueueoeuq@.",
    "..@i*#*#*#*#*#*#@.",
    "...@@@@@@@@@@@@@.."};

static const char* folder_xpm[]={
    "17 12 12 1",
    "i c #eeeee6", "j c #eeeade", "u c #dddd00", "e c #b4ba9c",
    "a c #b4b69c", "h c #a5a48b", "g c #7b796a", "q c #6a6d62",
    "* c #52554a", "# c #313029", "@ c #000000", ". c None",
    "....@@@@..........",
    "...@ahah@.........",
    "..@ahahah@@@@@@@..",
    "..@ijijijiijjiuu@.",
    "..@jeueueueueuex@.",
    "..@iueueueueueug@.",
    "..@jeueueueueueg@.",
    "..@iueueueueueuq@.",
    "..@jeueueueueueg@.",
    "..@iueueueueueuq@.",
    "..@i*#*#*#*#*#*#@.",
    "...@@@@@@@@@@@@@.."};

/* 
static const char* open_folder_xpm[]={
    "17 12 10 1",
    "i c #eeeee6",
    "j c #eeeade",
    "u c #dddd00",
    "e c #b4ba9c",
    "a c #b4b69c",
    "h c #a5a48b",
    "* c #52554a",
    "# c #313029",
    "@ c #000000",
    ". c None",
    "....@@@@..........",
    "...@ahah@.........",
    "..@ahahah@@@@@@@..",
    "..@hahahahahahah@.",
    ".@@@@@@@@@@@haha@.",
    "@jijijijijijj@ah@.",
    "@ieueueueueue@ha@.",
    ".@jeueueueueue@h@.",
    ".#iueueueueueu@a@.",
    "..#iueueueueueu@@.",
    "..@i*#*#*#*#*#*@@.",
    "...@@@@@@@@@@@@@.."};
*/

   /*
class MyMenuItem : public QCustomMenuItem
{
   public:
      MyMenuItem( const QString& s, const QFont& f )
         : string( s ), font( f ){};
      virtual ~MyMenuItem(){}

      void paint( QPainter* p, const QColorGroup&, bool, bool, int x, int y, int w, int h )
      {
         p->setFont ( font );
         p->drawText( x, y, w, h, AlignLeft | AlignVCenter | ShowPrefix | DontClip, string );
      }

      QSize sizeHint()
      {
         return QFontMetrics( font ).size( AlignLeft | AlignVCenter | ShowPrefix | DontClip,  string );
      }
   private:
      QString string;
      QFont font;
};
*/


DirList::DirList(QWidget *parent=NULL, char *name=NULL) : QListView(parent, name, WDestructiveClose){
   int x, y;

   m_launchMenu = NULL;
   m_launchMenu = getLaunchMenu();
   
   /* Basic init */
   setMinimumSize(385, 260);
   setSelectionMode(Extended);

   sort_policy    = prefs.getDirListOpts() | (prefs.asciiSort());

   char family[64];
   int fsize=10;
   prefs.getFont(family, &fsize);
   font = new QFont(family, fsize, QFont::Normal, FALSE);
   setFont(*font);

   currentDir.clear();

   /* Columns */
   addColumn("", -1);
   addColumn("name", -1);
   addColumn("size", -1);
   addColumn("perm", -1);
   addColumn("modified", -1);

   setSorting(-1, false);          /* We sort manually */
   setAllColumnsShowFocus( true );

   setColumnWidthMode(0, Manual);
   setColumnWidthMode(1, Manual);
   setColumnWidthMode(2, Manual);
   setColumnWidthMode(3, Manual);
   setColumnWidthMode(4, Manual);

   (this->header())->setMovingEnabled( false );

   connect(this->header(), SIGNAL(sectionClicked(int)),  
         this, SLOT(sortColumn(int)));
   connect(this, SIGNAL(doubleClicked(QListViewItem *)), 
         this, SLOT(openItem(QListViewItem *)));

   if ( parent && parent->parent())
   {
      /* Isn't _this_ nasty?  MainWin should be a singleton, 
       * I have no ambition */
      connect(parent->parent(), SIGNAL(refreshDirectory(const QString &)), 
            this, SLOT(refreshDirectory(const QString &)));
   }
   else
   {
      connect(mainwin, SIGNAL(refreshDirectory(const QString &)), 
            this, SLOT(refreshDirectory(const QString &)));
   }

   /* this->header()->setClickEnabled(FALSE); */

   /* Context menu init */
   context_menu = new QPopupMenu(this, "");

//   QFont f = context_menu->font();
//   f.setBold( TRUE );
//   int boldID = context_menu->insertItem( new MyMenuItem( "Open", f ) );
//   context_menu->connectItem( boldID, this, SLOT(openItem()) );

   context_menu->insertItem("Open",             this, SLOT(openItem()));
   context_menu->insertItem("Open with...",     this, SLOT(openItemWith()));
   context_menu->insertItem("Custom commands",  m_launchMenu);

   context_menu->insertSeparator();
   context_menu->insertItem("Copy",             this, SLOT(copyCommand()));
   context_menu->insertItem("Move",             this, SLOT(moveCommand()));
   context_menu->insertItem("Paste",            this, SLOT(pasteCommand()));
   context_menu->insertItem("Rename...",        this, SLOT(renameCommand()));
   context_menu->insertItem("Delete",           this, SLOT(deleteCommand()));

   context_menu->insertSeparator();
   context_menu->insertItem("Copy to...",       this, SLOT(copyTo()));
   context_menu->insertItem("Move to...",       this, SLOT(moveTo()));

   context_menu->insertSeparator();
   context_menu->insertItem("Properties...",    this, SLOT(properties()));
   context_menu->insertItem("Change permissions...",   this, SLOT(chmod()));

   context_menu->insertSeparator();
   QPopupMenu * new_menu = new QPopupMenu(this, "");
   new_menu->insertItem("New window",           this, SLOT(newWindow()));
   new_menu->insertItem("New shell",            this, SLOT(launchShell()));
   new_menu->insertSeparator();
   new_menu->insertItem("New directory...",     this, SLOT(newDirectory()));
   new_menu->insertItem("New file...",          this, SLOT(newFile()));
   new_menu->insertItem("New archive...",       this, SLOT(createTar()));
   context_menu->insertItem("New",  new_menu);

   QPopupMenu * select_menu = new QPopupMenu(this, "");
   select_menu->insertItem("Select all",        this, SLOT(selectAll()));
   select_menu->insertItem("Unselect all",      this, SLOT(unSelectAll()));
   select_menu->insertItem("Select mask...",    this, SLOT(selectMask()));
   context_menu->insertItem("Select",  select_menu);

   QPopupMenu * find_menu = new QPopupMenu(this, "");
   find_menu->insertItem("Find",                this, SLOT(findWindow()));
   find_menu->insertItem("Grep",                this, SLOT(grepWindow()));
   context_menu->insertItem("Find",  find_menu);

   /* Shortcuts */
   accel = new QAccel(this);
   accel->connectItem(accel->insertItem(Key_M+CTRL),       this, SLOT(mount()));
   accel->connectItem(accel->insertItem(Key_A+CTRL),       this, SLOT(selectAll()));
   accel->connectItem(accel->insertItem(Key_A+CTRL+SHIFT), this, SLOT(unSelectAll()));
   accel->connectItem(accel->insertItem(Key_Menu),         this, SLOT(contextMenu()));
//   accel->connectItem(accel->insertItem(Key_Up+CTRL),      this, SLOT(upDir()));

   accel->connectItem(accel->insertItem(Key_N+CTRL),       this, SLOT(newWindow()));

   accel->connectItem(accel->insertItem(Key_C+CTRL),       this, SLOT(copyCommand()));
   accel->connectItem(accel->insertItem(Key_X+CTRL),       this, SLOT(moveCommand()));
   accel->connectItem(accel->insertItem(Key_V+CTRL),       this, SLOT(pasteCommand()));
   accel->connectItem(accel->insertItem(Key_Delete),       this, SLOT(deleteCommand()));
   accel->connectItem(accel->insertItem(Key_R+CTRL),       this, SLOT(renameCommand()));
   accel->connectItem(accel->insertItem(Key_F5),           this, SLOT(refresh()));
   accel->connectItem(accel->insertItem(Key_F+CTRL),       this, SLOT(findWindow()));
   accel->connectItem(accel->insertItem(Key_G+CTRL),       this, SLOT(grepWindow()));
   accel->connectItem(accel->insertItem(Key_S+CTRL),       this, SLOT(launchShell()));

   accel->connectItem(accel->insertItem(Key_Q+CTRL),       qApp, SLOT(quit()));
   accel->connectItem(accel->insertItem(Key_K+CTRL),       this, SLOT(selectMask()));
   accel->connectItem(accel->insertItem(Key_T+CTRL),       this, SLOT(createTar()));
   if (parent == NULL) accel->connectItem(accel->insertItem(Key_W+CTRL),       this, SLOT(close()));

   /* ListView behavior */
   QScrollView::setVScrollBarMode(AlwaysOn);
   setMultiSelection(TRUE);

   if (parent == NULL){
      prefs.getFloatSize(&x, &y);
      this->resize(x, y);
   }

   strcpy(full_path, "/");
   strcpy(last_path, "/");

   m_doRefresh = true;
   
   return;
}

DirList::~DirList()
{
}

void DirList::resizeEvent(QResizeEvent *e){
   setColumnWidth(0, 20);
   setColumnWidth(1, width() - 285);  /* All but 'name' are fixed width */
   setColumnWidth(2, 75);
   setColumnWidth(3, 90);
   setColumnWidth(4, 80);

   if (this->parent() == NULL)
   {
      prefs.setFloatSize(this->width(), this->height());
      prefs.save();
   }

   QListView::resizeEvent(e);
   triggerUpdate();
   return;
}

int DirList::reLoadDir(const char *dir)
{
   m_doRefresh = true;
   return loadDir(dir);
}

int DirList::loadDir(const char *dir){
   QListViewItem *temp = NULL;
   bool restore_old = false;

   QString     old_current = "";
   QStringList old_list;
   int old_x = 0;
   int old_y = 0;

   if(strcmp(dir, full_path) == 0)
   {
      if ( ! m_doRefresh )
      {
         return 0;
      }
      restore_old = true;

      if ( this->currentItem()) 
      {
         old_current = (this->currentItem()->text(1)).latin1();
      }
      old_x       = this->contentsX();
      old_y       = this->contentsY();
      QListViewItem *i = this->firstChild();

      while ( i )
      {
         if( i->isSelected() )
         {
            old_list << i->text(1);
         }
         i = i->nextSibling();
      }
   }
   else
   {
      strcpy(last_path, full_path);
   }

   m_doRefresh = false;

   keybuf[0]  = '\0';
   keybuf_cur = 0;

   setUpdatesEnabled(FALSE);
   strcpy(full_path, dir);
   setSorting(-1, FALSE);

   clear();
   currentDir.clear();
   currentDir.setSorting(sort_policy);

   if(strncmp("find", full_path, 4) == 0){
      LinkedList list;
      char glob[1024];
      char path[1024];
      int n, t, len;

      len = strlen(full_path);
      n = 4;

      while((n<len) && full_path[n]==' ')
         n++;

      t = 0;
      while((n<len) && full_path[n]!=' ')
         glob[t++] = full_path[n++];
      glob[t] = 0;
      
      while((n<len) && full_path[n]==' ')
         n++;

      t = 0;
      while((n<len))
         path[t++] = full_path[n++];
      path[t] = 0;

//      findFile(path, glob, NULL);
      /*
      findFile(path, glob, &list, NULL);
      if(list.first(path)){
         do{
            currentDir.add(path);
         } while(list.next(path));

         showContents();
      }
      else {
         temp = new QListViewItem(this, NULL, "", "No files found");
      }
      */
   }

   else if(strncmp("tar", full_path, 3) == 0){
      char file[1024];
      char path[1024];
      int n, t, len;
      struct stat sbuf;

      char *args[2];
      char cmd[1024];
      char opts[10];
      char contents[1024];
      char *s;
      FILE *fp;


      len = strlen(full_path);
      n = 4;

      while((n<len) && full_path[n]==' ')
         n++;

      t = 0;
      while((n<len) && full_path[n]!=':')
         file[t++] = full_path[n++];
      file[t] = 0;
      
      while((n<len) && (full_path[n]==':' || full_path[n]=='/'))
         n++;

      t = 0;
      while((n<len))
         path[t++] = full_path[n++];
      path[t] = 0;

      sbuf.st_mode  = 0;
      sbuf.st_size  = 0;
      sbuf.st_atime = 0;
      sbuf.st_mtime = 0;
      sbuf.st_ctime = 0;

      strcpy(opts, "-tzf");
      if(strncmp(&file[strlen(file)-4], ".tar", 4) == 0)
         strcpy(opts, "-tf");
      if(strncmp(&file[strlen(file)-4], ".tbz", 4) == 0)
         strcpy(opts, "-tjf");
      if(strncmp(&file[strlen(file)-5], ".tbz2", 5) == 0)
         strcpy(opts, "-tjf");
      if(strncmp(&file[strlen(file)-6], ".tar.Z", 6) == 0)
         strcpy(opts, "-tZf");
      if(strncmp(&file[strlen(file)-7], ".tar.bz", 7) == 0)
         strcpy(opts, "-tjf");
      if(strncmp(&file[strlen(file)-8], ".tar.bz2", 8) == 0)
         strcpy(opts, "-tjf");

      snprintf(cmd, sizeof(cmd), "tar %s", opts);   

      args[0] = file;
      args[1] = NULL;

      BroncoExec program;
      program.setPipeStdout( true );
      program.setPipeStderr( true );
      program.exec(cmd, (const char **) args);
      if( (fp = program.getPipeStdout()) != NULL) 
      {
         while( fgets(contents, sizeof(contents), fp) )
         {
            chomp(contents);
            if( (strlen(contents) != strlen(path)) && (strncmp(contents, path, strlen(path)) == 0) )
            {
               s = &contents[strlen(path)];

               if ( path[0] == '\0' || (*s) == '/' )
               {
                  while(*s == '/')
                  {
                     s++;
                  }

                  if(*s)
                  {
                     if(inCurrentDir(s))
                     {
                        sbuf.st_mode  = S_IFREG;
                           currentDir.add(s, &sbuf);
                     }
                     else if(isTarDir(s))
                     {
                        sbuf.st_mode  = S_IFDIR;
                        if ( strcmp(s, ".") != 0 && strcmp(s, "..") != 0 )
                           currentDir.add(s, &sbuf);
                     }
                  }
               }
            }
         }
         showContents();
      }
      else
      {
         QMessageBox::warning( NULL, file, strerror(errno), QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape, 0);
      }
   }
   else{
      if(currentDir.loadDir(dir) >= 0){
         showContents();
      }
      else{
         temp = new QListViewItem(this, NULL, "", currentDir.errorString());
      }
   }

   if ( parent() == NULL )
   {
      setCaption(full_path);
   }

   if ( restore_old )
   {
      QListViewItem *i = NULL;

      for( QStringList::Iterator it = old_list.begin(); it != old_list.end(); it++ )
      {
         i = this->firstChild();

         while ( i )
         {
            if ( i->text(1) == *it )
            {
               i->setSelected( true );
               break;
            }
            if ( ! old_current.isEmpty() && old_current == i->text(1) )
            {
               this->setCurrentItem( i );
               old_current = "";
            }

            i = i->nextSibling();
         }
      }

      if ( ! old_current.isEmpty() )
      {
         i = this->firstChild();
         
         while ( i )
         {
            if ( i->text(1) == old_current)
            {
               setCurrentItem( i );
               break;
            }

            i = i->nextSibling();
         }
      }

      if ( this->currentItem() )
      {
         setContentsPos(old_x, old_y);
      }
   }
   else
   {
      temp = this->firstChild();
      if(temp)
      {
         this->setCurrentItem(temp);
         ensureItemVisible( temp );
      }
   }

   setUpdatesEnabled(TRUE);
   triggerUpdate();

   emit directoryChanged( QString(full_path) );

   return(0);
}

void DirList::showContents(void){
   QListViewItem *temp = NULL;

   char name[2048];
   char perm[12];
   char size[20];
   char date[30];
   struct stat statbuf;

   struct tm tmcur;
   struct tm *tmbuf;

   time_t t;
   t = time(NULL);
   tmbuf = localtime(&t);
   memcpy(&tmcur, tmbuf, sizeof(struct tm));

   clear();
   setSorting(-1, FALSE);

   if(currentDir.first(name, &statbuf)){
      do{
         if(statbuf.st_mode & (S_IRWXU | S_IRWXG | S_IRWXO) || statbuf.st_mtime || statbuf.st_size)
         {
            snprintf(perm, sizeof(perm), "%c%c%c %c%c%c %c%c%c",
               (S_IRUSR & statbuf.st_mode) ? 'r' : '-',
               (S_IWUSR & statbuf.st_mode) ? 'w' : '-',
               (S_IXUSR & statbuf.st_mode) 
                  ? ( S_ISUID & statbuf.st_mode ? 's' : 'x' ) 
                  : ( S_ISUID & statbuf.st_mode ? 'S' : '-' ),

               (S_IRGRP & statbuf.st_mode) ? 'r' : '-',
               (S_IWGRP & statbuf.st_mode) ? 'w' : '-',
               (S_IXGRP & statbuf.st_mode) 
                  ? ( S_ISGID & statbuf.st_mode ? 's' : 'x' ) 
                  : ( S_ISGID & statbuf.st_mode ? 'S' : '-' ),

               (S_IROTH & statbuf.st_mode) ? 'r' : '-',
               (S_IWOTH & statbuf.st_mode) ? 'w' : '-',
               (S_IXGRP & statbuf.st_mode) 
                  ? ( S_ISVTX & statbuf.st_mode ? 't' : 'x' ) 
                  : ( S_ISVTX & statbuf.st_mode ? 'T' : '-' ));

            snprintf(size, sizeof(size), "%lu", (long unsigned) statbuf.st_size);

            tmbuf = localtime(&statbuf.st_mtime);
            if(tmbuf->tm_year == tmcur.tm_year && tmbuf->tm_mon == tmcur.tm_mon && tmbuf->tm_mday == tmcur.tm_mday){
               snprintf(date, sizeof(date), "%02d:%02d:%02d", tmbuf->tm_hour, tmbuf->tm_min, tmbuf->tm_sec);
            }
            else{
               if(prefs.getFourDigitYear()){
                  if(prefs.getDayFirst())
                     snprintf(date, sizeof(date), "%02d/%02d/%04d", tmbuf->tm_mday, tmbuf->tm_mon+1, tmbuf->tm_year + 1900);
                  else
                     snprintf(date, sizeof(date), "%02d/%02d/%04d", tmbuf->tm_mon+1, tmbuf->tm_mday, tmbuf->tm_year + 1900);
               }
               else{
                  if(prefs.getDayFirst())
                     snprintf(date, sizeof(date), "%02d/%02d/%02d", tmbuf->tm_mday, tmbuf->tm_mon+1, tmbuf->tm_year%100);
                  else
                     snprintf(date, sizeof(date), "%02d/%02d/%02d", tmbuf->tm_mon+1, tmbuf->tm_mday, tmbuf->tm_year%100);
               }
            }
         }
         else
         {
            strcpy(perm, "");
            strcpy(size, "");
            strcpy(date, "");
         }

         temp = new QListViewItem(this, temp, "", name, size, perm, date);
         if(S_ISDIR(statbuf.st_mode)){
            temp->setPixmap(0, folder_xpm);
         }
         else if(S_ISSDIR(statbuf.st_mode)){
            temp->setPixmap(0, folder_link_xpm);
         }
         else if(S_ISLNK(statbuf.st_mode)){
            temp->setPixmap(0, link_xpm);
         }
      } while(currentDir.next(name, &statbuf));
   }
   setCurrentItem(this->firstChild());

   return;
}

void DirList::changeDir(DirItem *item){
   loadDir(item->fullPath());
}

void DirList::sortColumn(int t){
   int last_policy = sort_policy;
   int reverse     = 0;

   switch(t){
      case 0:
         return;
         break;
      case 1:
         sort_policy = (prefs.asciiSort());
         break;
      case 2:
         sort_policy = SORT_SIZE;
         break;
      case 3:
         sort_policy = SORT_PERM;
         break;
      case 4:
         sort_policy = SORT_MTIME;
         break;
      default:
         break;
   }

   if((last_policy & SORT_MASK) == (sort_policy & SORT_MASK)){
      reverse = (last_policy ^ SORT_REVERSE);
   }
   sort_policy = sort_policy | prefs.getDirListOpts() | reverse;

   currentDir.setSorting(sort_policy);
   showContents();
}

void DirList::sortChange(int new_sort){
   if(sort_policy != new_sort){
      sort_policy = new_sort;
      m_doRefresh = true;
      loadDir(full_path);
   }
}

void DirList::selectAll(void){
   QListViewItem *d;
   setUpdatesEnabled(FALSE);
   clearSelection();
   for(d = firstChild(); d; d = d->nextSibling()){
      if(d->text(1) != "." && d->text(1) != "..")
      {
         setSelected(d, TRUE);
      }
   }
   setUpdatesEnabled(TRUE);
   triggerUpdate();
}

void DirList::unSelectAll(void){
   clearSelection();
   triggerUpdate();
}

void DirList::selectMask(void){
   QListViewItem *item;
   QString mask;

   item = currentItem();

   if ( (item->text(1)).findRev('.') >= 0)
   {
      mask = item->text(1);
      mask = mask.right( mask.length() - mask.findRev('.'));
      mask.prepend("*");
   }
   else
   {
      mask = "*.";
   }

   bool ok;
   QString glob = QInputDialog::getText( "Select mask", "Enter file mask to select on:", QLineEdit::Normal, mask, &ok, this );
//   QString glob = QInputDialog::getText( "Select mask", "Enter file mask to select on:", mask, &ok, this );
   if(ok != TRUE)
      return;

   setUpdatesEnabled(FALSE);
   item = firstChild();

   int p = getFindPrefs();
   setFindPrefs(0);

//   clearSelection();
   do {
      if (match(glob, item->text(1))) {
         item->setSelected(TRUE);
         setCurrentItem(item);
      }
   } while((item = item->nextSibling()));

   setFindPrefs(p);
   
   setUpdatesEnabled(TRUE);
   triggerUpdate();
   return;
}

#if (QT_VERSION >= 300 )

void DirList::contentsContextMenuEvent( QContextMenuEvent *e)
{
   QPoint p = e->pos();
   p.setY(p.y() - contentsY());
   QListViewItem *item = itemAt(p);
   if(item){
      setCurrentItem(item);
      if(!item->isSelected())
      {
         clearSelection();
         item->setSelected(TRUE);
      }
      triggerUpdate();
   }
   context_menu->popup(e->globalPos());
}

#else

void DirList::contentsMousePressEvent(QMouseEvent *e){
   if(e->button() == RightButton && strncmp("tar", full_path, 3)){
      QPoint p = e->pos();
      p.setY(p.y() - contentsY());
      QListViewItem *item = itemAt(p);
      if(item){
         setCurrentItem(item);
         if(!item->isSelected())
         {
            clearSelection();
            item->setSelected(TRUE);
         }
         triggerUpdate();
      }
      context_menu->popup(e->globalPos());
   }
   else{
      QListView::contentsMousePressEvent(e);
   }
}

#endif

void DirList::contentsMouseReleaseEvent(QMouseEvent *e){
   if(e->button() == MidButton){
      upDir();
   }
   else{
      QListView::contentsMouseReleaseEvent(e);
   }
}

void DirList::contextMenu(void){
   context_menu->popup(mapToGlobal(QPoint(5, currentItem()->itemPos() + header()->height())));
}

void DirList::deleteCommand(void){
   if ( full_path[0] == '/' || strncmp(full_path, "find", 4) == 0 )
   {
      listToClip();
      doDelete();
   }
}

void DirList::copyCommand(void){
   if ( full_path[0] == '/' || strncmp(full_path, "find", 4) == 0 )
   {
      move_operation = 0;
      listToClip();
   }
}

void DirList::moveCommand(void){
   if ( full_path[0] == '/' )
   {
      move_operation = 1;
      listToClip();
   }
}

void DirList::renameCommand(void){
   if ( full_path[0] == '/' )
   {
      bool ok = FALSE;
      if(this->currentItem()){
         QString file  = (this->currentItem())->text(1);
         QString input = QInputDialog::getText( "Rename file", "Rename to:", QLineEdit::Normal, file, &ok, this );
         if(ok == TRUE)
         {
            file.prepend("/");
            file.prepend(full_path);
            input.prepend("/");
            input.prepend(full_path);
            rename((const char *) file, (const char *) input);
            refresh();
         }
      }
   }
}

void DirList::pasteCommand(void){
   if ( full_path[0] == '/' )
   {
      switch(move_operation){
         case 0:
            doCopy();
            break;
         case 1:
            doMove();
            break;
         case -1:
            QMessageBox::warning( NULL, "Bronco error", "You can't paste until you copy or move" , QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape, 0);
            break;
         default:
            QMessageBox::warning( NULL, "Bronco error", "move_operation has unknown value.\nHow'd you do that!?" , QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape, 0);
            break;
      }
   }
}

void DirList::doMove(void){
   FileWin *win;
   win = new FileWin(NULL, "Copy file(s)...");
   win->setDest(full_path);
   if( ! win->clipMove())
   {
     win->goAway();
   }

   move_operation = -1;
   clipDir.clear();
}

void DirList::doCopy(void){
   FileWin *win;
   win = new FileWin(NULL, "Copy file(s)...");
   win->setDest(full_path);
   if( ! win->clipCopy() )
   {
      win->goAway();
   }
}

void DirList::doDelete(void){
   if ( full_path[0] == '/' || strncmp(full_path, "find", 4) == 0)
   {
      FileWin *win;
      win = new FileWin(NULL, "Delete file(s)...");
      if ( full_path[0] == '/' )
         win->setDest(full_path);
      else
         win->setDest("/");

      if( ! win->clipDelete() )
      {
         win->goAway();
      }

      move_operation = -1;
      clipDir.clear();
   }
}

void DirList::listToClip(void){
   QListViewItem *item;
   QString clipText = "";

   clipDir.clear();

   if ( full_path[0] == '/' )
   {
      strcpy(clip_dir, full_path);
   }
   else
   {
      strcpy(clip_dir, "/");
   }

   for(item=firstChild(); item; item = item->nextSibling()){
      if(item->isSelected() && (item->text(1) != "." && item->text(1) != "..")){
         clipText += item->text(1);
         clipText += "\n";

         clipDir.add( item->text(1) );
      }
   }

   QClipboard *clip = QApplication::clipboard();
   clip->setText(clipText);
   if(clipText.isEmpty())
   {
      move_operation = -1;
   }
}

void DirList::openItem(void){
   openItem(currentItem());
   return;
}


void DirList::openItem(QListViewItem *item){
   struct stat statbuf;
   char progname[512];
   LinkedList globlist;

   if(item == NULL)
      return;

   if(currentDir.find((const char *) item->text(1), &statbuf)){
      if(S_ISDIR(statbuf.st_mode) || S_ISSDIR(statbuf.st_mode)){
         QString dirbuf;
         if(strcmp(".", item->text(1)) == 0){
            return;
         }
         if(strcmp("..", item->text(1)) == 0){
            upDir();
            return;
         }

         if(((const char *) item->text(1))[0] == '/'){
            dirbuf = item->text(1);
         }
         else{
            dirbuf = full_path;
            if(strcmp(full_path, "/")){
               dirbuf += "/";
            }
            dirbuf += item->text(1);
         }

         if ( full_path[0] == '/' || (strncmp(full_path, "tar", 3) == 0) )
            loadDir( dirbuf.latin1());
         else if ( strncmp(full_path, "find", 4 ) == 0)
            newWindow( dirbuf.latin1() );
      }
      else if(isTarFile((const char *) item->text(1)))
      {
         if(strncmp(full_path, "tar", 3) == 0)
            return;
         tarWindow();
      }
      else
      {
         const char *prog;

         if(strncmp(full_path, "tar", 3) == 0)
            return;

         item = firstChild();
         do
         {
            if(item->isSelected()){
               if((prog = prefs.getCommand((const char *) item->text(1))) != NULL){
                  globlist.add(prog);
               }
               else{
                  OpenWith p(NULL, "");
                  p.setCaption((const char *) item->text(1));

                  if(p.exec()){
                     QString base_command = p.getInput();

                     if ( ! base_command.isEmpty() )
                     {
                        QStringList arglist;
                        arglist << item->text(1);
                        openList( base_command, arglist );
                     }
                  }
               }
            }
         } while((item = item->nextSibling()));

         if(globlist.first(progname))
         {
            do
            {
               QString base_command = progname;
               QStringList arglist;

               item = firstChild();
               while ( item )
               {
                  if(item->isSelected()){
                     if((strcmp(progname, prefs.getCommand( (item->text(1)).latin1()))==0)){
                        arglist << item->text(1);
                     }
                  }

                  item = item->nextSibling();
               }

               openList( base_command, arglist );

            } while(globlist.next(progname));
         }
      }
   }
   else{
//      QMessageBox::warning( NULL, item->text(1), "Item not found", QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape, 0);
   }
}

void DirList::openItemWith(void){
   openItemWith(currentItem());
   return;
}

/* Note:  *item is completely disregarded (overwritten)
 * We use the selections and current item instead.
 */
void DirList::openItemWith(QListViewItem *i){
   QListViewItem *item;

   if(strncmp(full_path, "tar", 3) == 0)
      return;

   OpenWith p(NULL, "");
   p.setCaption("Open with...");

   if(p.exec())
   {
      QString     program = p.getInput();
      QStringList arglist;

      if ( ! program.isEmpty() )
      {
         prefs.setLastOpenWith( p.getInput() );
         prefs.save();

         item = firstChild();
         while ( item )
         {
            if(item->isSelected())
            {
               arglist << item->text(1);
            }

            item = item->nextSibling();
         }

         openList(program, arglist);
      }
   }
}

void DirList::upDir(void){
   int n;
   if(strcmp("/", full_path) == 0){
      return;
   }
   if(strncmp("find", full_path, 4) == 0){
      return;
   }
   if(strncmp("tar", full_path, 3) == 0){
      char *s = index(full_path, ':');
      s++;
      if( s[0] == '\0' )
         return;
   }

   char *path = strdup(full_path);

   for(n = strlen(path) - 1; n; n--){
      if(path[n] == '/'){
         path[n] = '\0';
         break;
      }
   }
   if(n == 0){
      path[1] = '\0';
   }

   loadDir(path);

   free(path);
   return;
}

void DirList::backDir(void){
   char *path = strdup(last_path);

   loadDir(path);

   free(path);
   return;
}

const char *DirList::getDir(void){
   return((const char *) full_path);
}

void DirList::newWindow(void){
   if ( full_path[0] == '/' )
   {
      newWindow(full_path);
   }
   if ( strncmp(full_path, "find", 4) == 0)
   {
      QListViewItem *item = this->currentItem();
      if ( item )
      {
         QString newdir = item->text(1);
         if ( newdir.findRev('/') > 1 )
         {
            newdir.truncate( newdir.findRev('/') );
            newWindow( newdir.latin1() );
         }
      }
   }
   
   return;
}

void DirList::findWindow(void){
   FindWin *f;
   f = new FindWin(NULL, "");

   if ( full_path[0] == '/' )
   {
      f->setDir(full_path);
   }
   else
   {
      f->setDir("/");
   }

   return;
}

void DirList::grepWindow(void){
   GrepWin *f;
   f = new GrepWin(NULL, "");

   if ( full_path[0] == '/' )
   {
      f->setDir(full_path);
   }
   else
   {
      f->setDir("/");
   }

   QListViewItem *item = this->firstChild();

   while ( item )
   {
      if ( item->isSelected() )
         f->addFile( item->text(1) );

      item = item->nextSibling();
   }

   f->show();

   return;
}

void DirList::tarWindow(void){
   TarWin *f;
   QString item;
   if ( full_path[0] == '/' )
   {
      item = tr("tar ") + tr(full_path) + tr("/") + this->currentItem()->text(1);
   }
   else if ( this->currentItem()->text(1).startsWith(QString("/")) )
   {
      item = tr("tar ") + this->currentItem()->text(1);
   }
   else
   {
      QMessageBox::warning( NULL, "Bronco", QString("Don't know how to open:\n") + this->currentItem()->text(1), QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape, 0);
   }
   f = new TarWin(item, NULL, "");

   return;
}

void DirList::newWindow(const char *p){
   DirList *temp = new DirList(NULL, "");
   char *path;
   
   path = strdup(p);

   for(int t=strlen(path)-1; t>1 && path[t]=='/'; t--){
      path[t] = '\0';
   }

   connect(mainwin->getPrefWin(), SIGNAL(preferencesChanged()), temp, SLOT(rebuildLaunchMenu()));
   connect(mainwin->getPrefWin(), SIGNAL(fontChanged()), temp, SLOT(changeFont()));

   temp->loadDir(path);
   temp->show();
   free(path);

   return;
}

void DirList::properties(void){
   int count;
   QString filename;
   PropWin *propwin = new PropWin();

   count = 0;
   QListViewItem *item = firstChild();

   while(item){
      if(item->isSelected()){
         if ( full_path[0] == '/' )
         {
            filename  = full_path;
            filename += "/";
            filename += item->text(1);
            propwin->addFile(filename);
            count++;
         } 
         else if ( strncmp(full_path, "find", 4) == 0)
         {
            filename  = item->text(1);
            propwin->addFile(filename);
            count++;
         }
      }

      item = item->nextSibling();
   }
   filename = full_path;

   if ( count )
   {
      propwin->show();
   }
   else
   {
      QMessageBox::warning( NULL, "Bronco", "No file(s) selected", QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape, 0);
      delete propwin;
   }
   return;
}

void DirList::chmod(void){
   if ( full_path[0] == '/' )
   {
      int count;
      QString filename;
      Chmod *cm = new Chmod();

      count = 0;
      QListViewItem *item = firstChild();

      while(item){
         if(item->isSelected()){
            filename  = full_path;
            filename += "/";
            filename += item->text(1);
            cm->addFile(filename);
            count++;
         }

         item = item->nextSibling();
      }
      filename = full_path;

      if ( count )
      {
         cm->show();
      }
      else
      {
         QMessageBox::warning( NULL, "Bronco", "No file(s) selected", QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape, 0);
         delete cm;
      }
   }
   return;
}

void DirList::keyPressEvent(QKeyEvent *e){
   QString dir;
   int k = e->key();
   int s = e->state();

   if( k >= Key_0 && k <= Key_9 && full_path[0] == '/' && prefs.getNumberShortcuts() )
   {
      if(s & ControlButton)
      {
         prefs.setDir((k - Key_0), full_path);
         prefs.save();
      }
      else
      {
         dir = prefs.getDir(k - Key_0);
         if(!dir.isNull() && !dir.isEmpty())
            loadDir(dir);
      }
      return;
   }
   else if(k == Key_Minus && full_path[0] == '/')
   {
      backDir();
   }
   else if(k == Key_Slash && full_path[0] == '/')
   {
      loadDir("/");
   }
   else if(k == Key_AsciiTilde && full_path[0] == '/')
   {
      loadDir(prefs.homeDir());
   }
   else if(k == Key_Return || k == Key_Enter)
   {
      openItem(currentItem());
   }
   else if( (k >= Key_A && k <= Key_Z) 
         || (k >= Key_0 && k <= Key_9)
         || (k >= Key_Plus && k <= Key_Period)
         || (k >= Key_BracketLeft && k <= Key_BraceRight)
         || (k >= Key_Colon && k <= Key_Question)
         || (k >= Key_Space && k <= Key_ParenRight)
         ||  k == Key_BackSpace)
   {
      char ch = e->ascii();

      if(k == Key_Backspace){
         keybuf_cur = 0;
         keybuf[keybuf_cur] = '\0';
      }
      else{
         keybuf[keybuf_cur]   = ch;
         keybuf[keybuf_cur+1] = '\0';

         keybuf_cur++;

         if(keybuf_cur >= KEYBUFSIZE){
            keybuf_cur = KEYBUFSIZE - 1;
         }
      }

      if ( strlen(keybuf) >= 2 )
      {
         if(strcmp( &keybuf[ strlen(keybuf) - 2], "..") == 0)
         {
            upDir();
            return;
         }
      }

      QListViewItem *item;
      item = this->firstChild();
      while(item)
      {
         if(strncmp(item->text(1), keybuf, keybuf_cur) == 0)
         {
            clearSelection();
            this->setCurrentItem(item);
            item->setSelected(TRUE);
            ensureItemVisible(item);
            break;
         }
         item = item->nextSibling();
      }
   }
   else
   {
      keybuf[0]  = '\0';
      keybuf_cur = 0;
      QListView::keyPressEvent(e);
   }
}

void DirList::refresh(void)
{
   m_doRefresh = true;
//   sort_policy = (prefs.getDirListOpts() | prefs.asciiSort());
   loadDir(full_path);
   return;
}

void DirList::hide(void)
{
   QListView::hide();
   return;
}

void DirList::newFile(void){
   if ( full_path[0] == '/' )
   {
      char name[1024];
      FILE *fp;

      bool ok;
      QString input = QInputDialog::getText( "New file", "New file name?", QLineEdit::Normal, NULL, &ok, this );

      if(ok && !input.isEmpty() && !input.isNull()){
         strcpy(name, input);
         if(name[0] != '/'){
            snprintf(name, sizeof(name), "%s/%s", full_path, (const char *) input);
         }
         else{
            strcpy(name, input);
         }

         if((fp=fopen(name, "a")) == NULL){
            QMessageBox::warning( NULL, name, strerror(errno), QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape, 0);
         }
         else{
            fclose(fp);
         }
         this->refresh();

         this->clearSelection();

         QListViewItem *item;
         item = this->firstChild();
         while(item){
            if(strcmp(item->text(1), input) == 0){
               this->setCurrentItem(item);
               item->setSelected(TRUE);
               ensureItemVisible(item);
               break;
            }
            item = item->nextSibling();
         }
      }
   }
}

void DirList::newDirectory(void){
   if ( full_path[0] == '/' )
   {
      char name[1024];
      bool ok;
      QString input = QInputDialog::getText( "New directory", "New directory name?", QLineEdit::Normal, NULL, &ok, this );

      if(ok && !input.isEmpty() && !input.isNull()){
         strcpy(name, input);
         if(name[0] != '/'){
            snprintf(name, sizeof(name), "%s/%s", full_path, (const char *) input);
         }

         if(mkdir(name, (S_IRWXU | S_IRWXG | S_IRWXO)) != 0){
            QMessageBox::warning( NULL, name, strerror(errno), QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape, 0);
         }
         this->refresh();
         this->clearSelection();

         QListViewItem *item;
         item = this->firstChild();
         while(item){
            if(strcmp(item->text(1), input) == 0){
               this->setCurrentItem(item);
               item->setSelected(TRUE);
               ensureItemVisible(item);
               break;
            }
            item = item->nextSibling();
         }
      }
   }
}

void DirList::launchShell(void)
{
   const char * dir;
   const char * xterm = prefs.getXterm();

   dir = prefs.homeDir();
   if(full_path[0] == '/')
   {
      dir = full_path;
   }
   else if ( strncmp(full_path, "find", 4) == 0 )
   {
      dir = full_path;
      QListViewItem *item = this->currentItem();
      
      if ( item )
      {
         QString file = item->text(1);
         if ( file.findRev('/') > 1 )
         {
            file.truncate( file.findRev('/') );

            dir = file;
         }
      }
   }

   if ( xterm[0] == '\0' )
   {
      xterm = "xterm";
   }

   chdir(dir);
   BroncoExec program;
   program.exec(xterm);
}

void DirList::mount(void)
{
   FILE *fp;
   QListViewItem *item = this->currentItem();
   QString mount_cmd;

   if(item)
   {
      if(strcmp(full_path, "/") == 0)
         mount_cmd.sprintf("/%s", (const char *) item->text(1));
      else
         mount_cmd.sprintf("%s/%s", full_path, (const char *) item->text(1));

      int found = 0;
      char input[256];
      if( (fp = fopen("/etc/mtab", "r")) != NULL)
      {
         while(fgets(input, sizeof(input), fp))
         {
            char *s = input;
            char *t;

            /* t points to begining of mount point
             * s points to end (null terminator) */
            while(*s && !isspace(*s)) 
               s++;

            while(*s &&  isspace(*s)) 
               s++;

            t = s;

            while(*s && !isspace(*s)) 
               s++;

            *s = '\0';

            if(strcmp(mount_cmd, t) == 0)
            {
               found = 1;
               break;
            }
         }
         fclose(fp);
      }

      if(found)
         mount_cmd.prepend("umount ");
      else
         mount_cmd.prepend("mount ");

      BroncoExec mount;
      
      mount.setPipeStderr(true);
      mount.setPipeStdout(true);
      mount.exec(mount_cmd.latin1());

      fp = mount.getPipeStderr();

      if(mount.wait() != 0)
      {
         if ( fp )
         {
            char input[80];
            QString error = "Mount failed\n";
            while ( fgets(input, sizeof(input), fp ) )
            {
               error += input;
            }
            QMessageBox::warning( NULL, "Bronco", error, QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape, 0);
         }
         else
         {
            QMessageBox::warning( NULL, "Bronco", "Could not mount", QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape, 0);
         }
      }

      if(found)
      {
         mount_cmd.replace(0, 6, "eject");
         BroncoExec eject;
         eject.setPipeStderr(true);
         eject.setPipeStdout(true);
         eject.exec(mount_cmd.latin1());
      }
   }
}

void DirList::createTar(void){
   if ( full_path[0] == '/' )
   {
      int n;
      const char *file_list[MAXARGS];

      TarDlg win;

      QString file, filename;
      QListViewItem *item;

      item = currentItem();

      if(item)
      {
         filename = item->text(1) + tr(".tar.gz");

         win.setFilename( filename );

         if ( win.exec() )
         {
            filename = win.getFilename();

            if ( !filename.isEmpty() && !filename.isNull() )
            {
               file_list[0] = strdup( filename.latin1() );

               switch ( win.getCompression( ) )
               {
                  case 0:
                     file = "tar -czf ";
                     break;
                  case 1:
                     file = "tar -cjf ";
                     break;
                  case 2:
                     file = "tar -cZf ";
                     break;
                  case 3:
                  default:
                     file = "tar -cf ";
                     break;
               }

               item = firstChild();
               n = 1;
               while ( item ){
                  if(item->isSelected()){
                     file_list[n++] = (item->text(1)).latin1();
                  }

                  item = item->nextSibling();
               }
               file_list[n] = NULL;

               chdir(full_path);
               BroncoExec prog;

               prog.exec(file.latin1(), file_list);
               prog.wait();

               refresh();

               for ( item = firstChild(); item; item = item->nextSibling() )
               {
                  if ( item->text(1) == filename )
                  {
                     clearSelection();
                     setCurrentItem( item );
                     ensureItemVisible( item );
                     item->setSelected( true );
                  }
               }
            }
            else
            {
               QMessageBox::warning( NULL, "Bronco", "No filename specified", QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape, 0);
            }
         }
      }
   }
}

void DirList::openList(QString & base_command, QStringList & list)
{
   const char *args[MAXARGS];
   BroncoExec prog;

   QString command;
   QString remainder;
   QString current;

   if ( full_path[0] == '/' )
   {
      chdir( full_path );
   }

   base_command = base_command.simplifyWhiteSpace();

   if ( base_command.isEmpty() )
   {
      QMessageBox::warning(this, "Bronco", "No command to execute");
      return;
   }

   if ( base_command.contains("$f", false) )
   {
      if ( base_command.contains("$l", false) )
      {
         QMessageBox::warning(this, "Bronco", base_command + tr("\ncontains $f and $l"));
         return;
      }

      for ( QStringList::Iterator it = list.begin(); it != list.end(); it++ )
      {
         BroncoExec prog;
         QStringList arglist;

         QString file = *it;
         QString path = "/";

         if ( full_path[0] == '/' )
         {
            path = full_path;

         }

         if ( file.contains("/") )
         {
            path += tr("/") + file.left( file.findRev("/") );
            file.replace( 0, file.findRev("/"), "");
         }

         chdir(path.latin1());

         command = base_command;

         remainder = command.right( command.length() - command.find( QRegExp("\\s") ) );
         command.truncate( command.find( QRegExp("\\s")) );

         if ( remainder.contains("$p") )
         {
            bool ok = false;
//            QString promptText = QInputDialog::getText("Bronco", tr("Extra arguments for ") + command, QString::null, &ok);
            QString promptText = QInputDialog::getText( "Bronco", tr("Extra arguments for ") + command, QLineEdit::Normal, QString::null, &ok );
            if ( ok )
            {
               remainder.replace( remainder.find("$p"), 2, promptText );
               remainder = remainder.simplifyWhiteSpace();
            }
         }

         remainder = remainder.stripWhiteSpace();

         while ( ! remainder.isEmpty() )
         {
            current = remainder;
            if ( remainder.find( QRegExp("\\s")) >= 0)
            {
               remainder = remainder.right( remainder.length() - remainder.find( QRegExp("\\s") ) );
               current.truncate( current.find( QRegExp("\\s")) );
            }
            else
            {
               remainder = "";
            }

            if ( current.contains("$F") )
            {
               current.replace( current.find("$F"), 2, path + tr("/") + file );
            }
            else if ( current.contains("$f") )
            {
               current.replace( current.find("$f"), 2, file );
            }
            else if ( current.contains("$n") )
            {
               current.replace( current.find("$n"), 2, "0");
            }
            else if ( current.contains("$d") )
            {
               current.replace( current.find("$d"), 2, path );
            }

            arglist << current;

            remainder = remainder.stripWhiteSpace();
         }

         int n = 0;
         for ( QStringList::Iterator it = arglist.begin(); it != arglist.end(); it++ )
         {
            args[n] = (*it).latin1();
            n++;

            if ( n == MAXARGS - 1 )
               break;
         }
         args[n] = NULL;

         prog.exec(command.latin1(), args);
      }
   }
   else
   {
      BroncoExec prog;
      QStringList arglist;

      QString path = "/";

      if ( full_path[0] == '/' )
      {
         path = full_path;
      }
      else
      {
         if ( (*list.begin())[0] != QChar('/') )
         {
            int answer = QMessageBox::warning(this, "Bronco", "Neither path nor file begin with '/'\nContinue?", QMessageBox::Yes | QMessageBox::Default, QMessageBox::No );

            if ( answer == QMessageBox::No )
            {
               return;
            }
         }
      }

      chdir(path.latin1());

      command = base_command;

      remainder = command.right( command.length() - command.find( QRegExp("\\s") ) );
      command.truncate( command.find( QRegExp("\\s")) );

      if ( remainder.contains("$p") )
      {
         bool ok = false;
//         QString promptText = QInputDialog::getText("Bronco", tr("Extra arguments for ") + command, QString::null, &ok);
         QString promptText = QInputDialog::getText( "Bronco", tr("Extra arguments for ") + command, QLineEdit::Normal, QString::null, &ok );
         if ( ok )
         {
            remainder.replace( remainder.find("$p"), 2, promptText );
            remainder = remainder.simplifyWhiteSpace();
         }
      }

      remainder = remainder.stripWhiteSpace();

      while ( ! remainder.isEmpty() )
      {
         current = remainder;
         if ( remainder.find( QRegExp("\\s")) >= 0)
         {
            remainder = remainder.right( remainder.length() - remainder.find( QRegExp("\\s") ) );
            current.truncate( current.find( QRegExp("\\s")) );
         }
         else
         {
            remainder = "";
         }

         if ( current.contains("$n") )
         {
            current.replace( current.find("$n"), 2, "0");
            arglist << current;
         }
         else if ( current.contains("$d") )
         {
            if ( full_path[0] == '/' )
            {
               current.replace( current.find("$d"), 2, full_path );
            }
            else
            {
               current.replace( current.find("$d"), 2, "/");
            }
            arglist << current;
         }
         else if ( current.contains("$L") )
         {
            QString pre;
            QString post;

            pre  = current.left(   current.find("$L") );
            post = current.right( (current.length() - current.findRev("$L")) - 2);

            for ( QStringList::Iterator it = list.begin(); it != list.end(); it++ )
            {
               if ( full_path[0] == '/' )
                  current = path + tr("/") + (*it);
               else
                  current = *it;

               if ( ! pre.isEmpty() )
                  arglist << pre;

               arglist << current;

               if ( ! post.isEmpty() )
                  arglist << post;
            }
         }
         else if ( current.contains("$l") )
         {
            QString pre;
            QString post;

            pre  = current.left(   current.find("$l") );
            post = current.right( (current.length() - current.findRev("$l")) - 2);

            for ( QStringList::Iterator it = list.begin(); it != list.end(); it++ )
            {
               if ( ! pre.isEmpty() )
                  arglist << pre;

               arglist << *it;

               if ( ! post.isEmpty() )
                  arglist << post;
            }
         }
         else
         {
            arglist << current;
         }

         remainder = remainder.stripWhiteSpace();
      }

      int n = 0;
      for ( QStringList::Iterator it = arglist.begin(); it != arglist.end(); it++ )
      {
         args[n] = (*it).latin1();
         n++;

         if ( n == MAXARGS - 1 )
            break;
      }
      args[n] = NULL;

      prog.exec(command.latin1(), args);
   }
}

void DirList::refreshDirectory(const QString &dir )
{
   if ( strcmp(dir.latin1(), full_path) == 0 )
   {
      this->refresh();
   }
}

void DirList::launchItem(int item_id )
{
   QString     item;
   QString     command;
   QStringList args;

   int index = m_launchMenu->indexOf(item_id);

   if ( index < prefs.launches() )
   {
      prefs.getLaunch( index, item, command);

      QListViewItem *item = this->firstChild();
      while ( item )
      {
         if ( item->isSelected() )
         {
            args << item->text(1);
         }

         item = item->nextSibling();
      }

      openList(command, args);
   }
   else
   {
      QMessageBox::warning(this, "Bronco", "Command not specified.\nGo to Settings->Preferences->Custom Commands.");
   }
}

void DirList::rebuildLaunchMenu(void)
{
   QString item, command;
   m_launchMenu->clear();
   if ( prefs.launches() > 0 )
   {
      for (int n = 0; n < prefs.launches(); n++ )
      {
         prefs.getLaunch(n, item, command);
         m_launchMenu->insertItem(item);
      }
   }
   else
   {
      m_launchMenu->insertItem("None");
   }
}

QPopupMenu * DirList::getLaunchMenu(void)
{
   if ( m_launchMenu == NULL )
   {
      m_launchMenu = new QPopupMenu(this, "launch_menu");
      connect(m_launchMenu, SIGNAL(activated(int)), this, SLOT(launchItem(int)));
      rebuildLaunchMenu();
   }
   return m_launchMenu;
}

void DirList::addFile( const char * file )
{
   currentDir.add( file );
}

void DirList::clearList(void)
{
   currentDir.clear();
   showContents();
}

void DirList::doListUpdate(void)
{
   showContents();
}

void DirList::shortMenu(void)
{
   /* Context menu init */
   context_menu->clear();

   context_menu->insertItem("Open",             this, SLOT(openItem()));
   context_menu->insertItem("Open with...",     this, SLOT(openItemWith()));
   context_menu->insertItem("Custom Commands",  m_launchMenu);

   context_menu->insertSeparator();
   context_menu->insertItem("Copy",             this, SLOT(copyCommand()));
   context_menu->insertItem("Copy to...",       this, SLOT(copyTo()));
   context_menu->insertItem("Delete",           this, SLOT(deleteCommand()));

   context_menu->insertSeparator();
   QPopupMenu * new_menu = new QPopupMenu(this, "");
   new_menu->insertItem("New Window",           this, SLOT(newWindow()));
   new_menu->insertItem("New Shell",            this, SLOT(launchShell()));
   context_menu->insertItem("New",  new_menu);

   QPopupMenu * select_menu = new QPopupMenu(this, "");
   select_menu->insertItem("Select all",        this, SLOT(selectAll()));
   select_menu->insertItem("Unselect all",      this, SLOT(unSelectAll()));
   select_menu->insertItem("Select mask...",    this, SLOT(selectMask()));
   context_menu->insertItem("Select",  select_menu);
}

void DirList::changeFont(void)
{
   char family[64];
   int  fsize = 10;

   prefs.getFont(family, &fsize);
   QFont font(family, fsize);
   this->setFont( font );
}

void DirList::copyTo(void)
{
   if ( full_path[0] == '/' || strncmp(full_path, "find", 4) == 0)
   {
      move_operation = 0;
      listToClip();
      QString dir = QFileDialog::getExistingDirectory(QString::null, this, "", 
            "Copy To...", true);

      if ( !dir.isNull() && !dir.isEmpty() )
      {
         FileWin *win = new FileWin(NULL, "Copy file(s)...");
         win->setDest( dir.latin1() );
         if( ! win->clipCopy() )
         {
            win->goAway();
         }
      }
   }
}

void DirList::moveTo(void)
{
   if ( full_path[0] == '/' )
   {
      move_operation = 1;
      listToClip();
      QString dir = QFileDialog::getExistingDirectory(QString::null, this, "", 
            "Move To...", true);

      if ( !dir.isNull() && !dir.isEmpty() )
      {
         FileWin *win = new FileWin(NULL, "Move file(s)...");
         win->setDest( dir.latin1() );
         if( ! win->clipMove() )
         {
            win->goAway();
         }
      }
   }
}
