/* dirlist.h - QListView widget for a single directory listing
 * 
 *  Bronco File Manager
 * 
 *  Copyright (c) 2000, 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#ifndef DIRLISTH
#define DIRLISTH

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <qlistview.h>
#include <qaccel.h>
#include <qpopupmenu.h>
#include <qevent.h>
#include <qfont.h>
#include "direct.h"
#include "diritem.h"
#include "propwin.h"

#define KEYBUFSIZE 64

class DirList : public QListView{
   Q_OBJECT

   public:
      DirList(QWidget *, char *);
      virtual ~DirList();

      int loadDir(const char *);
      int reLoadDir(const char *);
      const char *getDir(void);

      QPopupMenu * getLaunchMenu(void);

   public slots:
      void changeFont(void);

      void hide(void);
      void refresh(void);
      void shortMenu(void);

      void refreshDirectory(const QString &);

      void changeDir(DirItem *);
      void upDir(void);
      void backDir(void);

      void addFile(const char *);
      void clearList(void);
      void doListUpdate(void);

      void sortColumn(int);
      void sortChange(int);

      void selectAll(void);
      void unSelectAll(void);
      void selectMask(void);

      void copyTo(void);
      void moveTo(void);

      void copyCommand(void);
      void moveCommand(void);
      void pasteCommand(void);
      void deleteCommand(void);
      void renameCommand(void);
      void listToClip(void);

      void openItem(QListViewItem *);
      void openItem(void);
      void openItemWith(QListViewItem *);
      void openItemWith(void);
      void openList(QString &, QStringList &);

      void properties(void);
      void chmod(void);

      void newDirectory(void);
      void newFile(void);

      void findWindow(void);
      void grepWindow(void);
      void tarWindow(void);
      void newWindow(void);
      void newWindow(const char *);
      void contextMenu(void);

      void launchShell(void);
      void mount(void);
      void createTar(void);

      void launchItem(int);
      void rebuildLaunchMenu();

   signals:
      void closeMe(DirList *);
      void directoryChanged(const QString &dir);

   protected:
#if (QT_VERSION >= 300 )
      void contentsContextMenuEvent(QContextMenuEvent *);
#else
      void contentsMousePressEvent(QMouseEvent *);
#endif
      void contentsMouseReleaseEvent(QMouseEvent *);
      void keyPressEvent(QKeyEvent *);
      void resizeEvent(QResizeEvent *);
      void showContents(void);
      void doCopy(void);
      void doMove(void);
      void doDelete(void);

      QPopupMenu * m_launchMenu;
      QAccel *accel;
      QPopupMenu *context_menu;
      QFont *font;

      Directory currentDir;

      char keybuf[KEYBUFSIZE];
      char full_path[4096];
      char last_path[4096];
      int sort_policy;
      int keybuf_cur;
      bool m_doRefresh;
};

#endif
