/* dirtree.cpp - Directory tree widget
 * 
 *  Bronco File Manager
 * 
 *  Copyright (c) 2000, 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#include <qaccel.h>
#include <qmessagebox.h>
#include "dirtree.h"
#include "direct.h"
#include "llist.h"
#include "bronco.h"

DirTree::DirTree(QWidget *parent=NULL, char *name=NULL) : QListView(parent, name){
   char family[64];
   int fsize  = 10;
   prefs.getFont(family, &fsize);
   setFont(QFont(family, fsize));
   setAllColumnsShowFocus(TRUE);
   QScrollView::setVScrollBarMode(AlwaysOn);

   connect(this, SIGNAL(currentChanged(QListViewItem *)), this, SLOT(newCurrentItem(QListViewItem *)));
   connect(this, SIGNAL(doubleClicked(QListViewItem *)), this, SLOT(newCurrentItem(QListViewItem *)));

   QAccel *accel = new QAccel(this);
   accel->connectItem(accel->insertItem(Key_F5), this, SLOT(refreshTree()));
   return;
}

int DirTree::addRoot(char *dir){
   class DirItem *rootItem;
   rootItem = new DirItem(this, NULL, dir, dir);
   rootItem->setOpen(TRUE);

   return(0);
}

int DirTree::refresh(void){
   return(0);
}

void DirTree::newCurrentItem(QListViewItem *item){
   DirItem *realitem;
   realitem = (DirItem *) item;
   emit currentChanged(realitem);
}

void DirTree::setDir(const char *path){
   QListViewItem *item = this->firstChild();
   QListViewItem *parent;
   while(*path && *path=='/') path++;
   char dirbuf[1024];
   int n;

   parent = NULL;

   while(*path && item){
      item->setOpen(TRUE);
      for(n=0; *path && *path != '/'; n++, path++){
         dirbuf[n] = *path;
      }
      dirbuf[n] = 0;
      if(*path) path++;

      parent = item;
      item = item->firstChild();
      while(item){
         if(strcmp(dirbuf, (const char *) item->text(0)) == 0){
            break;
         }
         item = item->nextSibling();
      }
      if(!item){
         // We're probably looking at a '.directory'.  Ignore and return;
         return;
      }
   }
   if(item){
      setCurrentItem(item);
      ensureItemVisible(item);
   }
}

void DirTree::openDir(const char *path){
   QListViewItem *item = this->firstChild();
   while(*path && *path=='/') path++;
   char dirbuf[1024];
   int n;

   if(item) item->setOpen(TRUE);
   while(*path && item){
      for(n=0; *path && *path != '/'; n++, path++){
         dirbuf[n] = *path;
      }
      dirbuf[n] = 0;
      if(*path) path++;

      item = item->firstChild();

      while(item){
         if(strcmp(dirbuf, (const char *) item->text(0)) == 0){
            item->setOpen(TRUE);
            break;
         }
         item = item->nextSibling();
      }
      if(!item){
         QMessageBox::warning( NULL, "Bronco error",
                            "Item not found", QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape, 0);
      }
   }
}

void DirTree::refreshTree(void){
   LinkedList dir;
   QListViewItem *cur;
   char path[1024];
   char work[1024];
   char current_path[1024];

   cur = this->currentItem();
   current_path[0] = '\0';
   while(cur){
      snprintf(work, sizeof(work), "%s/%s", (const char *) cur->text(0), current_path);
      strcpy(current_path, work);
      cur = cur->parent();
   }

   QListViewItemIterator item(this);

   item = this->firstChild();
   while(item.current()){
      if(item.current()->isOpen()){
         cur = item.current();
         path[0] = '\0';
         while(cur){
            snprintf(work, sizeof(work), "%s/%s", (const char *) cur->text(0), path);
            strcpy(path, work);
            cur = cur->parent();
         }
         dir.add(path);
      }
      item++;
   }

   this->clear();
   setUpdatesEnabled(FALSE);
   addRoot("/");
   if(dir.first(path)){
      do{
         openDir(path);
      }while(dir.next(path));
   }

   item = this->firstChild();
   while(item.current()){
      cur = item.current();
      path[0] = '\0';
      while(cur){
         snprintf(work, sizeof(work), "%s/%s", (const char *) cur->text(0), path);
         strcpy(path, work);
         cur = cur->parent();
      }
      if(strcmp(path, current_path) == 0){
         cur = item.current();
         setCurrentItem(cur);
         break;
      }
      item++;
   }

   setUpdatesEnabled(TRUE);

   return;
}
