/* execbr.h - Wrapper to fork, exec, pipe, ect..
 * 
 *  Bronco File Manager
 * 
 *  Copyright (c) 2000, 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#ifndef EXECBR_H
#define EXECBR_H

#include <stdio.h>

#define MAXARGS 1024

class BroncoExec
{
   public:
      BroncoExec();
      virtual ~BroncoExec();

      void setPipeStdout(bool);
      void setPipeStderr(bool);
      void setPipeStdin(bool);

      void setWait(bool);

      FILE * getPipeStdout();
      FILE * getPipeStderr();
      FILE * getPipeStdin();

      int exec(const char *command, const char * args[]);
      int exec(const char *command, const char * file);
      int exec(const char *command);
      int wait();
      
      int killChild(void);

   protected:
      FILE * m_stdout;
      FILE * m_stderr;
      FILE * m_stdin;

      bool   m_wait;
      bool   m_pipeStdout;
      bool   m_pipeStderr;
      bool   m_pipeStdin;

      int    m_childPid;
      int    m_parentPid;

};

#endif /* EXECBR_H */
