/* fifolist.cpp - ListView for Bronco helpers
 * 
 *  Bronco File Manager
 * 
 *  Copyright (c) 2000, 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#include "fifolist.h"

#include <qlistview.h>
#include <qheader.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

FifoList::FifoList( int childPid )
    : QDialog( NULL, "", false, 0 )
{
   resize( 596, 480 ); 
   setCaption( tr( "Bronco Helper" ) );
   FifoListLayout = new QVBoxLayout( this ); 
   FifoListLayout->setSpacing( 6 );
   FifoListLayout->setMargin( 11 );

   list_v = new QListView( this, "list_v" );
   list_v->setAllColumnsShowFocus( true );
   list_v->setSorting(-1, false);
   list_v->header()->setClickEnabled( false );
   list_v->header()->setMovingEnabled( false );
   FifoListLayout->addWidget( list_v );

   Layout1 = new QHBoxLayout; 
   Layout1->setSpacing( 6 );
   Layout1->setMargin( 0 );
   QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
   Layout1->addItem( spacer );

   ok_b = new QPushButton( this, "ok_b" );
   ok_b->setText( tr( "Ok" ) );
   ok_b->setDefault( true );
   Layout1->addWidget( ok_b );

   QSpacerItem* spacer_2 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
   Layout1->addItem( spacer_2 );
   FifoListLayout->addLayout( Layout1 );

   m_childPid = childPid;
   m_lastItem = NULL;

   connect( ok_b, SIGNAL( clicked() ), this, SLOT( accept() ) );

   this->show();
}

FifoList::~FifoList()
{
}

void FifoList::appendItem(const QStringList &list )
{
   QListViewItem *item = new QListViewItem(list_v, m_lastItem);
   m_lastItem = item;

   for ( unsigned t = 0; t < (unsigned) list.count() && t < (unsigned) list_v->columns(); t++ )
   {
      item->setText(t, list[t]);
   }
}

void FifoList::setColumns(const QStringList &list )
{
   for ( unsigned t = 0; t < list.count(); t++ )
   {
      list_v->addColumn( list[t] );
   }
}

void FifoList::addColumn(const QString &s)
{
   list_v->addColumn( s );
}

void FifoList::hide()
{
   emit closeFifoList( m_childPid );

   QDialog::hide();
}
