/* filewin.h - Class to handle file operations
 * 
 *  Bronco File Manager
 * 
 *  Copyright (c) 2000, 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#ifndef FILEWINH

#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>

#include <qapplication.h>
#include <qwidget.h>
#include <qprogressbar.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qlayout.h>

#include <hash_map>

#include "bronco.h"

class FileWin;

typedef hash_map<int, FileWin *, hash<int> > FileWinHash;

class FileWin : QWidget
{
   Q_OBJECT

   public:
      FileWin(QWidget *, const char *);
      virtual ~FileWin();

      int clipCopy(void);
      int clipMove(void);
      int clipDelete(void);
      int setDest(const char *);

      void sigYes(int s);
      void sigNo(int s);
      void operationDone(void);

      void setProgress(int);
      void setCurrentFile(const char *);

      static void sigCancel(int s);
      static FileWin * getWinNumber( int w );

   public slots:
      void goAway(void);

   protected slots:
      void cancelPressed();

   protected:
      int move(const char *, const char *);
      int copy(const char *, const char *);
      int copy_file(const char *, const char *, mode_t);
      int del(const char *);
      long int compute_blocks(const char *);
      void blocks_done(long int);
      void errorlog(const char *, ...);

      long int btotal;
      long int bdone;
      int overwrite;
      char dest[1024];

      QProgressBar *pbar;
      QLineEdit    *curfile;
      QPushButton  *cancel_b;

      FILE * m_fifofp;
      FILE * m_errorFp;
      int m_answer;
      int m_winNumber;
      int m_pid;
      int percent;

      static int s_nextFileWin;
      static FileWinHash s_winHash;

};

#endif /* FILEWINH */
