/* find.cpp - Routines for pattern matching and file search
 * 
 *  Bronco File Manager
 * 
 *  Copyright (c) 2000, 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>
#include <sys/stat.h>
#include "llist.h"
#include "direct.h"
#include "find.h"

int find_prefs = (FIND_REG | FIND_REC);
int find_error = 0;

int match(const char *glob, const char *string){
   int exact   = 1;
   int advance = 0;
   unsigned length = 0;
   const char *gptr;
   
   if(strcmp("*", glob) == 0){
      return(1);
   }

   unsigned n;

   if(glob[0] == '*'){
      gptr = glob + 1;
      advance = 1;
   }
   else{
      advance = 0;
      gptr = glob;
   }
   for(n=0; n<strlen(gptr); n++){
      if(gptr[n] == '*'){
         exact  = 0;
         length = n;
         break;
      }
   }
   if(length == 0) length = strlen(gptr);

   while(strlen(string) >= length){
      if(find_prefs & FIND_CASE){
         if(strncmp(gptr, string, length) == 0){
            if(exact){
               if(length==strlen(string)){
                  return(1);
               }
            }
            else{
               if(match(gptr+length, string+length)){
                  return(1);
               }
            }
         }
         if(!advance) break;
         string++;
      }
      else{
         if(strncasecmp(gptr, string, length) == 0){
            if(exact){
               if(length==strlen(string)){
                  return(1);
               }
            }
            else{
               if(match(gptr+length, string+length)){
                  return(1);
               }
            }
         }
         if(!advance) break;
         string++;
      }
   }
   return 0;
}

int regMatch(const char *glob, const char *string){
   KRegex r;
   if(find_prefs & FIND_CASE)
   {
      r.setcflags(REG_EXTENDED);
   }
   else
   {
      r.setcflags(REG_EXTENDED | REG_ICASE);
   }
   
   if(r.set(glob))
   {
      if(r.match(string))
      {
         return(1);
      }
      else{
         find_error = r.getError();
      }
   }
   else
   {
      find_error = r.getError();
   }
   return(0);
}

int findFile(const char *path, const char *glob, LinkedList *list, FILE *fp){
   DIR *dp;
   struct dirent *d;
   struct stat   s;

   char dirbuf[16384];

   if((dp = opendir(path)) != NULL){
      while((d = readdir(dp)) != NULL){
         if((strcmp(".", d->d_name) == 0) || (strcmp("..", d->d_name)==0)) 
            continue;
         snprintf(dirbuf, sizeof(dirbuf), "%s/%s", path, d->d_name);

         if(lstat(dirbuf, &s) >= 0){
            if(find_prefs & FIND_REG){
               if(regMatch(glob, d->d_name)){
                  if ( list )
                     list->add(dirbuf);
                  else if ( fp )
                     fprintf(fp, "%s\n", dirbuf);
                  else
                     printf("%s\n", dirbuf);
               }
            }
            else{
               if(match(glob, d->d_name)){
                  if ( list ) 
                     list->add(dirbuf);
                  else if ( fp )
                     fprintf(fp, "%s\n", dirbuf);
                  else
                     printf("%s\n", dirbuf);
               }
            }

            if(S_ISDIR(s.st_mode) && (find_prefs & FIND_REC)){
               findFile(dirbuf, glob, list, fp);
            }
         }
      }
      closedir(dp);
   }
   else{
      return(-1);
   }
   return 0;
}

void setFindPrefs(int p){
   find_prefs = p;
}

int getFindPrefs(void){
   return(find_prefs);
}

int getFindError(void){
   return(find_error);
}
