/* globedit.cpp - Edit a command glob
 * 
 *  Bronco File Manager
 * 
 *  Copyright (c) 2000, 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#include "globedit.h"

#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qmessagebox.h>
#include <qregexp.h>

GlobEdit::GlobEdit( QWidget* parent,  const char* name)
   : QDialog( parent, name, TRUE, 0 )
{
   if ( !name )
      setName( "GlobEdit" );
   setFixedSize( 402, 112 ); 
   setCaption( tr( "Edit File Association"  ) );

   QWidget* Layout4 = new QWidget( this, "Layout4" );
   Layout4->setGeometry( QRect( 11, 11, 377, 52 ) ); 
   grid = new QGridLayout( Layout4 ); 
   grid->setSpacing( 6 );
   grid->setMargin( 0 );

   command_l = new QLabel( Layout4, "command_l" );
   command_l->setText( tr( "Command"  ) );

   grid->addWidget( command_l, 1, 0 );

   command_v = new QLineEdit( Layout4, "command_v" );

   grid->addWidget( command_v, 1, 1 );

   glob_l = new QLabel( Layout4, "glob_l" );
   glob_l->setText( tr( "File Glob"  ) );

   grid->addWidget( glob_l, 0, 0 );

   glob_v = new QLineEdit( Layout4, "glob_v" );

   grid->addWidget( glob_v, 0, 1 );

   QWidget* Layout5 = new QWidget( this, "Layout5" );
   Layout5->setGeometry( QRect( 180, 70, 207, 30 ) ); 
   hbox = new QHBoxLayout( Layout5 ); 
   hbox->setSpacing( 6 );
   hbox->setMargin( 0 );
   QSpacerItem* spacer = new QSpacerItem( 33, 21, QSizePolicy::Maximum, QSizePolicy::Minimum );
   hbox->addItem( spacer );

   ok_b = new QPushButton( Layout5, "ok_b" );
   ok_b->setText( tr( "&Ok"  ) );
   ok_b->setDefault( TRUE );
//   ok_b->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, ok_b->sizePolicy().hasHeightForWidth() ) );
   hbox->addWidget( ok_b );

   cancel_b = new QPushButton( Layout5, "cancel_b" );
   cancel_b->setText( tr( "&Cancel"  ) );
//   cancel_b->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, cancel_b->sizePolicy().hasHeightForWidth() ) );
   hbox->addWidget( cancel_b );

//   connect( ok_b, SIGNAL( clicked() ), this, SLOT( accept() ) );
//   connect( command_v, SIGNAL( returnPressed() ), this, SLOT( accept() ) );
   connect( ok_b, SIGNAL( clicked() ), this, SLOT( okClicked() ) );
   connect( cancel_b, SIGNAL( clicked() ), this, SLOT( reject() ) );
   connect( glob_v, SIGNAL( returnPressed() ), command_v, SLOT( setFocus() ) );
   connect( command_v, SIGNAL( returnPressed() ), this, SLOT( okClicked() ) );

   setTabOrder(glob_v, command_v);
   setTabOrder(command_v, ok_b);
   setTabOrder(ok_b, cancel_b);
   setTabOrder(cancel_b, glob_v);

   glob_v->setFocus();
}

GlobEdit::~GlobEdit()
{
}

QString GlobEdit::getCommand(void)
{
   return(command_v->text());
}

QString GlobEdit::getGlob(void)
{
   return(glob_v->text());
}

void GlobEdit::setCommand(QString s)
{
   command_v->setText(s);
}

void GlobEdit::setGlob(QString s)
{
   glob_v->setText(s);
}

void GlobEdit::okClicked(void)
{
   QString command = command_v->text();

   if ( ! command.contains( QRegExp("\\$[fl]", false) ) )
   {
      int answer = QMessageBox::warning(this, "Bronco", "The command you entered does not contain $F or $L\n$F or $L are replaced with the selected file name(s).\nContinue anyway?",
            QMessageBox::Yes | QMessageBox::Default, QMessageBox::No);

      if ( answer == QMessageBox::No )
      {
         return;
      }
   }
   if ( command.contains( QRegExp("\\$f", false) ) && command.contains( QRegExp("\\$l", false) ) )
   {
      int answer = QMessageBox::warning(this, "Bronco", "The command you entered contains both $F and $L\nThis behavior is undefined.  Continue anyway?",
            QMessageBox::Yes | QMessageBox::Default, QMessageBox::No);

      if ( answer == QMessageBox::No )
      {
         return;
      }
   }
   accept();
}
