/* globwin.cpp - Window for modifying command globs
 * 
 *  Bronco File Manager
 * 
 *  Copyright (c) 2000, 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#include "globwin.h"

#include <qheader.h>
#include <qlistview.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include "globedit.h"
#include "bronco.h"

GlobWin::GlobWin( QWidget* parent,  const char* name)
   : QDialog( parent, name, FALSE, 0 )
{
   if ( !name )
      setName( "GlobWin" );
   resize( 400, 300 ); 
   setCaption( tr( "Configure File Associations"  ) );
   vbox = new QVBoxLayout( this ); 
   vbox->setSpacing( 6 );
   vbox->setMargin( 11 );

   glob_l = new QListView( this, "glob_l" );
   glob_l->addColumn( tr( "Glob" ) );
   glob_l->addColumn( tr( "Command" ) );
   glob_l->setHScrollBarMode( QListView::AlwaysOff );
   glob_l->setAllColumnsShowFocus( TRUE );
   (glob_l->header())->setMovingEnabled( FALSE );
   vbox->addWidget( glob_l );

   hbox = new QHBoxLayout; 
   hbox->setSpacing( 6 );
   hbox->setMargin( 0 );

   new_b = new QPushButton( this, "new_b" );
   new_b->setText( tr( "&New"  ) );
   hbox->addWidget( new_b );

   delete_b = new QPushButton( this, "delete_b" );
   delete_b->setText( tr( "&Delete"  ) );
   hbox->addWidget( delete_b );

   close_b = new QPushButton( this, "close_b" );
   close_b->setText( tr( "Close"  ) );
   close_b->setDefault(true);
   close_b->setFocus();
   hbox->addWidget( close_b );
   vbox->addLayout( hbox );

   // signals and slots connections
   connect( close_b, SIGNAL( clicked() ), this, SLOT( accept() ) );
   connect( new_b, SIGNAL( clicked() ), this, SLOT( newGlob() ) );
   connect( delete_b, SIGNAL( clicked() ), this, SLOT( deleteGlob() ) );
   connect( glob_l, SIGNAL( doubleClicked(QListViewItem *) ), this, SLOT( editGlob(QListViewItem *) ) );
   connect( glob_l, SIGNAL( returnPressed(QListViewItem *) ), this, SLOT( editGlob(QListViewItem *) ) );
}

GlobWin::~GlobWin()
{
}

void GlobWin::newGlob()
{
   editGlob(NULL);
}

void GlobWin::editGlob(QListViewItem *i)
{
   GlobEdit e(this, "globedit");

   if(i)
   {
      e.setGlob(i->text(0));
      e.setCommand(i->text(1));
   }
   else
   {
      e.setGlob("");
      e.setCommand("");
   }

   if(e.exec())
   {
      if(i)
      {
         prefs.deleteCommand(i->text(0));
         delete i;
      }

      QString glob    = e.getGlob();
      QString command = e.getCommand();
      prefs.deleteCommand(glob);
      prefs.addCommand(glob, command);

      QListViewItem *item = glob_l->firstChild();

      while(item)
      {
         if(item->text(0) == glob)
         {
            item->setText(1, command);
            return;
         }
         item = item->nextSibling();
      }

      item = new QListViewItem(glob_l, NULL, e.getGlob(), e.getCommand());
   }
}

void GlobWin::deleteGlob()
{
   QListViewItem *i = glob_l->currentItem();
   if(i)
   {
      prefs.deleteCommand(i->text(0));
      delete i;
   }
}

void GlobWin::openWindow()
{
   QListViewItem *item;
   char glob[1024];
   char command[1024];

   glob_l->clear();

   if(prefs.firstCommand(glob, command))
   {
      item = NULL;

      do
      {
         item = new QListViewItem(glob_l, item, glob, command);
      } while(prefs.nextCommand(glob, command));
   }
   show();
}

void GlobWin::hide(void)
{
   prefs.save();
   QWidget::hide();
}
