/* grepwin.h - Window for grepping
 * 
 *  Bronco File Manager
 * 
 *  Copyright (c) 2000, 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#ifndef GREPWINH
#define GREPWINH

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <qaccel.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qlistview.h>
#include <qlayout.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qpushbutton.h>
#include <qtimer.h>

class BroncoExec;

class GrepWin : public QWidget {
      Q_OBJECT

   public:
      GrepWin(QWidget *, const char *);
      virtual ~GrepWin();

   public slots:
      void doSearch(void);
      void hide(void);
      void show(void);
      void setDir(const char *);
      void addFile( const QString &s );
      void toggleWhere(bool);

   protected slots:
      void updateSearchIn(const QString &);
      void openItem(QListViewItem *);
      void readTimeout(void);
      void cancelClicked(void);

   protected:
      void resizeEvent(QResizeEvent *e);

      FILE * grep_fp;

      QListViewItem *m_lastItem;
      BroncoExec    *m_prog;

      QString     m_startDir;
      QStringList m_selectedFiles;

      QString     m_lastSearchDir;
      QListView * list;

      QLineEdit * what_v;
      QLineEdit * where_v;

      QCheckBox * selected_v;
      QCheckBox * case_v;
      QCheckBox * recurse_v;

      QPushButton *search_b;
      QPushButton *cancel_b;

      QLabel *what_l;
      QLabel *where_l;

      QVBoxLayout* GrepWinLayout;
      QGridLayout* Layout2;
      QHBoxLayout* Layout1;

      QTimer * read_t;
};

#endif // GREPWINH
