/* itemedit.cpp - Generic window for editing items of a list
 * 
 *  Bronco File Manager
 * 
 *  Copyright (c) 2000, 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#include "itemedit.h"

#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

ItemEdit::ItemEdit( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
   if ( !name )
      setName( "ItemEdit" );

   resize( 360, 147 ); 
   setCaption( tr( "Item Edit" ) );

   ItemEditLayout = new QVBoxLayout( this ); 
   ItemEditLayout->setSpacing( 6 );
   ItemEditLayout->setMargin( 11 );

   Layout3 = new QGridLayout; 
   Layout3->setSpacing( 6 );
   Layout3->setMargin( 0 );

   ItemEditLayout->addLayout( Layout3 );

   Layout4 = new QHBoxLayout; 
   Layout4->setSpacing( 6 );
   Layout4->setMargin( 0 );
   QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
   Layout4->addItem( spacer );

   ok_b = new QPushButton( this, "ok_b" );
   ok_b->setText( tr( "Ok" ) );
   ok_b->setDefault( TRUE );
   Layout4->addWidget( ok_b );

   cancel_b = new QPushButton( this, "cancel_b" );
   cancel_b->setText( tr( "Cancel" ) );
   Layout4->addWidget( cancel_b );
   ItemEditLayout->addLayout( Layout4 );

   connect( ok_b,     SIGNAL( clicked() ), this, SLOT( accept() ) );
   connect( cancel_b, SIGNAL( clicked() ), this, SLOT( reject() ) );
}

ItemEdit::~ItemEdit()
{
}

void ItemEdit::setLabels( QStringList & list )
{
   QLabel *label;

   int n = 0;
   for ( QStringList::Iterator it = list.begin(); it != list.end(); it++ )
   {
      label = new QLabel( this, "" );
      label->setText( *it );
      Layout3->addWidget( label, n, 0 );
      n++;
   }
}

void ItemEdit::setValues( QStringList & list )
{
   QLineEdit *value;

   m_valueList.clear();

   int n = 0;
   for ( QStringList::Iterator it = list.begin(); it != list.end(); it++ )
   {
      value = new QLineEdit( this, "value_2" );
      value->setText( *it );
      Layout3->addWidget( value, n, 1 );

      if ( n == 0 )
         value->setFocus();

      n++;

      m_valueList.push_back( value );
   }
}

void ItemEdit::getValues( QStringList & list )
{
   list.clear();
   for ( LineEditList::iterator it = m_valueList.begin(); it != m_valueList.end(); it++ )
   {
      list << (*it)->text();
   }
}


