/* kregex.cpp - Regex class for file searches
 * 
 *  Bronco File Manager
 * 
 *  Copyright (c) 2000, 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#include "kregex.h"

/*
 * Compile flags (cflags)
 *   REG_EXTENDED
 *   REG_ICASE
 *   REG_NOSUB
 *   REG_NEWLINE
 *
 * Execute flags (eflags)
 *   REG_NOTBOL
 *   REG_NOTEOL
 *
 */

KRegex::KRegex(){
   int t;
   cflags = REG_EXTENDED;
   eflags = 0;

   for(t=0; t<MAXSUBS; t++){
      match_string[t] = NULL;
   }
}

KRegex::KRegex(const char *s){
   KRegex();
   set(s);
}

KRegex::~KRegex(){
   matchFree();
}

void KRegex::matchFree(){
   int t;
   for(t=0; t<MAXSUBS; t++){
      if(match_string[t]){
         free(match_string[t]);
      }
   }
}

void KRegex::setcflags(int f){
   cflags = f;
   return;
}

void KRegex::seteflags(int f){
   eflags = f;
   return;
}

int KRegex::set(const char *s){
   char scratch[50];
   int t, n, c, i;
   int len;
   int replaced;
   char esc;

   matchFree();

   len = strlen(s);

   for(t=0, n=0; n<len; t++, n++){

      replaced = 0;
      if(s[n] == '\\'){
         esc = s[n+1];
         for(c=0; c<MAXREPLACE; c++){
            if(replace_strings[c][0] == esc){
               for(i=1; replace_strings[c][i]; i++){
                  scratch[t] = replace_strings[c][i];
                  t++;
               }
               n++;
               t--;
               replaced = 1;
               break;
            }
         }
      }

      if(!replaced){
         scratch[t] = s[n];
      }
   }
   scratch[t] = '\0';
   if((error_code = regcomp(&pattern, scratch, cflags)) == 0){
      return(1);
   }
   else{
      return(0);
   }
}

int KRegex::match(const char *s){
   int t;
   int len;

   matchFree();

   if(regexec(&pattern, s, MAXSUBS, matches, eflags) == 0){
      for(t=0; t<MAXSUBS; t++){
         if(matches[t].rm_so >= 0){
            len = matches[t].rm_eo - matches[t].rm_so + 1;
            match_string[t] = (char *) malloc(len);
            strncpy(match_string[t], s + matches[t].rm_so, len);
            match_string[t][len-1] = '\0';
         }
         else{
            break;
         }
      }
      return(1);
   }
   else{
      error_code = REG_NOMATCH;
      return(0);
   }
}

int KRegex::getError(void){
   return(error_code);
}

const char * KRegex::stringMatch(int n){
   /* Okay to return uninitialized because it's NULL */
   return(match_string[n]);
}
