/* kregex.h - Regex class for file searches
 * 
 *  Bronco File Manager
 * 
 *  Copyright (c) 2000, 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#ifndef KREGEXH
#define KREGEXH

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <regex.h>

/*
 * Compile flags (cflags)
 *   REG_EXTENDED
 *   REG_ICASE
 *   REG_NOSUB
 *   REG_NEWLINE
 *
 * Execute flags (eflags)
 *   REG_NOTBOL
 *   REG_NOTEOL
 *
 */

#define MAXREPLACE 6

const char replace_strings[MAXREPLACE][15] ={
   { "s[ \t\n\r\f]" },
   { "S[^ \t\n\r\f]" },
   { "w[A-Za-z_]" },
   { "W[^A-Za-z_]" },
   { "d[0-9]" },
   { "D[^0-9]" }
};

#define MAXSUBS 10

class KRegex {

   public:
      KRegex();
      KRegex(const char *);
      virtual ~KRegex();
      int set(const char *);
      void setcflags(int);
      void seteflags(int);
      int match(const char *);
      const char *stringMatch(int);
      int getError(void);

   protected:
      void matchFree(void);

      regex_t pattern;
      regmatch_t matches[MAXSUBS];
      int error_code;
      char *match_string[MAXSUBS];
      int cflags;
      int eflags;
};

#endif  /* KREGEXH */
