/* listedit.cpp - Generic window for editing lists
 * 
 *  Bronco File Manager
 * 
 *  Copyright (c) 2000, 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#include "listedit.h"

#include <qheader.h>
#include <qlistview.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qstringlist.h>

#include "itemedit.h"

static const char* arrowdown_xpm[]={
    "17 16 4 1",
    "c c #98a6ff", "o c #000000", "@ c #000000", ". c None",
    "..................",
    "......oooooo......",
    "......oooooo......",
    "......oooooo......",
    "......oooooo......",
    "......oooooo......",
    "......oooooo......",
    "..oooooooooooooo..",
    "..oooooooooooooo..",
    "...oooooooooooo...",
    "....oooooooooo....",
    ".....oooooooo.....",
    "......oooooo......",
    ".......oooo.......",
    "........oo........",
    ".................."};

static const char* arrowup_xpm[]={
    "17 16 4 1",
    "c c #98a6ff", "o c #000000", "@ c #000000", ". c None",
    "..................",
    "........oo........",
    ".......oooo.......",
    "......oooooo......",
    ".....oooooooo.....",
    "....oooooooooo....",
    "...oooooooooooo...",
    "..oooooooooooooo..",
    "..oooooooooooooo..",
    "......oooooo......",
    "......oooooo......",
    "......oooooo......",
    "......oooooo......",
    "......oooooo......",
    "......oooooo......",
    ".................."};

ListEdit::ListEdit( QWidget* parent,  const char* name, WFlags fl )
    : QWidget( parent, name, fl )
{
   if ( !name )
      setName( "ListEdit" );

   resize( 478, 274 ); 
   setCaption( tr( "List Edit" ) );

   ListEditLayout = new QVBoxLayout( this );
   ListEditLayout->setMargin(11);
   ListEditLayout->setSpacing(6);

   list_v = new QListView( this, "list_v" );
   list_v->setAllColumnsShowFocus( true );
   list_v->setSorting(-1, false);
   list_v->header()->setClickEnabled(false);
   list_v->header()->setMovingEnabled( false );
   ListEditLayout->addWidget( list_v );

   Layout2 = new QHBoxLayout( ListEditLayout ); 
   Layout2->setSpacing( 6 );
   Layout2->setMargin( 0 );

   up_b = new QPushButton( this, "up_b" );
   up_b->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)1, up_b->sizePolicy().hasHeightForWidth() ) );
   up_b->setPixmap( arrowup_xpm );
   Layout2->addWidget( up_b );

   down_b = new QPushButton( this, "down_b" );
   down_b->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)1, down_b->sizePolicy().hasHeightForWidth() ) );
   down_b->setPixmap( arrowdown_xpm );
   Layout2->addWidget( down_b );

   QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
   Layout2->addItem( spacer );

   edit_b = new QPushButton( this, "edit_b" );
   edit_b->setText( tr( "Edit" ) );
   Layout2->addWidget( edit_b );

   new_b = new QPushButton( this, "new_b" );
   new_b->setText( tr( "New" ) );
   Layout2->addWidget( new_b );

   delete_b = new QPushButton( this, "delete_b" );
   delete_b->setText( tr( "Delete" ) );
   Layout2->addWidget( delete_b );

   // signals and slots connections
   connect( up_b,     SIGNAL( clicked() ), this, SLOT( upClicked() ) );
   connect( down_b,   SIGNAL( clicked() ), this, SLOT( downClicked() ) );
   connect( edit_b,   SIGNAL( clicked() ), this, SLOT( editClicked() ) );
   connect( new_b,    SIGNAL( clicked() ), this, SLOT( newClicked() ) );
   connect( delete_b, SIGNAL( clicked() ), this, SLOT( deleteClicked() ) );
   connect( list_v,   SIGNAL( doubleClicked( QListViewItem *) ), 
         this, SLOT( editClicked(QListViewItem *) ) );

   m_last = NULL;

   show();
}

ListEdit::~ListEdit()
{
}

void ListEdit::addColumn(const QString & title, int width = -1)
{
   list_v->addColumn( title, width );
}

void ListEdit::addItem( QStringList & list )
{
   QListViewItem *item = new QListViewItem(list_v, m_last);

   int n = 0;
   for ( QStringList::Iterator it = list.begin(); it != list.end(); it++ )
   {
      item->setText(n, *it);
      n++;
   }

   m_last = item;
}

void ListEdit::getItem(int n, QStringList & list )
{
   QListViewItem *item = list_v->firstChild();

   int cur = 0;
   while ( item && (cur < n) )
   {
      item = item->nextSibling();
      cur++;
   }

   list.clear();
   if ( item )
   {
      for ( int t = 0; t < list_v->columns(); t++ )
      {
         list << item->text(t);
      }
   }
}

int ListEdit::items(void)
{
   int cur = 0;
   QListViewItem *item = list_v->firstChild();

   while ( item )
   {
      cur++;
      item = item->nextSibling();
   }

   return cur;
}

void ListEdit::deleteClicked()
{
   if ( list_v->currentItem() )
   {
      delete list_v->currentItem();
   }
}

void ListEdit::editClicked()
{
   editClicked( list_v->currentItem() );
}

void ListEdit::editClicked( QListViewItem *item )
{
   if ( item )
   {
      ItemEdit edit(this, "");

      QStringList temp;
      for ( int n = 0; n < list_v->columns(); n++ )
      {
         temp << list_v->columnText( n );
      }
      edit.setLabels(temp);

      temp.clear();
      for ( int n = 0; n < list_v->columns(); n++ )
      {
         temp << item->text( n );
      }
      edit.setValues(temp);

      if ( edit.exec() )
      {
         temp.clear();
         edit.getValues( temp );

         int n = 0;
         for ( QStringList::Iterator it = temp.begin(); it != temp.end(); it++ )
         {
            item->setText(n, *it);
            n++;
         }
      }
   }
}

void ListEdit::newClicked()
{
   QListViewItem *item = new QListViewItem(list_v, m_last);
   list_v->setCurrentItem(item);

   ItemEdit edit(this, "");

   QStringList temp;
   for ( int n = 0; n < list_v->columns(); n++ )
   {
      temp << list_v->columnText( n );
   }
   edit.setLabels(temp);

   temp.clear();
   for ( int n = 0; n < list_v->columns(); n++ )
   {
      temp << "";
   }
   edit.setValues(temp);

   if ( edit.exec() )
   {
      temp.clear();
      edit.getValues( temp );

      int n = 0;
      for ( QStringList::Iterator it = temp.begin(); it != temp.end(); it++ )
      {
         item->setText(n, *it);
         n++;
      }

      m_last = item;
   }
   else
   {
      delete item;
   }
}

void ListEdit::downClicked()
{
   QListViewItem *item  = list_v->currentItem();
   QListViewItem *after = (item) ? item->nextSibling() : NULL;

   if ( item && after )
   {
      QListViewItem *create = new QListViewItem(list_v, after);
      list_v->setCurrentItem( create );
      list_v->ensureItemVisible( create );

      for( int t = 0; t < list_v->columns(); t++ )
      {
         create->setText(t, item->text(t) );
      }

      delete item;
   }
}

void ListEdit::upClicked()
{
   QListViewItem *item  = list_v->currentItem();
   QListViewItem *after = (item) ? item->itemAbove() : NULL;

   if ( after )
      after = after->itemAbove();
   else
      after = NULL;

   if ( item )
   {
      QListViewItem *create = new QListViewItem(list_v, after);
      list_v->setCurrentItem( create );
      list_v->ensureItemVisible( create );

      for( int t = 0; t < list_v->columns(); t++ )
      {
         create->setText(t, item->text(t) );
      }

      delete item;
   }
}

void ListEdit::clear(void)
{
   list_v->clear();
}

