/* llist.cpp - Linked list of strings
 * 
 *  Bronco File Manager
 * 
 *  Copyright (c) 2000, 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#include "llist.h"

LinkedList::LinkedList(void){
   top = cur = NULL;
   return;
}

LinkedList::~LinkedList(void){
   while(top){
      cur = top;
      top = cur->next;
      delete cur;
   }
   return;
}

int LinkedList::compare(const char *s1, const char *s2){
   return(strcasecmp(s1, s2));
}

int LinkedList::add(const char *s){
   LList *temp, *tcur, *last;
   temp = new LList;
   int rval;

   if(temp){
      temp->string = strdup(s);
      temp->next = NULL;

      tcur = top;
      last = NULL;

      while(tcur && ((rval=compare(s, tcur->string)) >= 0)){
         if(rval == 0){
            delete temp;
            return(1);
         }
         last = tcur;
         tcur = tcur->next;
      }

      if(last){
         if(tcur){
            temp->next = tcur;
            temp->prev = last;
            last->next = temp;
            tcur->prev = temp;
         }
         else{
            temp->prev = last;
            last->next = temp;
         }
      }
      else{
         if(tcur){
            temp->next = tcur;
            tcur->prev = temp;
         }
         top = temp;
      }
      return(1);
   }
   else{
      return(0);
   }
}

int LinkedList::del(const char *s){
   cur = top;
   while(cur && compare(s, cur->string)){
      cur = cur->next;
   }
   if(cur){
      return(del());
   }
   else{
      return(0);
   }
}

int LinkedList::del(void){
   LList *temp;
   if(cur){
      temp = cur;
      cur = cur->next;
      if(temp->prev){
         temp->prev->next = temp->next;
      }
      else{
         top = temp->next;
      }
      if(temp->next){
         temp->next->prev = temp->prev;
      }

      delete temp;
      return(1);
   }
   else{
      return(0);
   }
}

int LinkedList::first(char *s){
   cur = top;
   if(cur && s){
      strcpy(s, cur->string);
      return(1);
   }
   else{
      return(0);
   }
}

int LinkedList::next(char *s){
   if((cur == NULL) || (s == NULL)) return(0);

   cur = cur->next;
   if(cur){
      strcpy(s, cur->string);
      return(1);
   }
   else{
      return(0);
   }
   return(0);
}

int LinkedList::find(const char *s){
   cur = top;
   while(cur){
      if(compare(s, cur->string) == 0){
         return(1);
      }
      cur = cur->next;
   }
   return(0);
}

void LinkedList::print(){
   LList *tcur = top;
   while(tcur){
      printf("%s\n", tcur->string);
      tcur = tcur->next;
   }
   return;
}

void LinkedList::clear(){
   LList *tcur = top;
   while(tcur){
      top = tcur;
      tcur = tcur->next;
      delete top;
   }
   top = NULL;
   return;
}
