/* mainwin.h - Main widget for Bronco
 * 
 *  Bronco File Manager
 * 
 *  Copyright (c) 2000, 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#ifndef MAINWINH
#define MAINWINH

#include <stdio.h>
#include <signal.h>
#include <qapplication.h>
#include <qpushbutton.h>
#include <qlistview.h>
#include <qwindowsstyle.h>
#include <qheader.h>
#include <qpixmap.h>
#include <qpopupmenu.h>
#include <qframe.h>
#include <qlayout.h>
#include <qsplitter.h>
#include <qtimer.h>
#include <qprogressdialog.h>
#include <hash_map>

#ifdef USE_KDE
#include <kmenubar.h>
#else
#include <qmenubar.h>
#endif /* USE_KDE */

class DirList;
class DirTree;
class PrefWin;
class GlobWin;
class AboutWin;
class License;
class Using;
class FifoText;
class FifoList;

typedef hash_map<int, QProgressDialog *, hash<int> > ProgressHash;
typedef hash_map<int, FifoText *, hash<int> > FifoTextHash;
typedef hash_map<int, FifoList *, hash<int> > FifoListHash;
typedef hash_map<int, FILE *, hash<int> > FileHash;

class MainWin : public QWidget{
      Q_OBJECT

   public:
      MainWin(QWidget *, char *);
      void passArgs(int argc, char *argv[]);

      PrefWin * getPrefWin();

   public slots:
      void sortChange(int);
      void sortName(void);
      void sortSize(void);
      void sortPermissions(void);
      void sortModified(void);
      void sortAscii(void);
      void refresh(void);
      void updateCaption(const QString &);
      void dirlistNewDirectory(const QString &dir);
      void childTimeout();
      void childCleanup(int);
      void showErrorLogs(void);
      void changeFont(void);

   signals:
      void refreshDirectory(const QString &s);

   protected:
      void resizeEvent(QResizeEvent *);

   protected slots:
      void fifoTimeout(void);
      void closeFifoList(int);
      void closeFifoText(int);

   protected:
      int name_id;
      int size_id;
      int perm_id;
      int mod_id;
      int ascii_id;
      int rev_id;

      DirTree *tree;
      DirList *list;
      PrefWin *pref_win;
      GlobWin *glob_win;
      License *license_win;
      Using *using_win;
      AboutWin *about_win;

      static ProgressHash s_progressHash;
      static FileHash     s_fifoHash;
      static FifoTextHash s_fifoText;
      static FifoListHash s_fifoList;

#ifdef USE_KDE
      KMenuBar *menu_bar;
#else
      QMenuBar *menu_bar;
#endif /* USE_KDE */

      QPopupMenu *file_menu, *select_menu, *help_menu, *settings_menu, *sort_menu;
      QTimer *fifo_t;
      QTimer *child_t;
};

#endif /* MAINWINH */
