/* openwith.cpp - Modal prompt for "Open With..."
 * 
 *  Bronco File Manager
 * 
 *  Copyright (c) 2000, 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#include "openwith.h"
#include "llist.h"

OpenWith::OpenWith(QWidget *parent=NULL, char *name=NULL) : QDialog(parent, name, TRUE, 0){
   char g[80];
   char c[80];
   LinkedList cmdList;

   const char * last_string = prefs.getLastOpenWith();

   ok     = new QPushButton("Ok", this, NULL);
   cancel = new QPushButton("Cancel", this, NULL);

   text   = new QLabel("Open with what program?", this, NULL);
   input  = new QComboBox(TRUE, this, NULL);

   if(prefs.firstCommand(g, c)){
      do{
         cmdList.add(c);
      } while(prefs.nextCommand(g, c));
   }
   
   int index     = 0;
   int last_used = -1;

   if(cmdList.first(c))
   {
      do
      {
         input->insertItem(c, index);

         if ( last_string[0] && (strcmp(last_string, c) == 0) )
            last_used = index;

         index++;
      } while(cmdList.next(c));
   }

   if ( last_used >= 0 )
   {
      input->setCurrentItem( last_used );
   }
   else
   {
      input->insertItem( last_string, index);
      input->setCurrentItem(index);
   }

   connect(ok, SIGNAL(clicked()), this, SLOT(accept()));
   connect(cancel, SIGNAL(clicked()), this, SLOT(reject()));

   input->setFocus();

   ok->setDefault( TRUE );

   setMinimumSize(350, 105);
   resize(350, 105);
   return;
}

void OpenWith::resizeEvent(QResizeEvent *){
   text->move(5, 2);
   text->resize(this->width() - 10, text->height());
   input->move(5, this->height() - input->height() - 40);

   input->resize(this->width() - 10, input->height());
   ok->move(this->width() - 210, this->height()-35);
   cancel->move(this->width() - 105, this->height()-35);
}

QString OpenWith::getInput(void){
   return( input->currentText() );
}

