/* prefs.cpp - Class to handle preferences for Bronco
 * 
 *  Bronco File Manager
 * 
 *  Copyright (c) 2000, 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <ctype.h>
#include "find.h"
#include "direct.h"
#include "prefs.h"

Preferences::Preferences(){
   setHome(getenv("HOME"));
   for(int t=0; t<10; t++){
      savedir[t] = NULL;
   }
   xterm_command[0] = '\0';
   last_openwith[0] = '\0';
   font_size = 10;
}

Preferences::~Preferences(){
   command.clear();
   for(int t=0; t<10; t++){
      if(savedir[t]) free(savedir[t]);
   }
   return;
}

char *key_match(char *key, char *string){
   char keybuf[100];
   snprintf(keybuf, sizeof(keybuf), "%s: ", key);
   if(strncasecmp(keybuf, string, strlen(keybuf)) == 0){
      return(string + strlen(keybuf));
   }
   return(NULL);
}

void chomp(char *s){
   int n = (int) strlen(s) - 1;

   while(n >= 0 && (s[n] == '\n' || s[n] == '\r')){
      s[n] = '\0';
      n--;
   }
   return;
}

int Preferences::useDefault(void){
   addCommand("*.txt",  "gvim +$n $F");
   addCommand("*.cpp",  "gvim +$n $F");
   addCommand("*.h",    "gvim +$n $F");
   addCommand("*.c",    "gvim +$n $F");
   addCommand("*rc",    "gvim +$n $F");
   addCommand("*.pl",   "gvim +$n $F");
   addCommand("*.cgi",  "gvim +$n $F");
   addCommand("*.conf", "gvim +$n $F");
   addCommand("conf.*", "gvim +$n $F");
   addCommand("*.mp3",  "xmms $L");
   addCommand("*.wav",  "xmms $L");
   addCommand("*.mod",  "xmms $L");
   addCommand("*.xm",   "xmms $L");
   addCommand("*.jpg",  "ee $L");
   addCommand("*.jpeg", "ee $L");
   addCommand("*.xpm",  "ee $L");
   addCommand("*.gif",  "ee $L");
   addCommand("*.xwd",  "ee $L");
   addCommand("*.mov",  "xanim $F");
   addCommand("*.mpeg", "xanim $F");
   addCommand("*.mpg",  "xanim $F");
   addCommand("*.avi",  "xanim $F");

   find_prefs = (FIND_REC | FIND_CASE);
   grep_prefs = (FIND_REC | FIND_CASE);
   tree_opts  = (DIRS_ONLY);
   list_opts  = (DIRS_FIRST | SHOW_DOTS);
   sort_ascii = 1;

   number_shortcuts = true;
   show_errors      = true;

   day_first  = 0;
   four_digit = 0;

   bronco_x   = 600;
   bronco_y   = 400;

   float_x    = 420;
   float_y    = 330;

   find_x     = 450;
   find_y     = 330;

   conf_copy        = 0;
   conf_move        = 0;
   conf_delete      = 1;
   conf_overwrite   = 1;
   copy_continue    = 0;

   tar_comp         = TarGzip;
   tar_modification = true;
   tar_overwrite    = true;

   strncpy(xterm_command, "xterm", sizeof(xterm_command) );
   xterm_command[ sizeof(xterm_command) - 1] = '\0';
   grep_edit[0] = '\0';
   last_openwith[0] = '\0';

   snprintf(font_family, sizeof(font_family), "Courier");
   font_size = 10;

   launch_item.clear();
   launch_command.clear();

   save();
   return(0);
}

int Preferences::load(char *file){
   FILE *fp;
   char input[100];
   char *ptr, *glob, *com;
   int n;
   int numdirs;

   bronco_x   = 600;
   bronco_y   = 400;

   float_x    = 420;
   float_y    = 330;

   find_x     = 450;
   find_y     = 330;

   numdirs    = 0;

   show_errors      = true;
   number_shortcuts = true;

   list_opts  = 0;
   tree_opts  = DIRS_ONLY;
   sort_ascii = 1;

   conf_copy        = 0;
   conf_move        = 0;
   conf_delete      = 1;
   conf_overwrite   = 1;
   copy_continue    = 0;

   find_prefs       = 0;
   grep_prefs       = 0;

   tar_comp         = TarGzip;
   tar_modification = true;
   tar_overwrite    = true;

   xterm_command[0] = '\0';
   last_openwith[0] = '\0';

   snprintf(font_family, sizeof(font_family), "Courier");
   font_size = 10;

   if(file){
      if((fp = fopen(file, "r")) != NULL){
         while(fgets(input, sizeof(input), fp)){
            chomp(input);
            if((ptr = key_match("glob", input)) != NULL){

               glob    = ptr;
               com = NULL;
               for(n=0; ptr[n]; n++){
                  if(ptr[n] == '&'){
                     ptr[n] = '\0';
                     com = ptr + n + 1;
                     break;
                  }
               }

               if(com){
                  addCommand(glob, com);
               }
            }
            if((ptr = key_match("launch", input)) != NULL){

               glob    = ptr;
               com = NULL;
               for(n=0; ptr[n]; n++){
                  if(ptr[n] == '&'){
                     ptr[n] = '\0';
                     com = ptr + n + 1;
                     break;
                  }
               }

               if(com){
                  addLaunch(glob, com);
               }
            }
            if((ptr = key_match("xterm", input)) != NULL){
               while (*ptr && isspace(*ptr))
                  ptr++;
               strncpy(xterm_command, ptr, sizeof(xterm_command));
               xterm_command[ sizeof(xterm_command) - 1 ] = '\0';
            }
            if((ptr = key_match("lastopenwith", input)) != NULL){
               while (*ptr && isspace(*ptr))
                  ptr++;
               strncpy(last_openwith, ptr, sizeof(last_openwith));
               last_openwith[ sizeof(last_openwith) - 1 ] = '\0';
            }
            if((ptr = key_match("grepedit", input)) != NULL){
               while (*ptr && isspace(*ptr))
                  ptr++;
               strncpy(grep_edit, ptr, sizeof(grep_edit));
               grep_edit[ sizeof(grep_edit) - 1 ] = '\0';
            }
            if((ptr = key_match("fontfamily", input)) != NULL){
               while (*ptr && isspace(*ptr))
                  ptr++;
               strncpy(font_family, ptr, sizeof(font_family));
               font_family[ sizeof(font_family) - 1 ] = '\0';
            }
            if((ptr = key_match("fontsize", input)) != NULL){
               font_size = atoi(ptr);
            }
            if((ptr = key_match("broncoy", input)) != NULL){
               bronco_y = atoi(ptr);
            }
            if((ptr = key_match("broncox", input)) != NULL){
               bronco_x = atoi(ptr);
            }
            if((ptr = key_match("floatx", input)) != NULL){
               float_x = atoi(ptr);
            }
            if((ptr = key_match("floaty", input)) != NULL){
               float_y = atoi(ptr);
            }
            if((ptr = key_match("findx", input)) != NULL){
               find_x = atoi(ptr);
            }
            if((ptr = key_match("findy", input)) != NULL){
               find_y = atoi(ptr);
            }
            if((ptr = key_match("sortascii", input)) != NULL){
               sort_ascii = atoi(ptr) ? true : false;
            }
            if((ptr = key_match("sortalpha", input)) != NULL){
               sort_ascii = atoi(ptr) ? false : true;
            }
            if((ptr = key_match("greprec", input)) != NULL){
               grep_prefs |= atoi(ptr) ? FIND_REC : 0;
            }
            if((ptr = key_match("grepcase", input)) != NULL){
               grep_prefs |= atoi(ptr) ? FIND_CASE : 0;
            }
            if((ptr = key_match("findrec", input)) != NULL){
               find_prefs |= atoi(ptr) ? FIND_REC : 0;
            }
            if((ptr = key_match("findreg", input)) != NULL){
               find_prefs |= atoi(ptr) ? FIND_REG : 0;
            }
            if((ptr = key_match("findcase", input)) != NULL){
               find_prefs |= atoi(ptr) ? FIND_CASE : 0;
            }
            if((ptr = key_match("dirsfirst", input)) != NULL){
               list_opts  |= atoi(ptr) ? DIRS_FIRST : 0;
            }
            if((ptr = key_match("dirreverse", input)) != NULL){
               list_opts  |= atoi(ptr) ? DIR_REVERSE : 0;
            }
            if((ptr = key_match("showdotfiles", input)) != NULL){
               list_opts  |= atoi(ptr) ? SHOW_DOTS : 0;
            }
            if((ptr = key_match("treedots", input)) != NULL){
               tree_opts |= atoi(ptr) ? SHOW_DOTS : 0;
            }
            if((ptr = key_match("numbershortcuts", input)) != NULL){
               number_shortcuts = atoi(ptr) ? true : false;
            }
            if((ptr = key_match("showerrors", input)) != NULL){
               show_errors = atoi(ptr) ? true : false;
            }
            if((ptr = key_match("fourdigit", input)) != NULL){
               four_digit = atoi(ptr) ? true : false;
            }
            if((ptr = key_match("dayfirst", input)) != NULL){
               day_first = atoi(ptr) ? true : false;
            }
            if((ptr = key_match("confcopy", input)) != NULL){
               conf_copy = atoi(ptr) ? true : false;
            }
            if((ptr = key_match("confmove", input)) != NULL){
               conf_move = atoi(ptr) ? true : false;
            }
            if((ptr = key_match("confdelete", input)) != NULL){
               conf_delete = atoi(ptr) ? true : false;
            }
            if((ptr = key_match("confoverwrite", input)) != NULL){
               conf_overwrite = atoi(ptr) ? true : false;
            }
            if((ptr = key_match("copycontinue", input)) != NULL){
               copy_continue = atoi(ptr) ? true : false;
            }
            if((ptr = key_match("taroverwrite", input)) != NULL){
               tar_overwrite = atoi(ptr) ? true : false;
            }
            if((ptr = key_match("tarmodification", input)) != NULL){
               tar_modification = atoi(ptr) ? true : false;
            }
            if((ptr = key_match("tarcompression", input)) != NULL){
               tar_comp = (TarCompression) atoi(ptr);
            }
            if((ptr = key_match("dir", input)) != NULL){
               if(ptr[0] && !isspace(ptr[0])){
                  if(numdirs < 10){
                     setDir(numdirs, ptr);
                  }
               }
               numdirs++;
            }
         }

         fclose(fp);
         return(1);
      }
      else{
         fprintf(stderr, "Could not open preferences: %s\nUsing defaults...\n", strerror(errno));
         useDefault();
      }
   }

   if ( xterm_command[0] == '\0' )
   {
      strcpy(xterm_command, "xterm" );
   }

   return(0);
}

int Preferences::load(void){
   char file[1024];
   snprintf(file, sizeof(file), "%s/.bronco/broncorc", home_dir);
   return(load(file));
}

int Preferences::save(char *file){
   FILE *fp;
   int n;

   if(file){
      if((fp = fopen(file, "w")) != NULL){
         fprintf(fp, "BroncoX: %d\n",         bronco_x);
         fprintf(fp, "BroncoY: %d\n",         bronco_y);
         fprintf(fp, "FloatX: %d\n",          float_x);
         fprintf(fp, "FloatY: %d\n",          float_y);
         fprintf(fp, "FindX: %d\n",           find_x);
         fprintf(fp, "FindY: %d\n",           find_y);
         fprintf(fp, "SortAscii: %d\n",       sort_ascii);

         if ( xterm_command[0] )
            fprintf(fp, "Xterm: %s\n",        xterm_command);

         if ( grep_edit[0] )
            fprintf(fp, "GrepEdit: %s\n",     grep_edit);

         if ( last_openwith[0] )
            fprintf(fp, "LastOpenWith: %s\n", last_openwith);

         fprintf(fp, "FontFamily: %s\n",      font_family);
         fprintf(fp, "FontSize: %d\n",        font_size);
         fprintf(fp, "GrepRec: %d\n",         (grep_prefs & FIND_REC)     ? 1 : 0);
         fprintf(fp, "GrepCase: %d\n",        (grep_prefs & FIND_CASE)    ? 1 : 0);
         fprintf(fp, "FindReg: %d\n",         (find_prefs & FIND_REG)     ? 1 : 0);
         fprintf(fp, "FindRec: %d\n",         (find_prefs & FIND_REC)     ? 1 : 0);
         fprintf(fp, "FindCase: %d\n",        (find_prefs & FIND_CASE)    ? 1 : 0);
         fprintf(fp, "DirsFirst: %d\n",       (list_opts  & DIRS_FIRST)   ? 1 : 0);
         fprintf(fp, "DirReverse: %d\n",      (list_opts  & DIR_REVERSE)  ? 1 : 0);
         fprintf(fp, "ShowDotFiles: %d\n",    (list_opts  & SHOW_DOTS)    ? 1 : 0);
         fprintf(fp, "TreeDots: %d\n",        (tree_opts  & SHOW_DOTS)    ? 1 : 0);
         fprintf(fp, "DayFirst: %d\n",        (day_first)                 ? 1 : 0);
         fprintf(fp, "NumberShortcuts: %d\n", (number_shortcuts)          ? 1 : 0);
         fprintf(fp, "ShowErrors: %d\n",      (show_errors)               ? 1 : 0);
         fprintf(fp, "FourDigit: %d\n",       (four_digit)                ? 1 : 0);
         fprintf(fp, "ConfCopy: %d\n",        (conf_copy)                 ? 1 : 0);
         fprintf(fp, "ConfMove: %d\n",        (conf_move)                 ? 1 : 0);
         fprintf(fp, "ConfDelete: %d\n",      (conf_delete)               ? 1 : 0);
         fprintf(fp, "ConfOverWrite: %d\n",   (conf_overwrite)            ? 1 : 0);
         fprintf(fp, "CopyContinue: %d\n",    (copy_continue)             ? 1 : 0);
         fprintf(fp, "TarOverWrite: %d\n",    (tar_overwrite)             ? 1 : 0);
         fprintf(fp, "TarModification: %d\n", (tar_modification)          ? 1 : 0);
         fprintf(fp, "TarCompression: %d\n",  (int) tar_comp);

         for(n=0; n<10; n++)
            fprintf(fp, "Dir: %s\n", (savedir[n] ? savedir[n] : ""));
         
         for( unsigned t = 0; t < launch_item.count(); t++ )
            fprintf(fp, "Launch: %s&%s\n", launch_item[t].latin1(), launch_command[t].latin1());

         for(Glob::iterator n = command.begin(); n != command.end(); n++)
            fprintf(fp, "Glob: %s&%s\n", (const char *) (*n).first, (const char *) (*n).second);

         fclose(fp);
         return(1);
      }
   }
   return(0);
}

int Preferences::save(void){
   char file[1024];
   snprintf(file, sizeof(file), "%s/.bronco/broncorc", home_dir);
   return(save(file));
}

int Preferences::addCommand(const char *glob, const char *cmdString){
   Glob temp;

   QString g = glob;
   QString c = cmdString;

   command[g] = c;
   return(1);
}

void Preferences::setBroncoSize(int x, int y){
   bronco_x = x;
   bronco_y = y;
}

void Preferences::getBroncoSize(int *x, int *y){
   *x = bronco_x;
   *y = bronco_y;
}

void Preferences::setFloatSize(int x, int y){
   float_x = x;
   float_y = y;
}

void Preferences::getFloatSize(int *x, int *y){
   *x = float_x;
   *y = float_y;
}

void Preferences::setFindSize(int x, int y){
   find_x = x;
   find_y = y;
}

void Preferences::getFindSize(int *x, int *y){
   *x = find_x;
   *y = find_y;
}

int Preferences::getDirTreeOpts(void){
   return(tree_opts);
}

int Preferences::getDirListOpts(void){
   return(list_opts);
}

int Preferences::setDirTreeOpts(int n){
   tree_opts = n;
   return(0);
}

int Preferences::setDirListOpts(int n){
   list_opts = n;
   return(0);
}

int Preferences::getFindPrefs(void){
   return(find_prefs);
}

int Preferences::setFindPrefs(int p){
   find_prefs = p;
   return(0);
}

int Preferences::getGrepPrefs(void){
   return(grep_prefs);
}

int Preferences::setGrepPrefs(int p){
   grep_prefs = p;
   return(0);
}

int Preferences::asciiSort(void){
   return(sort_ascii ? SORT_ASCII : SORT_ALPHA);
}

void Preferences::setAsciiSort(bool o){
   sort_ascii = o;
   save();
}

void Preferences::deleteCommand(const char *string){
   setFindPrefs(0);
   command.erase(QString(string));
   return;
}

const char * Preferences::getCommand(const char *string){
   setFindPrefs(0);
   for(Glob::iterator n = command.begin(); n != command.end(); n++){
      if(match( (const char*) (*n).first, string)){
         return( (const char *) (*n).second);
      }
   }
   return(NULL);
}

int Preferences::setHome(const char *path){
   strncpy(home_dir, path, sizeof(home_dir));
   home_dir[ sizeof(home_dir) - 1 ] = 0;
   snprintf(fifo_filename, sizeof(fifo_filename), "%s/.bronco/fifo", home_dir);
   return(0);
}

const char *Preferences::homeDir(void){
   return((const char *) home_dir);
}

int Preferences::setGrepEdit(const char *path){
   strncpy(grep_edit, path, sizeof(grep_edit));
   grep_edit[ sizeof(grep_edit) - 1 ] = 0;
   return(0);
}

const char *Preferences::getGrepEdit(void){
   return((const char *) grep_edit);
}

int Preferences::setXterm(const char *path){
   strncpy(xterm_command, path, sizeof(xterm_command));
   xterm_command[ sizeof(xterm_command) - 1 ] = 0;
   return(0);
}

const char *Preferences::getXterm(void){
   return((const char *) xterm_command);
}

int Preferences::setLastOpenWith(const char *cmd){
   strncpy(last_openwith, cmd, sizeof(last_openwith));
   last_openwith[ sizeof(last_openwith) - 1 ] = 0;
   return(0);
}

const char *Preferences::getLastOpenWith(void){
   return((const char *) last_openwith);
}

int Preferences::setFourDigitYear(bool o){
   four_digit = o;
   return(0);
}

bool Preferences::getFourDigitYear(void){
   return(four_digit);
}

int Preferences::setDayFirst(bool o){
   day_first = o;
   return(0);
}

bool Preferences::getDayFirst(void){
   return(day_first);
}

const char * Preferences::getDir(int d){
   if(savedir[d]) 
      return(savedir[d]);
   return("");
}

int Preferences::setDir(int d, const char *dir){
   if(d<0 && d>9) return(-1);

   if(savedir[d]) free(savedir[d]);
   savedir[d] = strdup(dir);

   if(savedir[d]) return(0);
   else           return(-1);
}

int Preferences::confCopy(void){
   return(conf_copy);
}

int Preferences::setConfCopy(bool o){
   conf_copy = o;
   return(0);
}

int Preferences::confMove(void){
   return(conf_move);
}

int Preferences::setConfMove(bool o){
   conf_move = o;
   return(0);
}

int Preferences::confDelete(void){
   return(conf_delete);
}

int Preferences::setConfDelete(bool o){
   conf_delete = o;
   return(0);
}

int Preferences::confOverWrite(void){
   return(conf_overwrite);
}

int Preferences::setConfOverWrite(bool o){
   conf_overwrite = o;
   return(0);
}

int Preferences::copyContinue(void){
   return(copy_continue);
}

int Preferences::setCopyContinue(bool o){
   copy_continue = o;
   return(0);
}

int Preferences::firstCommand(char *g, char *c){
   curglob = command.begin();
   if(curglob != command.end()){
      strcpy(g, (const char*) (*curglob).first);
      strcpy(c, (const char*) (*curglob).second);
      return(1);
   }
   return(0);
}

int Preferences::nextCommand(char *g, char *c){
   curglob++;
   if(curglob != command.end()){
      strcpy(g, (const char*) (*curglob).first);
      strcpy(c, (const char*) (*curglob).second);
      return(1);
   }
   return(0);
}

bool Preferences::getTarOverwrite(void)
{
   return tar_overwrite;
}

bool Preferences::getTarModification(void)
{
   return tar_modification;
}

TarCompression Preferences::getTarCompression(void)
{
   return tar_comp;
}

void Preferences::setTarOverwrite(bool o)
{
   tar_overwrite = o;
}

void Preferences::setTarModification(bool o)
{
   tar_modification = o;
}

void Preferences::setTarCompression(TarCompression t)
{
   tar_comp = t;
}

const char * Preferences::getFifoFileName()
{
   return fifo_filename;
}

void Preferences::addLaunch(const QString & item, const QString & command)
{
   launch_item    << item;
   launch_command << command;
}

void Preferences::getLaunch(unsigned n, QString & item, QString & command)
{
   if ( n < launch_item.count() )
   {
      item    = launch_item[n];
      command = launch_command[n];
   }
}

int Preferences::launches(void)
{
   return launch_item.count();
}

void Preferences::clearLaunches(void)
{
   launch_item.clear();
   launch_command.clear();
}

void Preferences::setShowErrors(bool o)
{
   show_errors = o;
}

bool Preferences::getShowErrors(void)
{
   return show_errors;
}

void Preferences::setNumberShortcuts(bool o)
{
   number_shortcuts = o;
}

bool Preferences::getNumberShortcuts(void)
{
   return number_shortcuts;
}

void Preferences::setFont(const char *family, int size)
{
   font_size = size;
   snprintf(font_family, sizeof(font_family), "%s", family);
}

void Preferences::getFont(char *family, int *size)
{
   *size = font_size;
   snprintf(family, sizeof(font_family), "%s", font_family);
}

