/* prefs.h - Class to handle preferences for Bronco
 * 
 *  Bronco
 * 
 *  Copyright (c) 2000, 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#ifndef PREFS_H
#define PREFS_H

#ifndef MAX_GLOBS
#define MAX_GLOBS 100
#endif

#include <qstring.h>
#include <qstringlist.h>
#include <map>

void chomp(char *);

struct command_glob{
   char *glob;
   char *cmdString;
};

struct ltqstr {
   bool operator()(QString s1, QString s2) {
      return (s1 < s2);
   }
};

typedef map<QString, QString, ltqstr> Glob;

enum TarCompression 
{
   TarGzip     = 0,
   TarBzip2    = 1,
   TarCompress = 2,
   TarNone     = 3
};

class Preferences {

   public:
      Preferences();
      virtual ~Preferences();

      int  load(char *);
      int  load(void);
      int  save(char *);
      int  save(void);

      void setBroncoSize(int, int);
      void getBroncoSize(int *, int *);
      void setFloatSize(int, int);
      void getFloatSize(int *, int *);
      void setFindSize(int, int);
      void getFindSize(int *, int *);

      int  addCommand(const char *, const char *);
      const char *getCommand(const char *);
      int  firstCommand(char *, char *);
      int  nextCommand(char *, char *);
      void deleteCommand(const char *);

      void setNumberShortcuts(bool);
      bool getNumberShortcuts(void);

      void setShowErrors(bool);
      bool getShowErrors(void);

      int  getDirListOpts(void);
      int  getDirTreeOpts(void);
      int  setDirListOpts(int);
      int  setDirTreeOpts(int);

      int  getFindPrefs();
      int  setFindPrefs(int);
      int  getGrepPrefs();
      int  setGrepPrefs(int);

      void setTarOverwrite(bool);
      bool getTarOverwrite(void);

      void setTarModification(bool);
      bool getTarModification(void);

      void getFont(char *family, int *size);
      void setFont(const char *family, int size);

      void setTarCompression( TarCompression );
      TarCompression getTarCompression( void );

      int  asciiSort(void);
      void setAsciiSort(bool);
      int  setHome(const char *);

      int setLastOpenWith(const char *);
      const char * getLastOpenWith(void);

      int  setFourDigitYear(bool);
      bool getFourDigitYear(void);
      int  setDayFirst(bool);
      bool getDayFirst(void);

      int copyContinue(void);
      int setCopyContinue(bool);
      int confOverWrite(void);
      int setConfOverWrite(bool);
      int confDelete(void);
      int setConfDelete(bool);
      int confMove(void);
      int setConfMove(bool);
      int confCopy(void);
      int setConfCopy(bool);

      const char * homeDir(void);
      int useDefault(void);

      int setXterm(const char *);
      const char * getXterm(void);
      int setGrepEdit(const char *);
      const char * getGrepEdit(void);

      int setDir(int, const char *);
      const char * getDir(int);

      const char *getFifoFileName(void);

      void addLaunch(const QString &, const QString &);
      void getLaunch(unsigned, QString &, QString &);
      int  launches(void);
      void clearLaunches(void);


   protected:

      Glob command;
      Glob::iterator curglob;

      int bronco_x, bronco_y;
      int float_x, float_y;
      int find_x, find_y;

      char *savedir[10];

      int list_opts;
      int tree_opts;
      int sort_ascii;
      int find_prefs;
      int grep_prefs;

      bool number_shortcuts;

      bool show_errors;

      bool day_first;
      bool four_digit;

      bool conf_copy;
      bool conf_move;
      bool conf_delete;
      bool conf_overwrite;
      bool copy_continue;

      TarCompression tar_comp;
      bool tar_overwrite;
      bool tar_modification;

      char home_dir[256];
      char xterm_command[256];
      char grep_edit[256];
      char fifo_filename[256];
      char last_openwith[256];

      char font_family[64];
      int  font_size;

      QStringList launch_item;
      QStringList launch_command;
};

#endif /* PREFS_H */
