/* prefwin.cpp - Bronco Preferences Window
 * 
 *  Bronco File Manager
 * 
 *  Copyright (c) 2000, 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qtabwidget.h>
#include <qwidget.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qfontdialog.h>

#include "direct.h"
#include "prefwin.h"
#include "bronco.h"
#include "find.h"
#include "listedit.h"

PrefWin::PrefWin(QWidget *parent, char *name) : QDialog(parent, name, false, 0)
{
   if ( !name )
      setName( "PrefWin" );

   resize( 570, 350 ); 
   setCaption( tr( "Bronco Preferences" ) );
   PrefWinLayout = new QVBoxLayout( this ); 
   PrefWinLayout->setSpacing( 6 );
   PrefWinLayout->setMargin( 11 );

   prefs_tab = new QTabWidget( this, "prefs_tab" );

   tab = new QWidget( prefs_tab, "tab" );
   tabLayout = new QVBoxLayout( tab ); 
   tabLayout->setSpacing( 6 );
   tabLayout->setMargin( 11 );

   date_g = new QButtonGroup( tab, "date_g" );
   date_g->setTitle( tr( "Date Format" ) );
   date_g->setColumnLayout(0, Qt::Vertical );
   date_g->layout()->setSpacing( 0 );
   date_g->layout()->setMargin( 0 );
   date_gLayout = new QVBoxLayout( date_g->layout() );
   date_gLayout->setAlignment( Qt::AlignTop );
   date_gLayout->setSpacing( 6 );
   date_gLayout->setMargin( 11 );

   dayfirst_box = new QCheckBox( date_g, "dayfirst_box" );
   dayfirst_box->setText( tr( "Day Before Year (dd/mm/yy)" ) );
   date_gLayout->addWidget( dayfirst_box );

   fourdigit_box = new QCheckBox( date_g, "fourdigit_box" );
   fourdigit_box->setText( tr( "Four-Digit Year" ) );
   date_gLayout->addWidget( fourdigit_box );
   tabLayout->addWidget( date_g );

   sort_g = new QButtonGroup( tab, "sort_g" );
   sort_g->setTitle( tr( "Sorting" ) );
   sort_g->setColumnLayout(0, Qt::Vertical );
   sort_g->layout()->setSpacing( 0 );
   sort_g->layout()->setMargin( 0 );
   sort_gLayout = new QVBoxLayout( sort_g->layout() );
   sort_gLayout->setAlignment( Qt::AlignTop );
   sort_gLayout->setSpacing( 6 );
   sort_gLayout->setMargin( 11 );

   asciion_box = new QRadioButton( sort_g, "asciion_box" );
   asciion_box->setText( tr( "ASCII-betical" ) );
   sort_gLayout->addWidget( asciion_box );

   asciioff_box = new QRadioButton( sort_g, "asciioff_box" );
   asciioff_box->setText( tr( "Alphabetical" ) );
   sort_gLayout->addWidget( asciioff_box );
   tabLayout->addWidget( sort_g );
   prefs_tab->insertTab( tab, tr( "Global" ) );

   tab_2 = new QWidget( prefs_tab, "tab_2" );
   tabLayout_2 = new QVBoxLayout( tab_2 ); 
   tabLayout_2->setSpacing( 6 );
   tabLayout_2->setMargin( 11 );

   dir_g = new QButtonGroup( tab_2, "dir_g" );
   dir_g->setTitle( tr( "Directory Options" ) );
   dir_g->setColumnLayout(0, Qt::Vertical );
   dir_g->layout()->setSpacing( 0 );
   dir_g->layout()->setMargin( 0 );
   dir_gLayout = new QVBoxLayout( dir_g->layout() );
   dir_gLayout->setAlignment( Qt::AlignTop );
   dir_gLayout->setSpacing( 6 );
   dir_gLayout->setMargin( 11 );

   dirsfirst_box = new QCheckBox( dir_g, "dirsfirst_box" );
   dirsfirst_box->setText( tr( "Group Directories" ) );
   dir_gLayout->addWidget( dirsfirst_box );

   dirreverse_box = new QCheckBox( dir_g, "dirreverse_box" );
   dirreverse_box->setText( tr( "List Directories Last When Reversed" ) );
   dir_gLayout->addWidget( dirreverse_box );

   showdot_box = new QCheckBox( dir_g, "showdot_box" );
   showdot_box->setText( tr( "Show Files And Directories Starting With '.' In Directory List" ) );
   dir_gLayout->addWidget( showdot_box );

   treedots_box = new QCheckBox( dir_g, "treedots_box" );
   treedots_box->setText( tr( "Show Directories Starting with '.' in Directory Tree" ) );
   dir_gLayout->addWidget( treedots_box );

   showerrors_v = new QCheckBox( dir_g, "showerrors_v" );
   showerrors_v->setText( tr( "Show Error Logs" ) );
   dir_gLayout->addWidget( showerrors_v );

   numbershortcuts_v = new QCheckBox( dir_g, "number_shortcuts_box" );
   numbershortcuts_v->setText( tr( "Use Numeric Directory Shortcuts" ) );
   dir_gLayout->addWidget( numbershortcuts_v );
   tabLayout_2->addWidget( dir_g );

   Layout6 = new QHBoxLayout; 
   Layout6->setSpacing( 6 );
   Layout6->setMargin( 0 );

   font_g = new QFrame( tab_2, "font_g" );
   font_g->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)5, font_g->sizePolicy().hasHeightForWidth() ) );
   font_g->setFrameShape( QFrame::Box );
   font_g->setFrameShadow( QFrame::Sunken );
   font_gLayout = new QHBoxLayout( font_g ); 
   font_gLayout->setSpacing( 6 );
   font_gLayout->setMargin( 11 );

   prefs.getFont(family, &fontsize_v);

   font_v = new QLabel( font_g, "font_v" );
   QFont font_v_font(  font_v->font() );
   font_v_font.setFamily( family );
   font_v_font.setPointSize( fontsize_v );
   font_v->setFont( font_v_font ); 
   font_v->setText( "Directory Font" );
   font_gLayout->addWidget( font_v );
   Layout6->addWidget( font_g );

   font_b = new QPushButton( tab_2, "font_b" );
   font_b->setText( tr( "Change Font" ) );
   Layout6->addWidget( font_b );
   tabLayout_2->addLayout( Layout6 );

   prefs_tab->insertTab( tab_2, tr( "Directory" ) );

   tab_3 = new QWidget( prefs_tab, "tab_3" );
   tabLayout_3 = new QVBoxLayout( tab_3 ); 
   tabLayout_3->setSpacing( 6 );
   tabLayout_3->setMargin( 11 );

   confirm_g = new QButtonGroup( tab_3, "confirm_g" );
   confirm_g->setTitle( tr( "Confirm" ) );
   confirm_g->setColumnLayout(0, Qt::Vertical );
   confirm_g->layout()->setSpacing( 0 );
   confirm_g->layout()->setMargin( 0 );
   confirm_gLayout = new QGridLayout( confirm_g->layout() );
   confirm_gLayout->setAlignment( Qt::AlignTop );
   confirm_gLayout->setSpacing( 6 );
   confirm_gLayout->setMargin( 11 );

   copy_box = new QCheckBox( confirm_g, "copy_box" );
   copy_box->setText( tr( "Copy" ) );

   confirm_gLayout->addWidget( copy_box, 0, 0 );

   overwrite_box = new QCheckBox( confirm_g, "overwrite_box" );
   overwrite_box->setText( tr( "Overwrite" ) );

   confirm_gLayout->addWidget( overwrite_box, 1, 1 );

   delete_box = new QCheckBox( confirm_g, "delete_box" );
   delete_box->setText( tr( "Delete" ) );

   confirm_gLayout->addWidget( delete_box, 0, 1 );

   move_box = new QCheckBox( confirm_g, "move_box" );
   move_box->setText( tr( "Move" ) );

   confirm_gLayout->addWidget( move_box, 1, 0 );
   tabLayout_3->addWidget( confirm_g );

   continue_box = new QCheckBox( tab_3, "continue_box" );
   continue_box->setText( tr( "Continue On Errors" ) );
   tabLayout_3->addWidget( continue_box );
   prefs_tab->insertTab( tab_3, tr( "File Operations" ) );

   tab_5 = new QWidget( prefs_tab, "tab_5" );
   tabLayout_5 = new QVBoxLayout( tab_5 ); 
   tabLayout_5->setSpacing( 6 );
   tabLayout_5->setMargin( 11 );

   extract_g = new QButtonGroup( tab_5, "extract_g" );
   extract_g->setTitle( tr( "Extract Defaults" ) );
   extract_g->setColumnLayout(0, Qt::Vertical );
   extract_g->layout()->setSpacing( 0 );
   extract_g->layout()->setMargin( 0 );
   extract_gLayout = new QVBoxLayout( extract_g->layout() );
   extract_gLayout->setAlignment( Qt::AlignTop );
   extract_gLayout->setSpacing( 6 );
   extract_gLayout->setMargin( 11 );

   taroverwrite_v = new QCheckBox( extract_g, "taroverwrite_v" );
   taroverwrite_v->setText( tr( "Overwrite Existing Files" ) );
   extract_gLayout->addWidget( taroverwrite_v );

   tarmodification_v = new QCheckBox( extract_g, "tarmodification_v" );
   tarmodification_v->setText( tr( "Extract Modication Time" ) );
   extract_gLayout->addWidget( tarmodification_v );
   tabLayout_5->addWidget( extract_g );

   create_g = new QButtonGroup( tab_5, "create_g" );
   create_g->setTitle( tr( "Creation Defaults" ) );
   create_g->setColumnLayout(0, Qt::Vertical );
   create_g->layout()->setSpacing( 0 );
   create_g->layout()->setMargin( 0 );
   create_gLayout = new QHBoxLayout( create_g->layout() );
   create_gLayout->setAlignment( Qt::AlignTop );
   create_gLayout->setSpacing( 6 );
   create_gLayout->setMargin( 11 );

   compression_l = new QLabel( create_g, "compression_l" );
   compression_l->setText( tr( "Compression:" ) );
   create_gLayout->addWidget( compression_l );

   compression_v = new QComboBox( false, create_g, "compression_v" );
   compression_v->insertItem( tr( "gzip" ), TarGzip );
   compression_v->insertItem( tr( "bzip2" ), TarBzip2 );
   compression_v->insertItem( tr( "compress" ), TarCompress );
   compression_v->insertItem( tr( "none" ), TarNone );
   compression_v->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)0, compression_v->sizePolicy().hasHeightForWidth() ) );
   create_gLayout->addWidget( compression_v );
   tabLayout_5->addWidget( create_g );
   prefs_tab->insertTab( tab_5, tr( "Archive" ) );

   tab_launch = new QWidget( prefs_tab, "tab_launch" );
   tabLayout_launch = new QGridLayout( tab_launch ); 
   tabLayout_launch->setSpacing( 6 );
   tabLayout_launch->setMargin( 11 );

   launch_v = new ListEdit( tab_launch, "launch_v" );
   launch_v->addColumn("Menu Item");
   launch_v->addColumn("Program");
   tabLayout_launch->addWidget( launch_v, 0, 0 );

   prefs_tab->insertTab( tab_launch, tr( "Custom Commands" ) );

   tab_6 = new QWidget( prefs_tab, "tab_6" );
   tabLayout_6 = new QGridLayout( tab_6 ); 
   tabLayout_6->setSpacing( 6 );
   tabLayout_6->setMargin( 11 );

   xterm_v = new QLineEdit( tab_6, "xterm_v" );
   tabLayout_6->addWidget( xterm_v, 0, 1 );

   xterm_l = new QLabel( tab_6, "xterm_l" );
   xterm_l->setText( tr( "XTerm:" ) );
   tabLayout_6->addWidget( xterm_l, 0, 0 );

   grep_v = new QLineEdit( tab_6, "grep_v" );
   tabLayout_6->addWidget( grep_v, 1, 1 );

   grep_l = new QLabel( tab_6, "grep_l" );
   grep_l->setText( tr( "Grep Edit:" ) );
   tabLayout_6->addWidget( grep_l, 1, 0 );

   QSpacerItem *spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
   tabLayout_6->addItem( spacer );

   prefs_tab->insertTab( tab_6, tr( "Programs" ) );
   PrefWinLayout->addWidget( prefs_tab );

   Layout5 = new QHBoxLayout; 
   Layout5->setSpacing( 6 );
   Layout5->setMargin( 0 );
   QSpacerItem* spacer_2 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
   Layout5->addItem( spacer_2 );

   ok_button = new QPushButton( this, "ok_button" );
   ok_button->setText( tr( "Ok" ) );
   ok_button->setDefault( true );
   Layout5->addWidget( ok_button );

   cancel_button = new QPushButton( this, "cancel_button" );
   cancel_button->setText( tr( "Cancel" ) );
   Layout5->addWidget( cancel_button );
   PrefWinLayout->addLayout( Layout5 );

   connect( font_b, SIGNAL( clicked() ), this, SLOT( changeFont() ) );
   connect( ok_button, SIGNAL( clicked() ), this, SLOT( okPressed() ) );
   connect( cancel_button, SIGNAL( clicked() ), this, SLOT( cancelPressed() ) );

   newFont = false;

   return;
}

PrefWin::~PrefWin(void){
   return;
}

void PrefWin::openWindow(void){
   int sort_policy = prefs.getDirListOpts();

   newFont = false;

   continue_box->setChecked(        prefs.copyContinue() );
   overwrite_box->setChecked(       prefs.confOverWrite() );
   delete_box->setChecked(          prefs.confDelete() );
   move_box->setChecked(            prefs.confMove() );
   copy_box->setChecked(            prefs.confCopy() );
   dayfirst_box->setChecked(        prefs.getDayFirst() );
   fourdigit_box->setChecked(       prefs.getFourDigitYear() );
   dirsfirst_box->setChecked(       sort_policy & DIRS_FIRST);
   dirreverse_box->setChecked(      sort_policy & DIR_REVERSE);
   showdot_box->setChecked(         sort_policy & SHOW_DOTS);
   treedots_box->setChecked(        prefs.getDirTreeOpts() & SHOW_DOTS);
   showerrors_v->setChecked(        prefs.getShowErrors() );
   numbershortcuts_v->setChecked(   prefs.getNumberShortcuts() );

   taroverwrite_v->setChecked(      prefs.getTarOverwrite() );
   tarmodification_v->setChecked(   prefs.getTarModification() );
   compression_v->setCurrentItem(   (int) prefs.getTarCompression() );
   xterm_v->setText(                prefs.getXterm() );
   grep_v->setText(                 prefs.getGrepEdit() );

   prefs.getFont(family, &fontsize_v);

   launch_v->clear();
   for ( int n = 0; n < prefs.launches(); n++ )
   {
      QString item;
      QString command;
      QStringList pair;

      prefs.getLaunch(n, item, command);

      pair << item;
      pair << command;
      launch_v->addItem( pair );
   }

   if(prefs.asciiSort()==SORT_ASCII) asciion_box->setChecked(true); 
   else                              asciioff_box->setChecked(true);
   
   show();
   return;
}

void PrefWin::cancelPressed(void){
   hide();
   return;
}

void PrefWin::okPressed(void){
   int listopts = 0;
   int treeopts = 0;

   prefs.setCopyContinue( continue_box->isChecked() );
   prefs.setConfOverWrite( overwrite_box->isChecked() );
   prefs.setConfDelete( delete_box->isChecked() );
   prefs.setConfMove( move_box->isChecked() );
   prefs.setConfCopy( copy_box->isChecked() );
   prefs.setDayFirst( dayfirst_box->isChecked() );
   prefs.setFourDigitYear( fourdigit_box->isChecked() );
   prefs.setAsciiSort( asciion_box->isChecked() );
   prefs.setTarModification( tarmodification_v->isChecked() );
   prefs.setTarOverwrite( taroverwrite_v->isChecked() );
   prefs.setTarCompression( (TarCompression) compression_v->currentItem() );
   prefs.setShowErrors( showerrors_v->isChecked() );
   prefs.setNumberShortcuts( numbershortcuts_v->isChecked() );

   if(treedots_box->isChecked())    treeopts = (treeopts | SHOW_DOTS);
   if(dirsfirst_box->isChecked())   listopts = (listopts | DIRS_FIRST);
   if(dirreverse_box->isChecked())  listopts = (listopts | DIR_REVERSE);
   if(showdot_box->isChecked())     listopts = (listopts | SHOW_DOTS);

   prefs.setDirTreeOpts(treeopts);
   prefs.setDirListOpts(listopts);

   if( strcmp( prefs.getXterm(), (xterm_v->text()).latin1()) != 0)
   {
      prefs.setXterm((xterm_v->text()).latin1());
   }

   if( strcmp( prefs.getGrepEdit(), (grep_v->text()).latin1()) != 0)
   {
      prefs.setGrepEdit((grep_v->text()).latin1());
   }

   /*
   int s = 0;
   s |= findregex_v->isChecked()     ? FIND_REG : 0;
   s |= findrecursive_v->isChecked() ? FIND_REC : 0;
   s |= findcase_v->isChecked()      ? FIND_CASE : 0;
   prefs.setFindPrefs(s);
   */

   prefs.clearLaunches();
   for ( int n = 0; n < launch_v->items(); n++ )
   {
      QString item;
      QString command;
      QStringList pair;

      launch_v->getItem(n, pair);

      item    = pair[0];
      command = pair[1];

      prefs.addLaunch(item, command);
   }

   prefs.setFont( family, fontsize_v );

   prefs.save();
   hide();
   emit preferencesChanged();

   if ( newFont )
   {
      emit fontChanged();
   }
   return;
}

void PrefWin::changeFont()
{
   bool ok = true;

   QFont font(family, fontsize_v);

   font = QFontDialog::getFont(&ok, font, this, "");
   if ( ok )
   {
      snprintf(family, sizeof(family), "%s", (font.family()).latin1());
      fontsize_v = font.pointSize();

      font_v->setFont( font );

      newFont = true;
   }
}
