/* propwin.cpp - Properties dialog window
 * 
 *  Bronco File Manager
 * 
 *  Copyright (c) 2000, 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#include <qstring.h>
#include <qlabel.h>
#include <sys/types.h>
#include <dirent.h>
#include <qmessagebox.h>
#include "propwin.h"

#define BLK_SIZE 512

PropWin::PropWin() : QWidget(NULL, "", WDestructiveClose){
   setCaption("Properties");

   grid = new QGridLayout(this, 0, 0, 5, 2, "");

   fname_l   = new QLabel("Name:", this, "");
   files_l   = new QLabel("Files:", this, "");
   size_l    = new QLabel("Size:", this, "");
   bsize_l   = new QLabel("Size (blocks):", this, "");
   ctime_l   = new QLabel("Creation Time:", this, "");
   mtime_l   = new QLabel("Last Modified:", this, "");
   atime_l   = new QLabel("Last Accessed:", this, "");

   read_l    = new QLabel("Read", this, "");
   write_l   = new QLabel("Write", this, "");
   exec_l    = new QLabel("Exec", this, "");
   sticky_l  = new QLabel("Sticky/ID", this, "");

   user_l    = new QLabel("User:", this, "");
   group_l   = new QLabel("Group:", this, "");
   other_l   = new QLabel("Other:", this, "");

   user_r    = new QCheckBox(this, "");
   user_w    = new QCheckBox(this, "");
   user_x    = new QCheckBox(this, "");
   group_r   = new QCheckBox(this, "");
   group_w   = new QCheckBox(this, "");
   group_x   = new QCheckBox(this, "");
   other_r   = new QCheckBox(this, "");
   other_w   = new QCheckBox(this, "");
   other_x   = new QCheckBox(this, "");
   sticky_u  = new QCheckBox(this, "");
   sticky_g  = new QCheckBox(this, "");
   sticky_o  = new QCheckBox(this, "");

   user_r    ->setEnabled(FALSE);
   user_w    ->setEnabled(FALSE);
   user_x    ->setEnabled(FALSE);
   group_r   ->setEnabled(FALSE);
   group_w   ->setEnabled(FALSE);
   group_x   ->setEnabled(FALSE);
   other_r   ->setEnabled(FALSE);
   other_w   ->setEnabled(FALSE);
   other_x   ->setEnabled(FALSE);
   sticky_u  ->setEnabled(FALSE);
   sticky_g  ->setEnabled(FALSE);
   sticky_o  ->setEnabled(FALSE);

   fname_v  = new QLabel("0", this, "");
   files_v  = new QLabel("0", this, "");
   size_v   = new QLabel("0", this, "");
   bsize_v  = new QLabel("0", this, "");
   ctime_v  = new QLabel("0", this, "");
   mtime_v  = new QLabel("0", this, "");
   atime_v  = new QLabel("0", this, "");

   grid->addWidget(fname_l,  0, 0, 0);
   grid->addWidget(files_l,  1, 0, 0);
   grid->addWidget(size_l,   2, 0, 0);
   grid->addWidget(bsize_l,  3, 0, 0);
   grid->addWidget(ctime_l,  4, 0, 0);
   grid->addWidget(mtime_l,  5, 0, 0);
   grid->addWidget(atime_l,  6, 0, 0);

   grid->addWidget(read_l,   7, 1, 0);
   grid->addWidget(write_l,  7, 2, 0);
   grid->addWidget(exec_l,   7, 3, 0);
   grid->addWidget(sticky_l, 7, 4, 0);

   grid->addWidget(user_l,   8, 0, 0);
   grid->addWidget(group_l,  9, 0, 0);
   grid->addWidget(other_l, 10, 0, 0);

   grid->addMultiCellWidget(fname_v,  0, 0, 1, 4, 0);
   grid->addMultiCellWidget(files_v,  1, 1, 1, 4, 0);
   grid->addMultiCellWidget(size_v,   2, 2, 1, 4, 0);
   grid->addMultiCellWidget(bsize_v,  3, 3, 1, 4, 0);
   grid->addMultiCellWidget(ctime_v,  4, 4, 1, 4, 0);
   grid->addMultiCellWidget(mtime_v,  5, 5, 1, 4, 0);
   grid->addMultiCellWidget(atime_v,  6, 6, 1, 4, 0);
   
   grid->addWidget(user_r,    8, 1, 0);
   grid->addWidget(user_w,    8, 2, 0);
   grid->addWidget(user_x,    8, 3, 0);
   grid->addWidget(group_r,   9, 1, 0);
   grid->addWidget(group_w,   9, 2, 0);
   grid->addWidget(group_x,   9, 3, 0);
   grid->addWidget(other_r,  10, 1, 0);
   grid->addWidget(other_w,  10, 2, 0);
   grid->addWidget(other_x,  10, 3, 0);
   grid->addWidget(sticky_u,  8, 4, 0);
   grid->addWidget(sticky_g,  9, 4, 0);
   grid->addWidget(sticky_o, 10, 4, 0);

   close = new QPushButton("Close", this, "Close");
   close->setDefault(true);
   close->setFocus();
   connect(close, SIGNAL(clicked()), this, SLOT(close()));
   grid->addMultiCellWidget(close,  11, 11, 0, 4, AlignCenter);

   close->setFocus();

   clear();
}

/* 
 * We actually set the values of the widgets here because the
 * values can change so much between creation and display
 */
void PropWin::show(void){
   char *str;
   QString s;
   char string[30];

   s = fname;
   fname_v->setText(s);

   s.setNum(files);
   files_v->setText(s);

   /* Qt doesn't support %llu internally... and neither does standard C++ */
   snprintf(string, sizeof(string), "%llu", (unsigned long long) size);
   size_v->setText(string);

   /* Qt doesn't support %llu internally... and neither does standard C++ */
   snprintf(string, sizeof(string), "%llu", (unsigned long long) bsize);
   bsize_v->setText(string);

   str = strdup(ctime(&ct));
   for(int t=0; str[t]; t++)
      if(str[t] == '\n' || str[t] == '\r')
         str[t] = '\0';
   ctime_v->setText(str);
   free(str);

   str = strdup(ctime(&mt));
   for(int t=0; str[t]; t++)
      if(str[t] == '\n' || str[t] == '\r')
         str[t] = '\0';
   mtime_v->setText(str);
   free(str);

   str = strdup(ctime(&at));
   for(int t=0; str[t]; t++)
      if(str[t] == '\n' || str[t] == '\r')
         str[t] = '\0';
   atime_v->setText(str);
   free(str);

   if(files > 1){
      user_r->setTristate(TRUE);
      user_w->setTristate(TRUE);
      user_x->setTristate(TRUE);
      group_r->setTristate(TRUE);
      group_w->setTristate(TRUE);
      group_x->setTristate(TRUE);
      other_r->setTristate(TRUE);
      other_w->setTristate(TRUE);
      other_x->setTristate(TRUE);
   }
   else{
      user_r->setTristate(FALSE);
      user_w->setTristate(FALSE);
      user_x->setTristate(FALSE);
      group_r->setTristate(FALSE);
      group_w->setTristate(FALSE);
      group_x->setTristate(FALSE);
      other_r->setTristate(FALSE);
      other_w->setTristate(FALSE);
      other_x->setTristate(FALSE);
   }

   if((S_IRUSR) & (min_mode & max_mode))
      user_r->setChecked(1);
   else if((S_IRUSR) & (min_mode ^ max_mode))
      user_r->setNoChange();
   else
      user_r->setChecked(0);

   if((S_IWUSR) & (min_mode & max_mode))
      user_w->setChecked(1);
   else if((S_IWUSR) & (min_mode ^ max_mode))
      user_w->setNoChange();
   else
      user_w->setChecked(0);

   if((S_IXUSR) & (min_mode & max_mode))
      user_x->setChecked(1);
   else if((S_IXUSR) & (min_mode ^ max_mode))
      user_x->setNoChange();
   else
      user_x->setChecked(0);

   if((S_ISUID) & (min_mode & max_mode))
      sticky_u->setChecked(1);
   else if((S_ISUID) & (min_mode ^ max_mode))
      sticky_u->setNoChange();
   else
      sticky_u->setChecked(0);

   if((S_IRGRP) & (min_mode & max_mode))
      group_r->setChecked(1);
   else if((S_IRGRP) & (min_mode ^ max_mode))
      group_r->setNoChange();
   else
      group_r->setChecked(0);

   if((S_IWGRP) & (min_mode & max_mode))
      group_w->setChecked(1);
   else if((S_IWGRP) & (min_mode ^ max_mode))
      group_w->setNoChange();
   else
      group_w->setChecked(0);

   if((S_IXGRP) & (min_mode & max_mode))
      group_x->setChecked(1);
   else if((S_IXGRP) & (min_mode ^ max_mode))
      group_x->setNoChange();
   else
      group_x->setChecked(0);

   if((S_ISGID) & (min_mode & max_mode))
      sticky_g->setChecked(1);
   else if((S_ISGID) & (min_mode ^ max_mode))
      sticky_g->setNoChange();
   else
      sticky_g->setChecked(0);

   if((S_IROTH) & (min_mode & max_mode))
      other_r->setChecked(1);
   else if((S_IROTH) & (min_mode ^ max_mode))
      other_r->setNoChange();
   else
      other_r->setChecked(0);

   if((S_IWOTH) & (min_mode & max_mode))
      other_w->setChecked(1);
   else if((S_IWOTH) & (min_mode ^ max_mode))
      other_w->setNoChange();
   else
      other_w->setChecked(0);

   if((S_IXOTH) & (min_mode & max_mode))
      other_x->setChecked(1);
   else if((S_IXOTH) & (min_mode ^ max_mode))
      other_x->setNoChange();
   else
      other_x->setChecked(0);

   if((S_ISVTX) & (min_mode & max_mode))
      sticky_o->setChecked(1);
   else if((S_ISVTX) & (min_mode ^ max_mode))
      sticky_o->setNoChange();
   else
      sticky_o->setChecked(0);

   if ( m_errorList.count() )
   {
      QString error = "The following errors occured:\n";

      for( QStringList::Iterator it = m_errorList.begin(); it != m_errorList.end(); it++ )
      {
         error += (*it) + tr("\n");
      }

      QMessageBox::warning( NULL, "Bronco", error);
   }

   QWidget::show();
}

//int PropWin::readDir(const char *path, unsigned long *f, off_t *s){
int PropWin::readDir(const char *path){
   char buffer[4096];
   DIR *dp;
   struct dirent *d;
   
   if((dp = opendir(path)) != NULL){
      while((d=readdir(dp)) != NULL){
         if(strcmp(d->d_name, ".") && strcmp(d->d_name, "..")){
            snprintf(buffer, sizeof(buffer), "%s/%s", path, d->d_name);
            addFile(buffer);
         }
      }
      closedir(dp);
      return(1);
   }
   else{
      return(0);
   }
}

int PropWin::addFile(const char *filename){
   struct stat statbuf;
   if(lstat(filename, &statbuf) < 0){
      bool found = false;
      QString error = strerror(errno);
      for ( QStringList::Iterator it = m_errorList.begin(); it != m_errorList.end(); it++ )
      {
         if ( *it == tr(error) )
         {
            found = true;
            break;
         }
      }

      if ( found == false )
      {
         m_errorList << error;
      }
   }
   return(addFile(filename, &statbuf));
}

int PropWin::addFile(const char *filename, const struct stat *statbuf){
   size   += statbuf->st_size;
   blocks += statbuf->st_blocks;
   bsize  += (BLK_SIZE * statbuf->st_blocks);
   uid     = statbuf->st_uid;
   gid     = statbuf->st_gid;
   files  += 1;

   if(statbuf->st_mtime > mt)
      mt      = statbuf->st_mtime;
   if(statbuf->st_atime > at)
      at      = statbuf->st_atime;
   if(statbuf->st_ctime > ct)
      ct      = statbuf->st_ctime;

   if(files > 1){
      max_mode |= statbuf->st_mode;
      min_mode &= statbuf->st_mode;
   }
   else{
      max_mode  = statbuf->st_mode;
      min_mode  = statbuf->st_mode;
   }

   if(fname[0] == '\0')
      strcpy(fname, filename);
   else
      strcpy(fname, "Selected files");

   if(S_ISDIR(statbuf->st_mode)){
//      readDir(filename, &files, &size);
      readDir(filename);
   }

   return(0);
}

void PropWin::clear(){
   m_errorList.clear();
   fname[0] = '\0';
   files    = 0;
   size     = 0;
   blocks   = 0;
   bsize    = 0;
   mt       = 0;
   at       = 0;
   ct       = 0;
   min_mode = 0;
   max_mode = 0;
   uid      = 0;
   gid      = 0;
}

