/* propwin.h - Properties dialog window
 * 
 *  Bronco File Manager
 * 
 *  Copyright (c) 2000, 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#ifndef PROPWINH
#define PROPWINH

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <time.h>
#include <qwidget.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qcheckbox.h>

class PropWin : public QWidget {
      Q_OBJECT

   public:
      PropWin();

      int addFile(const char *);
      int addFile(const char *, const struct stat *statbuf);
      void clear(void);

   public slots:
      void show(void);

   protected:
      //   int readDir(const char *, long unsigned *, off_t *);
      int readDir(const char *);

      char fname[1024];
      unsigned long files;
      unsigned long blocks;
      unsigned long long size;
      unsigned long long bsize;
      time_t mt;
      time_t at;
      time_t ct;
      mode_t max_mode;
      mode_t min_mode;
      uid_t uid;
      gid_t gid;

      QStringList m_errorList;

      QGridLayout *grid;

      QLabel *fname_l;
      QLabel *files_l;
      QLabel *size_l;
      QLabel *blocks_l;
      QLabel *bsize_l;
      QLabel *mtime_l;
      QLabel *atime_l;
      QLabel *ctime_l;
      QLabel *mode_l;

      QLabel *user_l;
      QLabel *other_l;
      QLabel *group_l;

      QLabel *read_l;
      QLabel *write_l;
      QLabel *exec_l;
      QLabel *sticky_l;

      QLabel *fname_v;
      QLabel *files_v;
      QLabel *size_v;
      QLabel *blocks_v;
      QLabel *bsize_v;
      QLabel *mtime_v;
      QLabel *atime_v;
      QLabel *ctime_v;
      QLabel *mode_v;

      QCheckBox *user_r;
      QCheckBox *user_w;
      QCheckBox *user_x;
      QCheckBox *group_r;
      QCheckBox *group_w;
      QCheckBox *group_x;
      QCheckBox *other_r;
      QCheckBox *other_w;
      QCheckBox *other_x;
      QCheckBox *sticky_u;
      QCheckBox *sticky_g;
      QCheckBox *sticky_o;

      QPushButton *close;
};

#endif // PROPWINH
