/* tardlg.cpp - Tar Creation Dialog
 * 
 *  Bronco File Manager
 * 
 *  Copyright (c) 2000, 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#include "bronco.h"
#include "tardlg.h"

#include <qcombobox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <stdio.h>

TarDlg::TarDlg( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
   if ( !name )
      setName( "TarDlg" );

   resize( 425, 130 ); 
   setCaption( tr( "Create Archive" ) );
   TarDlgLayout = new QVBoxLayout( this ); 
   TarDlgLayout->setSpacing( 6 );
   TarDlgLayout->setMargin( 11 );

   Layout1 = new QHBoxLayout; 
   Layout1->setSpacing( 6 );
   Layout1->setMargin( 0 );

   filename_l = new QLabel( this, "filename_l" );
   filename_l->setText( tr( "Filename:" ) );
   Layout1->addWidget( filename_l );

   filename_v = new QLineEdit( this, "filename_v" );
   Layout1->addWidget( filename_v );
   TarDlgLayout->addLayout( Layout1 );

   Layout4 = new QHBoxLayout; 
   Layout4->setSpacing( 6 );
   Layout4->setMargin( 0 );

   compression_l = new QLabel( this, "compression_l" );
   compression_l->setText( tr( "Compression:" ) );
   Layout4->addWidget( compression_l );

   compression_v = new QComboBox( FALSE, this, "compression_v" );
   compression_v->insertItem( tr( "gzip" ), TarGzip );
   compression_v->insertItem( tr( "bzip2" ), TarBzip2 );
   compression_v->insertItem( tr( "compress" ), TarCompress );
   compression_v->insertItem( tr( "none" ), TarNone );
   compression_v->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)0, compression_v->sizePolicy().hasHeightForWidth() ) );
   Layout4->addWidget( compression_v );
   TarDlgLayout->addLayout( Layout4 );

   Layout3 = new QHBoxLayout; 
   Layout3->setSpacing( 6 );
   Layout3->setMargin( 0 );
   QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
   Layout3->addItem( spacer );

   create_b = new QPushButton( this, "create_b" );
   create_b->setText( tr( "Create" ) );
   create_b->setDefault( TRUE );
   Layout3->addWidget( create_b );

   cancel_b = new QPushButton( this, "cancel_b" );
   cancel_b->setText( tr( "Cancel" ) );
   Layout3->addWidget( cancel_b );
   TarDlgLayout->addLayout( Layout3 );

   // signals and slots connections
   connect( create_b, SIGNAL( clicked() ), this, SLOT( accept() ) );
   connect( compression_v, SIGNAL( activated( int ) ), this, SLOT( compressionChanged( int ) ) );
   connect( cancel_b, SIGNAL( clicked() ), this, SLOT( reject() ) );
}

/*  
 *  Destroys the object and frees any allocated resources
 */
TarDlg::~TarDlg()
{
   // no need to delete child widgets, Qt does it all for us
}

void TarDlg::compressionChanged(int s)
{
   QString temp = filename_v->text();

   if( temp.right(4) == tr(".tar")
         || temp.right(6) == tr(".tar.Z")
         || temp.right(7) == tr(".tar.gz")
         || temp.right(8) == tr(".tar.bz2") )
   {
      temp.truncate( temp.findRev(".tar"));

      switch ( s )
      {
         case 0:
            temp += ".tar.gz";
            break;
         case 1:
            temp += ".tar.bz2";
            break;
         case 2:
            temp += ".tar.Z";
            break;
         case 3:
            temp += ".tar";
            break;
         default:
            break;
      }

      filename_v->setText( temp );
   }
}

void TarDlg::show()
{
   setCompression(prefs.getTarCompression() );
   compressionChanged( (int) prefs.getTarCompression() );
   QDialog::show();
}


void TarDlg::setFilename( const QString &s )
{
   filename_v->setText( s );
}

bool TarDlg::setCompression ( const QString &s )
{
   for ( int t = 0; t < compression_v->count(); t++ )
   {
      if ( s == compression_v->text( t ) )
      {
         compression_v->setCurrentItem( t );
         return ( true );
      }
   }

   return ( false );
}

bool TarDlg::setCompression ( TarCompression s )
{
   compression_v->setCurrentItem( s );
   return ( true );
}

const QString TarDlg::getFilename(void)
{
   return filename_v->text();
}

TarCompression TarDlg::getCompression(void)
{
   return (TarCompression) compression_v->currentItem();
}
