/* tarwin.cpp - Window for view tar archives
 * 
 *  Bronco File Manager
 * 
 *  Copyright (c) 2000, 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#include <qaccel.h>
#include <qlineedit.h>
#include <qlayout.h>
#include "bronco.h"
#include "dirlist.h"
#include "find.h"
#include "prefs.h"
#include "execbr.h"
#include "tarwin.h"

int isTarFile(const char *s)
{
   int len = strlen(s);
   
   if(len > 4 && strncmp(&s[len-4], ".tar", 4) == 0)   
      return(1);
   if(len > 4 && strncmp(&s[len-4], ".tgz", 4) == 0)   
      return(1);
   if(len > 6 && strncmp(&s[len-6], ".tar.Z", 6) == 0)   
      return(1);
   if(len > 7 && strncmp(&s[len-7], ".tar.gz", 7) == 0)   
      return(1);
   if(len > 8 && strncmp(&s[len-7], ".tar.bz", 7) == 0)   
      return(1);
   if(len > 8 && strncmp(&s[len-8], ".tar.bz2", 8) == 0)   
      return(1);

   return(0);
}

int inCurrentDir(const char *s)
{
   int n;
   int len = strlen(s);
   for(n=0; n<len; n++)
   {
      if(s[n] == '/')
      {
         return(0);
      }
   }
   return(1);
}

int isTarDir(char *s)
{
   int n;
   int len = strlen(s);
   for(n=0; n<len; n++)
   {
      if(s[n] == '/')
      {
         s[n] = '\0';
         if(s[n+1] == '\0')
            return(1);
         else
            return(0);
      }
   }
   return(0);
}

TarWin::TarWin(QString f, QWidget *parent = NULL, const char *name = "") : QWidget(parent, name, WDestructiveClose){
   QAccel *accel;

   file = f;

   setCaption( tr("View Archive: ") + file.right( file.length() - (file.findRev('/') + 1) ) );

   if ( ! name )
      setName( "TarWin" );

   resize( 570, 400 ); 

   TarWinLayout = new QVBoxLayout( this ); 
   TarWinLayout->setSpacing( 6 );
   TarWinLayout->setMargin( 11 );

   Layout1 = new QHBoxLayout; 
   Layout1->setSpacing( 6 );
   Layout1->setMargin( 0 );

   where_l = new QLabel( this, "where_l" );
   where_l->setText( tr( "Extract to" ) );
   Layout1->addWidget( where_l );

   where_v = new QLineEdit( this, "where_v" );
   Layout1->addWidget( where_v );

   untar_b = new QPushButton( this, "untar_b" );
   untar_b->setText( tr( "E&xtract" ) );
   Layout1->addWidget( untar_b );
   TarWinLayout->addLayout( Layout1 );

   Layout8 = new QHBoxLayout; 
   Layout8->setSpacing( 6 );
   Layout8->setMargin( 0 );

   Layout7 = new QVBoxLayout; 
   Layout7->setSpacing( 6 );
   Layout7->setMargin( 0 );

   overwrite_v = new QCheckBox( this, "overwrite_v" );
   overwrite_v->setText( tr( "&Overwrite Files" ) );
   Layout7->addWidget( overwrite_v );

//   permission_v = new QCheckBox( this, "permission_v" );
//   permission_v->setText( tr( "Extract &Permissions" ) );

//   Layout7->addWidget( permission_v );
   modification_v = new QCheckBox( this, "modification_v" );
   modification_v->setText( tr( "Extract &Modification Time" ) );

   Layout7->addWidget( modification_v );
   Layout8->addLayout( Layout7 );

   Layout6 = new QVBoxLayout; 
   Layout6->setSpacing( 6 );
   Layout6->setMargin( 0 );

   selected_v = new QCheckBox( this, "selected_v" );
   selected_v->setText( tr( "Only Extract &Selected Files" ) );
//   selected_v->setEnabled(false);
   Layout6->addWidget( selected_v );

   Layout8->addLayout( Layout6 );
   TarWinLayout->addLayout( Layout8 );

   list = new DirList(this, "");
   list->setSorting(-1, FALSE);

   TarWinLayout->addWidget( list );
   /* Initilization */

   connect(untar_b, SIGNAL(clicked()),       this, SLOT(doUntar()));
   connect(where_v, SIGNAL(returnPressed()), this, SLOT(doUntar()));

   where_v->setFocus();

   accel = new QAccel(this);

   if(parent == NULL) 
      accel->connectItem(accel->insertItem(Key_W+CTRL), this, SLOT(close()));

   QString path = file;
   path.truncate(path.findRev('/'));
   path = path.right(path.length() - 4);
   where_v->setText(path);

   file.append(":");
   list->loadDir(file);

   overwrite_v->setChecked( prefs.getTarOverwrite() );
   modification_v->setChecked( prefs.getTarModification() );
   show();
}

TarWin::~TarWin(){
}

void TarWin::resizeEvent(QResizeEvent *e){
   list->resize(this->width() - 10, this->height() - 60);

}

void TarWin::doUntar(void){
   char *args[MAXARGS];

   QString cmd;
   QString opts;

   QString filename = file;
   QString path     = list->getDir();

   filename.truncate(filename.findRev(':'));
   filename = filename.right(filename.length() - 4);

   int len = path.length() - (path.findRev(':') + 2);
   if ( len > 0 )
      path = path.right( len );
   else
      path = "";

   /* Assume tar.gz by default */
   opts = "-xz";

   /* Change extract method based on filename extension */
   if(filename.right(4) == QString(".tar"))
      opts = "-x";
   if(filename.right(6) == QString(".tar.Z"))
      opts = "-xZ";
   if(filename.right(8) == QString(".tar.bz2"))
      opts = "-xj";

   /* -m = don't extract modified time */
   if ( ! modification_v->isChecked() )
   {
      opts += "m";
   }

   /* -k = keep old files */
   if ( ! overwrite_v->isChecked() )
   {
      opts += "k";
   }
   else
   {
      opts += " --overwrite";
   }

   opts += " -f ";
   
   cmd = tr("tar -C ") + where_v->text() + tr(" ") + opts;

   /* Filename is first argument */
   int arg_count = 0;
   args[arg_count++] = strdup(filename.latin1());

   if ( selected_v->isChecked() )
   {
      int count = 0;
      QListViewItem *item = list->firstChild();

      while ( item )
      {
         if ( item->isSelected() )
         {
            QString file;
            if ( path.length() )
               file = path + QString("/") + item->text(1);
            else
               file = item->text(1);

            args[arg_count++] = strdup( file.latin1() );

            count++;
         }

         item = item->nextSibling();
      }


      if ( count == 0)
      {
         QMessageBox::warning( NULL, "Bronco", "No files selected", QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape, 0);
         return;
      }
   }

   args[arg_count] = NULL;

   BroncoExec program;

   if( program.exec(cmd.latin1(), (const char **) args) == 0 )
      QMessageBox::information( NULL, "Bronco",
                         "Archive extracted", QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape, 0);
   else
      QMessageBox::warning( NULL, "Bronco",
                         strerror(errno), QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape, 0);

   for ( int n = 0; args[n]; n++ )
   {
      free(args[n]);
   }

   this->close();
   return;
}

